@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -81,17 +81,17 @@ public:
     const double percent;       ///<The position of the label in percentage of the length
     const double offset;        ///<The position of the label as additional pixel offset
     bool is_extension = false;
-    /** Returns true if we shall start measuring from the start of the arrow based on 'percent', 
+    /** Returns true if we shall start measuring from the start of the arrow based on 'percent',
      * 'offset' and 'is_extension'. Else we measure from the end.*/
     bool IsFromStart() const { return (percent ? percent<0 : offset<0) == is_extension; }
     /** @} */
     bool has_marker = false;   ///<True if we shall draw a mid-arrowhead at this point
     mutable PathPos pos;       ///<The position of the marker
-    mutable XY point;          ///<The position of the marker  
+    mutable XY point;          ///<The position of the marker
     static std::optional<std::pair<double, double>>
-        ConvertPosition(BlockChart &chart, 
-                        gsl::owner<char *>num1, bool is_perc1, const FileLineColRange &l1,
-                        gsl::owner<char *>num2 = nullptr, bool is_perc2 = false, const FileLineColRange &l2 = FileLineColRange());
+        ConvertPosition(BlockChart &chart,
+                        std::string_view num1, bool is_perc1, const FileLineColRange &l1,
+                        std::string_view num2 = {}, bool is_perc2 = false, const FileLineColRange& l2 = FileLineColRange());
     ArrowLabel(BlockChart &chart, double perc, double offset, const FileLineColRange &l);
     ArrowLabel(const ArrowLabel &) = default;
     ArrowLabel(ArrowLabel &&) = default;
@@ -101,7 +101,8 @@ public:
     /** Add a list of possible attribute value names for block type `t` to `csh` for attribute `attr`.*/
     static bool AttributeValues(std::string_view /*attr*/, Csh &/*csh*/);
     BlockInstruction *Clone(const FileLineColRange &l, std::string_view prefix,
-                            gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const override;
+                            gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                            const MultiBlock* multi_parent) const override;
     void ApplyRunningStyle(bool cloning, bool recursive) override;
     Contour LayoutFinalize(Canvas &canvas, const LineAttr &line, const Path &path, double len = -1);
     void PostPosProcess(Canvas &, Chart *) override;
@@ -159,13 +160,13 @@ public:
     ArrowLabelList        labels;    ///<Our labels.
     size_t                my_index = 0; ///<Every arrow inserted into BlockCharts::Arrows in PostParseProcess() gets a unique index. Used to break ties of file_pos (coming from A->B,C where the file_pos is the same for the 2 arrows).
     std::array<std::map<Arrow*, Overlap>, 2>
-                          overlaps;  ///<Arrows we overlap with in the order of definition. We ourselves are in this list. 
+                          overlaps;  ///<Arrows we overlap with in the order of definition. We ourselves are in this list.
     std::array<bool, 2>   adjusted;  ///<If our end has already been adjusted during de-overlapping.
     bool                  redo;      ///<True after arrow re-overlap we need to recalculate layout
     std::array<bool, 2>   linearextend; ///<If true and we start/end at a block and our endpoint is not inside the block, we linearextend us to hit the block
     double                route_order = 0; ///<Specifies the re-layout order after de-overlapping (user can specify floating point numbers, so we allow)
 
-    
+
     mutable Path path;          ///< The selected path of the arrow.
     mutable Path path_straight; ///< The selected path of the arrow before made curvy and before trimming
     mutable Path path_visible;  ///< The actually visible selected path of the arrow on which we measure label pos.
@@ -182,15 +183,16 @@ public:
     bool AmIContent() const override;
     void AddLabels(const BlockInstrList *l);
     BlockInstruction *Clone(const FileLineColRange &l, std::string_view prefix,
-                            gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const override;
+                            gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                            const MultiBlock* multi_parent) const override;
     void ApplyRunningStyle(bool cloning, bool recursive) override;
-    bool PostParseProcess(Canvas &canvas, bool hide, Numbering &number) override;
+    bool PostParseProcess(Canvas &canvas, const BlockBlock* hide_by, Numbering &number) override;
     /** Allows ordering of Arrow objects by source location.
      * my_index is used to disambiguite A->B,C arrows, which have equivalent file_pos.*/
     bool earlier(const Arrow &o) const noexcept { return std::tie(file_pos, my_index)<std::tie(o.file_pos, o.my_index); }
     void FinalizeLabel(Canvas &canvas) override;
     AddConstraintResult
-        AddConstraints(Canvas & canvas, ConstraintSet &constraints, unsigned from, unsigned to, 
+        AddConstraints(Canvas & canvas, ConstraintSet &constraints, unsigned from, unsigned to,
                        unsigned parent_index_justify[2], const InternalAlignment *parent_attr_justify[2]) const override;
     unsigned CheckSegmentRoutability(std::array<std::optional<Contour>, 16> &C, unsigned index,
                                      const XY &p1, const XY &p2,
