@@ -1111,17 +1111,14 @@ Overlaps(iter1 first1, iter1 last1,
     XY xy[9];
     double my_pos[9], other_pos[9];
     const iter2 first2_save = first2;
-    bool overlap;
     for (unsigned u = 0; first1!=last1; ++u, ++first1)
         if (!visible_only || first1->IsVisible()) {
             first2 = first2_save;
             for (unsigned v = 0; first2!=last2; ++v, ++first2)
                 if (!visible_only || first2->IsVisible()) {
-                    const int num = first1->Crossing(*first2, false, xy, my_pos, other_pos, &overlap);
-                    if (!overlap) continue;
-                    if (num!=2) {
-                        _ASSERT(num==2);
-                    }
+                    const int num = first1->CrossingWithOverlap(*first2, false, xy, my_pos, other_pos); //num<0 if the two edges overlap
+                    if (num>=0) continue;
+                    _ASSERT(num==-2);
                     //store so that we increase in me first->second
                     if (my_pos[0]<my_pos[1])
                         ret.push_back({ { xy[0], u, my_pos[0], v, other_pos[0], {} },
