@@ -135,9 +135,9 @@ public:
      * we don't actually progress layout by blocks, but by constraints.*/
     static constexpr unsigned block_layout_bulk_per_block = 1000;
     //Names should be declared first, so at destruction, all blocks are destroyed before them.
-    BlockNameCatalog       BlockNames_Unique;  ///<Contains all the unique block names assigned by the user mapping to the block object
-    ReverseBNameMultiMap   BlockNames_Full;    ///<Contains all the full block names assigned by the user mapping to the block object
-    MultiBlockNameCatalog  BlockNames_Original;///<Contains all the original block names assigned by the user mapping to the block object
+    BlockNameCatalog       BlockNames_Unique;  ///<Contains all the unique block names assigned by the user mapping to the block object. Unique names are never empty.
+    ReverseBNameMultiMap   BlockNames_Full;    ///<Contains all the full block names assigned by the user mapping to the block object. May contain the empty name.
+    MultiBlockNameCatalog  BlockNames_Original;///<Contains all the original block names assigned by the user mapping to the block object. Does not contain the empty name.
     std::vector<BlockElement*> DrawOrder;      ///<A reference to elements in the order they need to be drawn.
     GUIState               GUI_State;          ///<Collapse/expand info
 
@@ -150,7 +150,7 @@ public:
 	static constexpr double arrow_scale = 0.5;          ///<How much we scale arrowheads compared to the default (the size for signalling charts is such that 'small' is the acceptable default. Here we scale down and take 'normal' as default.)
 	static constexpr double arrow_ambigous_percent = 20;///<How much going around a block may be longer than the other way to skip considering the alternative
     static constexpr double def_distance = 5;           ///<How much shall arrows avoid blocks.
-    static constexpr double def_distance_manhattan = 7; ///<How much shall arrows avoid blocks for manhattan routed arrows. It is more than 5, which would make normal sized arrows look bad on a sharp turn.
+    static constexpr double def_distance_manhattan = 7; ///<How much shall arrows avoid blocks for Manhattan routed arrows. It is more than 5, which would make normal sized arrows look bad on a sharp turn.
 	static constexpr double def_distance_cp = 10;       ///<How much shall arrows avoid start/stop blocks if a compass point is specified
 	static constexpr double def_margin = 10;            ///<The default external margin
 	static constexpr double def_imargin = 10;           ///<The default internal margin
