@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -26,17 +26,170 @@
 #include "blockstyle.h"
 #include "blockchart.h"
 #include "utf8utils.h"
+#include "block_parser_csh.h"
 
 using namespace block;
 
-BlockCsh::BlockCsh(Csh::FileListProc proc, void *param) :
-    Csh(BlockContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc, param)
+BlockCsh::BlockCsh(Csh::FileListProc proc) :
+    Csh(BlockContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc)
 {
     ForbiddenStyles.erase("row");
     ForbiddenStyles.erase("col");
     FillNamesHints();
 }
 
+void BlockCsh::CreateBlocksFromLast(const CshStringWithPosList* l) {
+    if (!l || l->size()<2) return;
+    //Here we assume that for the last element of the list we already have created names in 'blocks'.
+    //We ignore names with '.' in them (those come from a multi command).
+    //1. Find the 'last' such element.
+    const auto last = std::find_if(l->rbegin(), l->rend(), [](const CshStringWithPos& sp) { return sp.name.find('.')==std::string::npos; });
+    if (last != l->rend()) {
+        const std::string prefix = PrefixName(last->name)+".";
+        //Now iterate backwards
+        for (auto i = std::next(last); i!=l->rend(); i++)
+            if (i->name.find('.')==std::string::npos) //only names without a dot
+                //Search Blocks for names starting with 'prefix'
+                for (auto& [name, shape] : Blocks)
+                    if (name.starts_with(prefix)) {
+                        auto [itr, inserted] = Blocks.insert({ PrefixName(i->name + name.substr(prefix.size()-1)), shape });
+                        if (!inserted)
+                            itr->second = -2;
+                    }
+    }
+    //Now create a block for all names having a dot.
+    for (const CshStringWithPos& sp : *l)
+        if (size_t pos = sp.name.find('.'); pos!=std::string::npos) {
+            const auto i = Blocks.find(PrefixName(sp.name.substr(0, pos)));
+            const int shape = i==Blocks.end() ? -1 : i->second;
+            auto [itr, inserted] = Blocks.insert({ PrefixName(sp.name), shape });
+            if (!inserted)
+                itr->second = -2;
+            //These names should come from a multi
+            [[maybe_unused]] static const char* names[] = { "front", "back", "2", "3", "4", "5", "6", "7", "8", "9" };
+            _ASSERT(std::find(std::begin(names), std::end(names), sp.name.substr(pos+1))!=std::end(names));
+        }
+}
+
+void BlockCsh::DeleteBlock(std::string_view name) {
+    auto i = Blocks.find(PrefixName(name));
+    if (i!=Blocks.end() && i->second>=-1) //if the shape is valid this Block is mentioned only once, so we can delete it.
+        Blocks.erase(i);
+}
+
+void BlockCsh::Templatize(std::string_view name, const CshPos &p) {
+    std::string pref = PrefixName(name);
+    auto i = Blocks.find(pref);
+    if (i==Blocks.end()) { _ASSERT(0); return; }
+    auto& [bl, sh] = Templates[pref];
+    _ASSERT(bl.empty());
+    sh = i->second;
+    Blocks.erase(i);
+    pref.push_back('.');
+    for (auto i = Blocks.begin(); i!=Blocks.end(); /*nope*/)
+        if (!i->first.starts_with(pref)) { i++; continue; }
+        else {
+            bl.insert({ i->first.substr(pref.size()), i->second });
+            if (i->second<-1) i++;     //This name is used multiple times, keep it in Blocks
+            else i = Blocks.erase(i);  //This name is used only once, so move it to the template
+        }
+    for (CshEntry& e : CshList) {
+        if (e.IsWithin(p) && e.color==COLOR_ENTITYNAME_FIRST)
+            e.color = COLOR_MARKERNAME;
+        if (e==p) break;
+    }
+}
+
+void BlockCsh::Templatize(gsl::owner<CshStringWithPosList*> l) {
+    if (!l) return;
+    for (const CshStringWithPos& sp : *l)
+        if (sp.name.find('.')==std::string::npos)
+            Templatize(sp.name, sp.file_pos);
+    delete l;
+}
+
+bool BlockCsh::Copy(std::string_view from, std::string_view to, const CshPos& from_pos, const CshPos& to_pos) {
+    //drop any prefix from 'to'
+    if (const size_t pos = to.find_last_of('.'); pos != to.npos)
+        to.remove_prefix(pos + 1);
+    const auto t = SearchName(Templates, from, false, false);
+    switch (t.first) {
+    case -2: //partially typed template name
+        break; //try finding a fully matching block name
+    case -3:
+        AddCSH_Error(from_pos, "More than one template of this name found.");
+        return true;
+    case -4:
+        break;
+    default:
+        _ASSERT(0);
+        break;
+    case -1:
+    instantiate_template:
+        AddCSH_NewBlock(to_pos, to, /*shape=*/t.second->second.second);
+        std::string full_to = PrefixName(to);
+        full_to.push_back('.');
+        for (auto& [bn, sh] : t.second->second.first) {
+            auto [itr, inserted] = Blocks.insert({ full_to+bn, sh});
+            if (!inserted) itr->second = -2;
+        }
+        if (from_pos != to_pos) {
+            AddCSH(from_pos, t.first == -1 ? COLOR_MARKERNAME : COLOR_MARKERNAME_PARTIAL);
+            if (t.first==-2)
+                AddCSH_Error(from_pos, "No template of this name found, maybe '"+t.second->first+"'");
+        }
+        return false;
+    }
+    const auto b = SearchName(Blocks, from, false, true);
+    switch (b.first) {
+    case -3:
+        AddCSH_Error(from_pos, "More than one block of this name found.");
+        return true;
+    default:
+        _ASSERT(0); FALLTHROUGH;
+    case -4:
+        AddCSH_Error(from_pos, "No block of this name found.");
+        return true;
+    case -2: //partially typed block name
+        if (t.first == -2) goto instantiate_template;
+        FALLTHROUGH;
+    case -1:
+        //Copy a block
+        AddCSH_NewBlock(to_pos, to, /*shape=*/b.second->second);
+        BlockList add;
+        std::string full_from = PrefixName(from);
+        std::string full_to = PrefixName(to);
+        full_from.push_back('.');
+        full_to.push_back('.');
+        for (auto& [bn, sh] : Blocks)
+            if (bn.starts_with(full_from)) {
+                auto [itr, inserted] = add.insert({ full_to + bn.substr(full_from.size()), sh });
+                if (!inserted) itr->second = -2;
+            }
+        Blocks.merge(add); //elements of 'add' already in 'Blocks' remain in 'add'
+        for (auto& [bl, _] : add) Blocks[bl] = -2;
+        if (from_pos != to_pos) {
+            AddCSH(from_pos, b.first == -1 ? COLOR_ENTITYNAME : COLOR_ENTITYNAME_PARTIAL);
+            if (b.first == -2)
+                AddCSH_Error(from_pos, "No block of this name found, maybe '" + b.second->first + "'");
+        }
+        return false;
+    }
+}
+
+gsl::owner<CshStringWithPosList*> BlockCsh::MakeMulti(int num, gsl::owner<CshStringWithPosList*> l) {
+    if (!l || l->empty()) return l;
+    const size_t orig = l->size();
+    for (size_t u = 0; u<orig; u++) {
+        l->emplace_back(l->at(u).name+".front", l->at(u).file_pos);
+        l->emplace_back(l->at(u).name+".back", l->at(u).file_pos);
+        for (int n = 2; n<num; n++)
+            l->emplace_back(l->at(u).name+"."+ std::to_string(n), l->at(u).file_pos);
+    }
+    return l;
+}
+
+
 void BlockCsh::FillNamesHints()
 {
     BlockChart::AttributeNames(*this, false);
@@ -232,7 +385,7 @@ bool CshHintGraphicCallbackForBoxPorts(Canvas *canvas, CshHintGraphicParam p, Cs
 }
 
 
-/** Port names for a shape, -1 being the box, smaller numbers dont add anything.*/
+/** Port names for a shape, -1 being the box, smaller numbers don't add anything.*/
 void BlockCsh::AddPortsToHints(int shape)
 {
     if (shape==-1) {
@@ -262,10 +415,10 @@ void BlockCsh::AddWhatToUpdateToHints(bool include_keywords)
                            "Update attributes of all elements recursively.",
                            EHintType::KEYWORD));
     }
-    //AddEntityNamesAtTheEnd("Update attributes of block %s.");
-    AddToHints(CshHint(HintPrefixNonSelectable() + "<blockname>",
-                       "Update attributes of a single named block contained in the copied block.",
-                       EHintType::KEYWORD, false));
+    std::string n = GetCurrentPrefix()+".";
+    for (auto i = Blocks.lower_bound(n); i!=Blocks.end() && i->first.starts_with(n); i++)
+        AddToHints(CshHint(StrCat(HintPrefix(COLOR_ENTITYNAME), std::string_view(i->first).substr(n.size())),
+                            "A named block contained in the copied block.", EHintType::ENTITY));
     hintStatus = HINT_READY;
 
 }
@@ -281,9 +434,9 @@ void BlockCsh::AddOptionsToHints()
 }
 
 
-void BlockCsh::BeforeYaccParse(const std::string& input, int cursor_p)
+void BlockCsh::BeforeYaccParse(std::string&& input, int cursor_p)
 {
-    Csh::BeforeYaccParse(input, cursor_p);
+    Csh::BeforeYaccParse(std::move(input), cursor_p);
     parent_name.clear();
     prefix_at_hint_loc.clear();
     Context2.clear();
@@ -302,13 +455,14 @@ void BlockCsh::BeforeYaccParse(const std::string& input, int cursor_p)
  * @param [in] input The chart text
  * @param [in] cursor_p The current position of the cursor.
  * @param [in] pedantic The initial value of the pedantic chart option.*/
-void BlockCsh::ParseText(const std::string& input, int cursor_p, bool pedantic)
+void BlockCsh::ParseText(std::string&& input, int cursor_p, bool pedantic)
 {
     //initialize data struct
-    BeforeYaccParse(input, cursor_p);
+    BeforeYaccParse(std::move(input), cursor_p);
     Context2.back().pedantic = pedantic;
     //call parsing
-    BlockCshParse(*this, input_text.c_str(), unsigned(input_text.length()+1)); //include terminating zero
+    sv_reader<true> reader(input_text, 0);
+    block_csh_parse(*this, reader); //return value ignored
     //Tidy up afterwards
     AfterYaccParse();
 }
@@ -432,22 +586,22 @@ bool BlockCsh::AddCSH_NewBlock(const CshPos &pos, std::string_view pname, int sh
 {
     if (pname.empty()) {
         parent_name = GetCurrentPrefix();
+        if (pos.first_pos<=pos.last_pos)
+            AddCSH(pos, COLOR_ENTITYNAME); //probably ""
         return true;
     }
     if (shape<0) shape = -1; //box shape
     bool ret = true;
-    parent_name = GetCurrentPrefix().empty() ?
-        std::string(pname) :
-        StrCat(GetCurrentPrefix(), '.', pname);
+    parent_name = PrefixName(pname);
     if (!Contexts.back().SkipContent()) {
-        auto i = Blocks.insert(std::pair<std::string, int>(parent_name, shape));
-        if (!i.second)
-            i.first->second = -2;
+        auto [itr, inserted] = Blocks.insert(std::pair<std::string, int>(parent_name, shape));
+        if (!inserted)
+            itr->second = -2;
         else if (CursorIn(pos)>CURSOR_AFTER)
             //We are inside this particular name block.
             //exclude from providing hints if not yet defined
             exclude_entity_hint = parent_name;
-        ret = i.second;
+        ret = !inserted;
     }
     BlockMentions.emplace_back(pos, parent_name);
     if (ret)
@@ -457,45 +611,34 @@ bool BlockCsh::AddCSH_NewBlock(const CshPos &pos, std::string_view pname, int sh
     return ret;
 }
 
-
-/** Returns >=-1 if this is a valid entity name from the current context (the shape of the block)
- * Returns -2 if this is the prefix of a partial name.
- * Returns -3 if this is a matching name, but multiple blocks match and it is ambigous.
- * Returns -4 if this is not a valid name (or not found).
- * Also returns the full name of the block, if we dont skip content & found a valid unambigous match.*/
-std::pair<int, std::string_view> BlockCsh::SearchEntityName(std::string_view name) const
-{
-    //When recording a procedure, we accept anything as a block name.
-    if (Contexts.back().SkipContent()) return {-1, {}};
-    //In the string we may have a partial prefix. For example, if our parent is A.B.C,
-    //and we have a sibling named 'xyz', typing any of 'A.B.C.xyz', 'B.C.xyz', 'C.xyz' or 'xyz'
-    //may match it. We also have to find any of 'A.B.C.xy', 'B.C.xy', 'C.xy', 'xy' or
-    //'A.B.C.x', 'B.C.x', 'C.x', 'x' as partial matches. Also, 'A.', 'A.B.', etc.
-
-    //We do a linear search
-    std::pair<int, std::string_view> ret = {-4, {}};
-    for (auto &[bname, bshape] : Blocks) {
-        auto pos = bname.rfind(name);
-        if (pos==std::string_view::npos) continue;
-        //if the user omitted a prefix not ending in a dot we are not a valid partial name.
-        //So for bname="AAA.BBB.CCC" and name="BB.CCC" we will not match
-        if (pos>0 && bname[pos-1]!='.') continue;
-        if (pos+name.length()==bname.length()) {
-            //We are a good full match at the end. Check if current prefix works.
-            //That is for bname=AAA.BBB.CCC and name=BBB.CCC we must be withing namespace AAA
-            if (pos==0 || GetCurrentPrefix().substr(0, pos-1)==std::string_view(bname).substr(0, pos-1)) {
-                //If there is a match already existing, mark us as ambigous
-                if (ret.first>=-1) return {-3, {}};
-                //If there were a partial match or no match at all and now we have a full, use it.
-                ret = {bshape, bname};
-            }
-        } else if (bname.find('.', pos+name.length())==std::string::npos) {
-            //no dot in the name after us - A partial match.
-            //If there were no match so far, use it.
-            if (ret.first == -4) ret.first = -2;
-        }
+/** Finds an entity.
+ * @param [in] name The name of the entity as typed by the user. We search namespaces and apply
+ *             our current prefix. For unqualified names (no dot), we also do magic lookup.
+ * @param [in] may_be_new This is a place where we may either reference an existing entity but may
+ *             also define a new entity. If the name contains no dots (magic lookup applies) and
+ *             it does not exist in the current namespace (only magic lookup could yield a hit),
+ *             but it appears multiple times in other namespaces (so that magic lookup would
+ *             yield an ambiguous result), we do not return -3, but -4, to allow defining a new
+ *             entity.
+ * @Returns
+ * - >=-1 if this is a valid entity name from the current context (the shape of the block)
+ * - -2 if this is the prefix of a partial name.
+ * - -3 if this is a matching name, but multiple blocks match and it is ambiguous.
+ * - -4 if this is not a valid name (or not found).
+ *
+ * Also returns the full name of the block, if we don't skip content & found a valid unambiguous match.*/
+std::pair<int, std::string_view> BlockCsh::SearchEntityName(std::string_view string, bool may_be_new) const {
+    const auto r = SearchName(Blocks, string, may_be_new, true);
+    switch (r.first) {
+    case -1: //found
+        if (r.second == Blocks.end()) return { -1, {} };
+        return { r.second->second < -1 ? -3 : r.second->second, r.second->first };
+    case -2:
+    case -3:
+        return { r.second->second, r.second->first };
+    default:
+        return { -4, {} };
     }
-    return ret;
 }
 
 void BlockCsh::AddEntityNamesAtTheEnd(std::string_view msg)
@@ -594,73 +737,76 @@ void BlockCsh::AfterYaccParse()
     //Resolve Block forward references
     for (auto &bm : BlockMentions)
         if (bm.prefix) {
-            std::string_view match; //The full name of the entity we match
-            bool precise_match = false; //True if the match was fully specified and not something we added a prefix for
-            bool ambiguous = false;
-            //bm.name contains the suffix of the full name, bm.prefix contains the prefix we referenced the name from
-            //E.g., if we want to reference a.b.c.d and used bm.name=c.d from bm.prefix=a.b.c, we shall match.
-            //We also match anyone in a 'deeper' namespace than the point of reference, but then only tentatively
-            //E.g., bm.prefix='a', bm.name='c' will also match a.b.c, but only if we dont have 'c' or 'a.c'
-            for (const auto &[bname, btype] : Blocks)
-                if (dot_hierarchy_ends_with(bname, bm.name)) {
-                    //When testing bname='a.b.c.d' we get here for
-                    // 1. bm.prefix='', bm.name='a.b.c.d' (good, this is a match)
-                    // 2. bm.prefix='', bm.name='b.c.d' (bad, this is not a match)
-                    // 3. bm.prefix='a.b', bm.name='c.d' (good)
-                    // 4. bm.prefix='a.b', bm.name='b.c.d' (good)
-                    // 5. bm.prefix='a.b', bm.name='d' (bad)
-                    // 6. bm.prefix='hulu', bm.name='a.b.c.d' (good)
-                    // 7. bm.prefix='a.hulu', bm.name='b.c.d' (good)
-                    //So we first find the common (fully dot separated) part of bm.prefix and bname
-                    const std::string_view common_prefix{bname.data(), dot_hierarchy_common_prefix(bname, *bm.prefix)};
-                    //Then we test if the combined length of this prefix and bm.name  is longer than bname.
-                    //If the common prefix is '', we just test if bm.name is as long as bname.
-                    //Else we add 1 to cater for case #3 above.
-                    if ( (common_prefix.empty() && bname==bm.name)
+            if (bm.name.starts_with('$')) {
+                AddCSH(bm.pos, COLOR_PARAMNAME);
+                bm.name.clear(); //remove this mention - only a param name.
+            } else {
+                std::string_view match; //The full name of the entity we match
+                bool precise_match = false; //True if the match was fully specified and not something we added a prefix for
+                bool ambiguous = false;
+                //bm.name contains the suffix of the full name, bm.prefix contains the prefix we referenced the name from
+                //E.g., if we want to reference a.b.c.d and used bm.name=c.d from bm.prefix=a.b.c, we shall match.
+                //We also match anyone in a 'deeper' namespace than the point of reference, but then only tentatively
+                //E.g., bm.prefix='a', bm.name='c' will also match a.b.c, but only if we don't have 'c' or 'a.c'
+                for (const auto& [bname, btype] : Blocks)
+                    if (dot_hierarchy_ends_with(bname, bm.name)) {
+                        //When testing bname='a.b.c.d' we get here for
+                        // 1. bm.prefix='', bm.name='a.b.c.d' (good, this is a match)
+                        // 2. bm.prefix='', bm.name='b.c.d' (bad, this is not a match)
+                        // 3. bm.prefix='a.b', bm.name='c.d' (good)
+                        // 4. bm.prefix='a.b', bm.name='b.c.d' (good)
+                        // 5. bm.prefix='a.b', bm.name='d' (bad)
+                        // 6. bm.prefix='hulu', bm.name='a.b.c.d' (good)
+                        // 7. bm.prefix='a.hulu', bm.name='b.c.d' (good)
+                        //So we first find the common (fully dot separated) part of bm.prefix and bname
+                        const std::string_view common_prefix{bname.data(), dot_hierarchy_common_prefix(bname, *bm.prefix)};
+                        //Then we test if the combined length of this prefix and bm.name  is longer than bname.
+                        //If the common prefix is '', we just test if bm.name is as long as bname.
+                        //Else we add 1 to cater for case #3 above.
+                        if ((common_prefix.empty() && bname == bm.name)
                             || (common_prefix.size() + 1 + bm.name.size() >= bname.size())) {
                         //OK, this is a match.
                         //If a previous match is a suffix of us (or empty), we take precedence
                         //Else we have ambiguity.
                         //Thus if we have bm.name="c" bm.prefix="b"
                         //Then we match both b.c and c, but prefer b.c
-                        if (!precise_match || dot_hierarchy_ends_with(bname, match)) {
-                            match = bname;
-                            precise_match = true;
-                            ambiguous = false;
+                            if (!precise_match || dot_hierarchy_ends_with(bname, match)) {
+                                match = bname;
+                                precise_match = true;
+                                ambiguous = false;
+                            }
+                        } else if (!precise_match) {
+                            if (match.empty()) match = bname;
+                            else ambiguous = true;
                         }
-                    } else if (!precise_match) {
-                        if (match.empty()) match = bname;
-                        else ambiguous = true;
                     }
+                if (match.empty()) {
+                    if (hadProcReplay) //we have replayed a procedure - blocks not found can be quite valid.
+                        AddCSH(bm.pos, COLOR_ENTITYNAME); //just color it ENTITY, but clear from mentions (no rename)
+                    else
+                        AddCSH_Error(bm.pos, "Block name not found.");
+                    bm.name.clear();
+                } else if (ambiguous) {
+                    AddCSH_Error(bm.pos, "Block name ambiguous (compile to get candidates).");
+                    bm.name.clear();
+                } else {
+                    bm.name = match;
+                    bm.prefix.reset();
+                    AddCSH(bm.pos, COLOR_ENTITYNAME);
                 }
-            if (match.empty()) {
-                if (hadProcReplay) //we have replayed a procedure - blocks not found can be quite valid.
-                    AddCSH(bm.pos, COLOR_ENTITYNAME); //just color it ENTITY, but clear from mentions (no rename)
-                else
-                    AddCSH_Error(bm.pos, "Block name not found.");
-                bm.name.clear();
-            } else if (ambiguous) {
-                AddCSH_Error(bm.pos, "Block name ambiguous (compile to get candidates).");
-                bm.name.clear();
-            } else {
-                bm.name = match;
-                bm.prefix.reset();
-                AddCSH(bm.pos, COLOR_ENTITYNAME);
             }
         }
-    BlockMentions.erase(std::remove_if(BlockMentions.begin(), BlockMentions.end(),
-                                       [](const BlockMention &bm) {return bm.name.empty(); }),
-                        BlockMentions.end());
+    std::erase_if(BlockMentions, [](const BlockMention& bm) {return bm.name.empty(); });
     Blocks.erase("prev");
     Blocks.erase("next");
     Blocks.erase("first");
     Blocks.erase("last");
 }
 
-void BlockCsh::UpdateBlocks(const CshStringWithPosList *l) {
+void BlockCsh::UpdateBlocks(const CshStringViewWithPosList *l) {
     if (!l) return;
-    for (const CshStringWithPos &sp : *l) {
-        auto [i, full_name] = SearchEntityName(sp.name);
+    for (const CshStringViewWithPos &sp : *l) {
+        auto [i, full_name] = SearchEntityName(sp.name, false);
         if (i>=-1)
             BlockMentions.emplace_back(sp.file_pos, full_name);
     }
@@ -680,16 +826,15 @@ void BlockCsh::AddCSH_LineBeginSoloString(CshPos pos, std::string_view string) {
 
     //option_names already contain chart options and element attribute names
     const unsigned opt = FindPrefix(option_names, string);
-    const auto [ent, full_name] = SearchEntityName(string);
+    const auto [ent, full_name] = SearchEntityName(string, !GetPedantic());
     const unsigned key = FindPrefix(keyword_names, string);
     const auto color = [&]() {
         if (opt==2) return COLOR_OPTIONNAME;
         if (key==2) return COLOR_KEYWORD;
         if (ent>=-1) return COLOR_ENTITYNAME; //good block name
-        if (ent==-3) return COLOR_ENTITYNAME_FIRST; //ambigous, but fully matching block name
         if (key==1) { was_partial = true; return COLOR_KEYWORD_PARTIAL; }
         if (opt==1) { was_partial = true; return COLOR_OPTIONNAME_PARTIAL; }
-        if (ent==-2) { was_partial = true; return COLOR_ENTITYNAME; }
+        if (ent==-2) { was_partial = true; return COLOR_ENTITYNAME_PARTIAL; }
         return COLOR_ERROR;
     }();
     if (color==COLOR_ERROR)
@@ -716,41 +861,78 @@ void BlockCsh::AddCSH_AfterMultiPartial(CshPos pos, std::string_view string)
  * Works incorrectly for blocks defined in a procedure.*/
 int BlockCsh::AddCSH_BlockName(const CshPos&pos, std::string_view name, EColorSyntaxType cinstead)
 {
-    const auto [ret, full_name] = SearchEntityName(name);
+    const auto [ret, full_name] = SearchEntityName(name, false);
     if (ret>=-1) {
         AddCSH(pos, COLOR_ENTITYNAME);
         if (full_name.size())
             BlockMentions.emplace_back(pos, full_name);
         return ret;
     }
-    if (cinstead!=COLOR_ERROR)
+    if (cinstead==COLOR_ERROR)
+        AddCSH_BlockNameError(pos, name, ret);
+    else
         AddCSH(pos, cinstead);
     return ret;
 }
 
+/** Colors a block name, that must be from the local parent.
+ * It is used inside copy blocks when looking up the element to manipulate
+ * (drop X, update X, replace X) or after 'before' clause. */
+void BlockCsh::AddCSH_LocalBlockName(const CshPos& pos, std::string_view name) {
+    if (name.empty()) return;
+    std::string n = PrefixName(name);
+    auto i = Blocks.lower_bound(n);
+    if (i==Blocks.end()) return;
+    if (i->first==n) {
+        AddCSH(pos, COLOR_ENTITYNAME);
+        BlockMentions.emplace_back(pos, n);
+    } else if (i->first.starts_with(n))
+        AddCSH(pos, COLOR_ENTITYNAME_PARTIAL);
+    else
+        AddCSH_BlockNameError(pos, name, -4); //not found
+}
+
+
 /** This only gets called when we refer to an entity in an arrow or update command.
  * It behaves like AddCSH_BlockName() if pedantic is on.
  * Else it behaves as AddCSH_NewBlock() with the current default shape.
  * Works incorrectly for blocks defined in a procedure.*/
-int BlockCsh::AddCSH_BlockNameOrNew(const CshPos&pos, std::string_view name, EColorSyntaxType cinstead)
+int BlockCsh::AddCSH_BlockNameOrNew(const CshPos&pos, std::string_view name)
 {
-    const auto [ret, full_name] = SearchEntityName(name);
+    const auto [ret, full_name] = SearchEntityName(name, true);
     if (ret>=-1) {
         AddCSH(pos, COLOR_ENTITYNAME);
         if (full_name.size())
             BlockMentions.emplace_back(pos, full_name);
         return ret;
     }
-    if (!Context2.back().pedantic &&
-        std::string::npos == name.find('.')) {
-        AddCSH_NewBlock(pos, name, Context2.back().def_shape);
-        return Context2.back().def_shape;
+    if (ret!=-3 && !GetPedantic() && std::string::npos == name.find('.')) {
+        const int shape = Context2.empty() ? -1 : Context2.back().def_shape;
+        AddCSH_NewBlock(pos, name, shape);
+        return shape;
     }
-    if (cinstead!=COLOR_ERROR)
-        AddCSH(pos, cinstead);
+    AddCSH_BlockNameError(pos, name, ret);
     return ret;
 }
 
+/** Adds a CSH error for a block name, when not found or ambiguous.
+ * err can be
+ * - >=-1: No error is added, this turned up to be a valid name.
+ * - -2: This is a partial name, we use COLOR_ENTITYNAME_PARTIAL.
+ * - -3: This name is ambiguous, we give an error
+ * - -4: This name is not found, we give an error*/
+
+void BlockCsh::AddCSH_BlockNameError(const CshPos& pos, std::string_view /*name*/, int err) {
+    switch (err) {
+    default: return;
+    case -2: AddCSH(pos, COLOR_ENTITYNAME_PARTIAL); return;
+    case -3: AddCSH_Error(pos, "This name matches multiple blocks."); return;
+    case -4:
+        if (hadProcReplay) AddCSH(pos, COLOR_ENTITYNAME);
+        else AddCSH_Error(pos, "Block not found.");
+    }
+}
+
 /** This is called when we realized that an arrow_end is standing at the beginning of a line.
  * At this point we have already colored it.
  * If this is a simple string, then it may also be a partial or full keyword
@@ -761,15 +943,35 @@ int BlockCsh::AddCSH_BlockNameOrNew(const CshPos&pos, std::string_view name, ECo
 void BlockCsh::UpdateCSH_ArrowEndAtLineBegin(const CshPos& pos, std::string_view arrow_end) {
     if (arrow_end.find_first_of("@,()+-")!=std::string_view::npos)
         return;
-    _ASSERT(CshList.size());
-    //Find the previous color entry, if any
-    const auto i = std::ranges::find(CshList, pos);
+    if (CshList.empty() && CshErrors.error_ranges.empty()) {
+        AddCSH_LineBeginSoloString(pos, arrow_end);
+        return;
+    }
     //If the previous CSH entry at this pos is this arrow_end, but it is not a block mention
     //(which may happen if pedantic is on and this is not an already existing block)
     //then we re-color it as a line begin string (keywords, options, existing blocks, etc.)
-    if (i!=CshList.end() && std::ranges::find(BlockMentions, pos, &BlockMention::pos)!=BlockMentions.end()) {
-        CshList.erase(i);
-        AddCSH_LineBeginSoloString(pos, arrow_end);
+    if (const auto i = std::ranges::find(CshList, pos); i!=CshList.end()) {
+        if (std::ranges::find(BlockMentions, pos, &BlockMention::pos)==BlockMentions.end()) {
+            CshList.erase(i);
+            AddCSH_LineBeginSoloString(pos, arrow_end);
+        } //else it is a block mention and we keep its color
+    } else if (const auto i = std::ranges::find(CshErrors.error_ranges, pos); i!=CshErrors.error_ranges.end()) {
+        const unsigned opt = FindPrefix(option_names, arrow_end);
+        const auto [ent, full_name] = SearchEntityName(arrow_end, !GetPedantic());
+        const unsigned key = FindPrefix(keyword_names, arrow_end);
+        std::string_view msg;
+        const auto color = [&]() {
+            if (opt==2) return COLOR_OPTIONNAME;
+            if (key==2) return COLOR_KEYWORD;
+            if (ent>=-1) return COLOR_ENTITYNAME; //good block name
+            if (key==1) { was_partial = true; i->text.append(" Only a partial keyword.");  return COLOR_KEYWORD_PARTIAL; }
+            if (opt==1) { was_partial = true; i->text.append(" Only a partial option name."); return COLOR_OPTIONNAME_PARTIAL; }
+            if (ent==-2) { was_partial = true; i->text.append(" Only a partial entity name."); return COLOR_ENTITYNAME_PARTIAL; }
+            i->text.append(" Not a keyword or option.");
+            return COLOR_ERROR;
+        }();
+        if (color!=COLOR_ERROR)
+            AddCSH(pos, color);
     }
 }
 
@@ -809,8 +1011,8 @@ void BlockCsh::DropBlockMentions(CshPos pos) {
 /** Take a CshStringWithPosList and color according to it.
  * We may assume the elements come in order.
  * @param [in] l The list contains tokens (names) and their positions.
- *               Names may be empty, in this case an error occured there and
- *               we dont need to color.
+ *               Names may be empty, in this case an error occurred there and
+ *               we don't need to color.
  * @param [in] pos The whole span we color. We shall color before and after
  *                 any blocks within this position range.
  * @param [in] cname Color names to this color (except empty ones)
@@ -822,7 +1024,7 @@ void BlockCsh::DropBlockMentions(CshPos pos) {
  * @param [in] cinstead if not COLOR_ERROR and cname is COLOR_ENTITYNAME and the
  *                      list is only of one element and that is not a blockname,
  *                      we use this color instead.*/
-void BlockCsh::AddCSH_NameList(const CshStringWithPosList * l, const CshPos & pos,
+void BlockCsh::AddCSH_NameList(const CshStringViewWithPosList* l, const CshPos& pos,
                                EColorSyntaxType cname, EColorSyntaxType cbetween,
                                EColorSyntaxType cnumber, EColorSyntaxType cinstead)
 {
