@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #ifndef BLOCK_STYLE_H
 #define BLOCK_STYLE_H
 
+#include <variant>
 #include "style.h"
 
 namespace block
@@ -59,7 +60,8 @@ enum class EBlockType : unsigned char
     Cell,   ///<An invisible set of blocks, continuing the default alignment of the parent, no margin and no line
     Text,   ///<Text with normally no line around it
     Shape,  ///<A block having a specific shape, other than a box
-    Space   ///<Just empty space with a size
+    Space,  ///<Just empty space with a size. Breaks and indicators also have this type.
+    Indicator ///<An indicator
 };
 /** Shall we draw tracking rectangles */
 constexpr bool HasArea(EBlockType t) { return t!=EBlockType::Space; }
@@ -187,10 +189,10 @@ ParseCoord(std::string_view text, MscError *error, const FileLineCol *l, std::st
 struct OptAlignAttr
 {
     bool is_set = false;   ///<True if the attribute is set. If false, the value is to be ignored.
-    bool is_margin{};      ///<True if our margin shall be aligned, false if our visible side. Ignored on Mid or if is_set is false.
+    bool is_margin = false;///<True if our margin shall be aligned, false if our visible side. Ignored on Mid or if is_set is false.
     EStyleType set_from{}; ///<Whether set from a style or as an attribute (for better error messages)
     AlignTo value;         ///<The value to align to. Valid only if 'set' is true.
-    std::string text;      ///<Human readable text of the attribute, without its priority
+    std::string text{};    ///<Human readable text of the attribute, without its priority
 
     void SetText(std::string &&attrname);
     /** Returns true if we are actually set to a non-empty value.*/
@@ -285,7 +287,7 @@ struct InternalAlignment
 struct SizeAttr {
     double size;               ///<Pixel size specified by the user. If 'blocks' is non-empty, it is a multiplier of the size of the bounding box of those blocks.
     EAlignPrio prio;           ///<The priority of this attribute (stye, explicit, etc.)
-    StringWithPosList blocks;  ///<If this is non-empty, use the specific double value as a multiplier
+    StringWithPosList blocks{};///<If this is non-empty, use the specific double value as a multiplier
 };
 
 /** An optional size attribute (for width and height) with a priority */
@@ -326,13 +328,9 @@ enum class EBlockStyleType
  * Note that we use EStyleType::OPTION for the running style. */
 class BlockStyle : public SimpleStyleWithArrow<TwoArrowHeads>
 {
-    friend class BlockContext;
-    BlockStyle(EStyleType tt, EBlockStyleType btt, EColorMeaning cm, bool a,
-               bool t, bool l, bool f, bool s, bool nu, bool shp,
-               bool lab, bool al, bool c, bool m, bool r, bool mkr); ///Control which components are used
     bool f_label;           ///<Flag specifying if this style has labels, label.pos, label.align, label.orient
     bool f_alignment;       ///<Flag specifying if this style has alignment attributes "top", "bottom" etc,
-    bool f_content;         ///<Flag specifying if this style has attributes governing children alignemnt "content.*" and "allow_routing"
+    bool f_content;         ///<Flag specifying if this style has attributes governing children alignemnt "content.*" and "allow_routing" and "indicator"
     bool f_margin;          ///<Flag specifying if this style has margin attributes "margin.*" "imargin.*", "width", "height", "size" and "multi.*"
     bool f_routing;         ///<Flag specifying if this style has 'routing' attribute
     bool f_marker;          ///<Flag specifying if this style has 'marker' attributes
@@ -368,7 +366,11 @@ public:
     OneArrowHead               marker;                ///<If true and we have de-overlapped this arrow its path will be added to the blocking parts of its peers.
 	StringWithPosList          draw_time;             ///<If not empty, the value of the 'draw_before' or 'draw_after' attribute(s). A block(s), we need to be drawn before/after.
 	bool                       draw_time_is_before;   ///<Tells us if 'draw_time' is before or after
+    OptAttr<bool>              indicator;             ///<If collapsed blocks show an indicator or not
 
+    BlockStyle(EStyleType tt, EBlockStyleType btt, EColorMeaning cm, bool a,
+               bool t, bool l, bool f, bool s, bool nu, bool shp,
+               bool lab, bool al, bool c, bool m, bool r, bool mkr); ///Control which components are used
     explicit BlockStyle(EStyleType tt, EColorMeaning cm = EColorMeaning::NOHOW); //Has all the components, but is empty
     explicit BlockStyle(EBlockStyleType btt = EBlockStyleType::Generic, EColorMeaning cm = EColorMeaning::NOHOW); //Has all the components, but is empty
     BlockStyle(const BlockStyle&) = default;
@@ -376,6 +378,7 @@ public:
     BlockStyle &operator=(const BlockStyle&) = default;
     BlockStyle &operator=(BlockStyle&&) = default;
     void Empty() override;
+    bool IsEmpty() const noexcept override;
     void MakeCompleteButText() override;
     EAlignPrio PrioOnAttributeAdd() const;
     Style &operator +=(const Style &toadd) override;
@@ -385,13 +388,27 @@ public:
 	void Push(const FileLineCol &l);
 };
 
+class BlockBlock;
+
+/** This holds the parent block of the current context.
+ * Alternatively, during parsing of clone action modifier lists it contains the name of the
+ * (not yet existing) clone. */
+struct Parent : std::variant<const BlockBlock*, std::string> {
+    using std::variant<const BlockBlock*, std::string>::variant;
+    using std::variant<const BlockBlock*, std::string>::operator=;
+    Parent() noexcept : std::variant<const BlockBlock*, std::string>(nullptr) {}
+    const BlockBlock* AsBlock() const { return std::holds_alternative<const BlockBlock*>(*this) ? std::get<const BlockBlock*>(*this) : nullptr; }
+    std::string_view AsName() const { return std::holds_alternative<std::string>(*this) ? std::get<std::string>(*this) : std::string_view{}; }
+    std::string_view GetName() const;
+};
+
 /** Context for Graphs. Just a styleset and colors, really.*/
 class BlockContext : public ContextBase<BlockStyle>
 {
 public:
-    BlockStyle running_style_blocks; ///<The running style of this context. Added to all Blocks.
-    BlockStyle running_style_arrows; ///<The running style of this context. Added to all Arrows.
-    std::string blockname_prefix;    ///<The prefix (without the dot) added to block names. Actually the full name of the parent block.
+    StyleCoW<BlockStyle> running_style_blocks; ///<The running style of this context. Added to all Blocks.
+    StyleCoW<BlockStyle> running_style_arrows; ///<The running style of this context. Added to all Arrows.
+    Parent parent;                   ///<The parent block of the current context or the name of the block we clone into
     FillAttr background;             ///<The fill of the main block. Added here so that it can be part of designs.
 	OptAttr<bool> pedantic;          ///<If false, we allow block definitions via arrows
 
@@ -405,10 +422,10 @@ public:
      * @param [in] l The first character of the context in the input file.*/
     BlockContext(bool f, EContextParse p, EContextCreate t, const FileLineCol &l) :
         ContextBase<BlockStyle>(f, p, EContextCreate::CLEAR, l),
-        running_style_blocks(EStyleType::STYLE, EBlockStyleType::Running, EColorMeaning::FILL, false,
-                             true, true, true, true, true, true, true, true, true, true, false, false), //no arrow, routing, marker
-        running_style_arrows(EStyleType::STYLE, EBlockStyleType::Running, EColorMeaning::LINE_ARROW_TEXT, true,
-                             true, true, false, false, true, false, true, false, false, false, true, true) //just arrow, text, line and number, label, routing and marker
+        running_style_blocks(std::make_unique<BlockStyle>(EStyleType::STYLE, EBlockStyleType::Running, EColorMeaning::FILL, false,
+                                                          true, true, true, true, true, true, true, true, true, true, false, false)), //no arrow, routing, marker
+        running_style_arrows(std::make_unique<BlockStyle>(EStyleType::STYLE, EBlockStyleType::Running, EColorMeaning::LINE_ARROW_TEXT, true,
+                                                          true, true, false, false, true, false, true, false, false, false, true, true)) //just arrow, text, line and number, label, routing and marker
     { switch (t) {
         default: _ASSERT(0); FALLTHROUGH;
         case EContextCreate::PLAIN: Plain(); break;
@@ -420,13 +437,14 @@ public:
     BlockContext(const BlockContext &o, EContextParse p, const FileLineCol &l) :
         ContextBase<BlockStyle>(o, p, l),
         running_style_blocks(o.running_style_blocks), running_style_arrows(o.running_style_arrows),
-        blockname_prefix(o.blockname_prefix), background(o.background), pedantic(o.pedantic) {}
+        parent(o.parent), background(o.background), pedantic(o.pedantic) {}
     /** Semi copy operator: copy only user-defined context content. If o is full, we overwrite us, else just merge.*/
     void ApplyContextContent(const BlockContext &o);
     /** Semi move operator: move only user-defined context content. If o is full, we overwrite us, else just merge.*/
     void ApplyContextContent(BlockContext &&o);
     void Empty() override;
     void Plain() override;
+    std::string_view GetPrefix() const { return parent.GetName(); }
 };
 
 
