@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -69,10 +69,10 @@ public:
     Area &operator *= (const Area &b) {Contour::operator*=(b); mainline*=b.mainline; if (arc==nullptr) arc = b.arc; return *this;}
     Area &operator -= (const Area &b) {Contour::operator-=(b); mainline-=b.mainline; if (arc==nullptr) arc = b.arc; return *this;}
     Area &operator ^= (const Area &b) {Contour::operator^=(b);                       if (arc==nullptr) arc = b.arc; return *this;}
-    Area &operator += (Area &&b) {Contour::operator+=(static_cast<Contour&&>(b)); mainline+=std::move(b.mainline); if (arc==nullptr) arc = b.arc; return *this;}
-    Area &operator *= (Area &&b) {Contour::operator*=(static_cast<Contour&&>(b)); mainline*=std::move(b.mainline); if (arc==nullptr) arc = b.arc; return *this;}
-    Area &operator -= (Area &&b) {Contour::operator-=(static_cast<Contour&&>(b)); mainline-=std::move(b.mainline); if (arc==nullptr) arc = b.arc; return *this;}
-    Area &operator ^= (Area &&b) {Contour::operator^=(static_cast<Contour&&>(b));                                  if (arc==nullptr) arc = b.arc; return *this;}
+    Area &operator += (Area &&b) { mainline += std::move(b.mainline); Contour::operator+=(static_cast<Contour&&>(b)); if (arc==nullptr) arc = b.arc; return *this;}
+    Area &operator *= (Area &&b) { mainline *= std::move(b.mainline); Contour::operator*=(static_cast<Contour&&>(b)); if (arc==nullptr) arc = b.arc; return *this;}
+    Area &operator -= (Area &&b) { mainline -= std::move(b.mainline); Contour::operator-=(static_cast<Contour&&>(b)); if (arc==nullptr) arc = b.arc; return *this;}
+    Area &operator ^= (Area &&b) {                                    Contour::operator^=(static_cast<Contour&&>(b)); if (arc==nullptr) arc = b.arc; return *this;}
 
     Area &operator += (const Contour &b) { Contour::operator+=(b); return *this; }
     Area &operator *= (const Contour &b) { Contour::operator*=(b); return *this; }
@@ -124,16 +124,16 @@ public:
 
     /** Determine the relative vertical distance between two shapes. See Contour::OffsetBelow for more. Ignores 'mainline'.*/
     double OffsetBelow(const contour::Contour &below, double &touchpoint, double offset = CONTOUR_INFINITY) const;
-    /** Determine the relative vertical distance between two shapes. See Contour::OffsetBelow for more. 
+    /** Determine the relative vertical distance between two shapes. See Contour::OffsetBelow for more.
      * If bMainline is true, we also take mainlines into account not allowing them to overlap either.*/
     double OffsetBelow(const Area &below, double &touchpoint, double offset = CONTOUR_INFINITY, bool bMainline = true) const;
 };
 
 /** A list of Area objects.
- * No effort is made to avoid overlap, thus addition 
+ * No effort is made to avoid overlap, thus addition
  * operations are indeed just appends to the list.
  * We maintain a total boundingBox and mainline and
- * these are actually combined to a single block and 
+ * these are actually combined to a single block and
  * contour, respectively.
  * Ordering is important in the list for InWhich*
  * functions.*/
