@@ -140,9 +140,9 @@ enum class EPageSize {
     LETTER_P, LETTER_L,
     LEGAL_P, LEGAL_L,
     LEDGER, TABLOID,
-    WIDESCREEN_P, WIDESCREEN_L, 
-    ON_SCREEN_4_3_P, ON_SCREEN_4_3_L, 
-    ON_SCREEN_16_9_P, ON_SCREEN_16_9_L, 
+    WIDESCREEN_P, WIDESCREEN_L,
+    ON_SCREEN_4_3_P, ON_SCREEN_4_3_L,
+    ON_SCREEN_16_9_P, ON_SCREEN_16_9_L,
     ON_SCREEN_16_10_P, ON_SCREEN_16_10_L,
     MAX_PAGE ///<Not used, the max value.
 };
@@ -232,6 +232,9 @@ public:
      * Returns false if we shall output higher-level shapes, e.g., for a PPT. */
     bool does_graphics() const noexcept { return outType!=PPT; }
 
+    /** Returns this correction value */
+    bool shall_avoid_aligned_bk_clip() const noexcept { return avoid_aligned_bk_clip; }
+
     /** Return true if the canvas is not properly set up for drawing.
      * This can be used for 3 things:
      * 1. to spare drawing operations for dummy canvases;
@@ -253,41 +256,42 @@ public:
 protected:
     /** @name Low-level compatibility options
      * @{ */
-    bool     white_background;        /** Draw a white background before chart drawing.*/
-    double   fake_scale;              /** Final rendering should be scaled like this. Used for WMF, since WMF coordinates are 16-bit integers. We are better off if the chart is scaeld up (less rounding), but not more than 32K.*/
-    double 	 fallback_resolution;     /** The resolution of the fallback images for vector surfaces.*/
-    bool     needs_dots_in_corner;    /** Draw a dot in upper left and lower right corner. Needed for EMF/WMF for the output to include the white space around the chart.*/
-    bool     use_text_path;           /** Use cairo_text_path() instead of cairo_show_text(). I am not sure we have any backend needing this.*/
-    bool     use_text_path_rotated;   /** Use cairo_text_path() instead of cairo_show_text() for rotated text. WMF cannot display text if the transformation matrix is rotated. */
-    int      individual_chars_path;   /** 0: off, 1: Each character is drawn one by one, and not in a single call to cairo_show_text(); 2: non-ascii chars are shown with cairo_text_path(). WMF does not scale text nicely, this is a workaround. */
-    unsigned fake_gradients;          /** If nonzero then do not use cairo gradients, mimic them with a lot of fills. This is the number of steps. */
-    double   scale_for_shadows;       /** How many pixel lies below one logical pixel (needed for fine shadows) */
-    bool     fake_shadows;            /** Do not use alpha blending in shadows, use solid color as if on white background. (This avoids fallback image for EMF/WMF, which support no transparency.) */
-    bool     fake_dash;               /** Do not use cairo dash, mimic them with individual dashes. WMF supports no dash, EMF supports no dash-offset. */
-    bool     fake_spaces;             /** Add space for leading & trailing spaces at cairo_show_text(), assuming those are skipped by it. I do not remember, why we need it. */
-    bool     needs_arrow_fix;         /** Cannot do the convex clipping for arrowheads, use a simpler method. */
-    bool     avoid_linewidth_1;       /** Never draw lines of width exactly 1.0. EMF with cairo 1.12.2 needs to have wider lines like 1.01. */
-    bool     imprecise_positioning;   /** EMF/WMF has trouble doing lines on 0.5 boundaries, avoid this.*/
-    bool     needs_strokepath_rclboundsfix;  /** EMF/WMF Attaches a clip region to every stroke. If that is too small, rectilinear lines disappear when scaling to small object.*/
-    bool     can_and_shall_clip_total;/** True if we can clip to the total chart area. Does not work for WIN, so we need this flag */
-    bool     avoid_transparency;      /** Pipes and gradient fills will be flattened before output. */
+    bool     white_background;        ///< Draw a white background before chart drawing.
+    double   fake_scale;              ///< Final rendering should be scaled like this. Used for WMF, since WMF coordinates are 16-bit integers. We are better off if the chart is scaeld up (less rounding), but not more than 32K.
+    double 	 fallback_resolution;     ///< The resolution of the fallback images for vector surfaces.
+    bool     needs_dots_in_corner;    ///< Draw a dot in upper left and lower right corner. Needed for EMF/WMF for the output to include the white space around the chart.
+    bool     use_text_path;           ///< Use cairo_text_path() instead of cairo_show_text(). I am not sure we have any backend needing this.
+    bool     use_text_path_rotated;   ///< Use cairo_text_path() instead of cairo_show_text() for rotated text. WMF cannot display text if the transformation matrix is rotated.
+    int      individual_chars_path;   ///< 0: off, 1: Each character is drawn one by one, and not in a single call to cairo_show_text(); 2: non-ascii chars are shown with cairo_text_path(). WMF does not scale text nicely, this is a workaround.
+    unsigned fake_gradients;          ///< If nonzero then do not use cairo gradients, mimic them with a lot of fills. This is the number of steps.
+    double   scale_for_shadows;       ///< How many pixel lies below one logical pixel (needed for fine shadows)
+    bool     fake_shadows;            ///< Do not use alpha blending in shadows, use solid color as if on white background. (This avoids fallback image for EMF/WMF, which support no transparency.)
+    bool     fake_dash;               ///< Do not use cairo dash, mimic them with individual dashes. WMF supports no dash, EMF supports no dash-offset.
+    bool     fake_spaces;             ///< Add space for leading & trailing spaces at cairo_show_text(), assuming those are skipped by it. I do not remember, why we need it.
+    bool     needs_arrow_fix;         ///< Cannot do the convex clipping for arrowheads, use a simpler method.
+    bool     avoid_linewidth_1;       ///< Never draw lines of width exactly 1.0. EMF with cairo 1.12.2 needs to have wider lines like 1.01.
+    bool     imprecise_positioning;   ///< EMF/WMF has trouble doing lines on 0.5 boundaries, avoid this.
+    bool     needs_strokepath_rclboundsfix;  ///< EMF/WMF Attaches a clip region to every stroke. If that is too small, rectilinear lines disappear when scaling to small object.
+    bool     can_and_shall_clip_total;///< True if we can clip to the total chart area. Does not work for WIN, so we need this flag
+    bool     avoid_aligned_bk_clip;   ///< If set, we make sure that the background fill is not aligned on a page boundary. Page clip seems not to work for such rectangles in WMF/EMF.
+    bool     avoid_transparency;      ///< Pipes and gradient fills will be flattened before output.
     /** @} */
 
     /** @name Context data
      * @{ */
-    string           fileName;         /** The name of the output file (if any) in UTF-8*/
-    FILE*            outFile;          /** The opened output file (if opened in constructor). If we have to add iTXt chunks to a PNG file, this is a temp file.*/
-    FILE*            finalOutFile;     /** The eventual filename. Used if we add iTXt chunks to a PNG file.*/
-    cairo_surface_t *surface;          /** The cairo surface we draw to */
-    cairo_t         *cr;               /** The cairo context we draw to */
-    PptFile          ppt;              /** The PPT file, in case of PPT output */
-    std::vector<std::vector<SimpleTransform>> tr_stack; /** In case of non-graphics (PPT) output this is the transform stack. */
-    std::string      error;            /** Empty if the Canvas is OK, else contains an error. */
-    bool             fatal = true;     /** If set, any error above will be reported as fatal. */
-    bool             candraw;          /** False, if Canvas cannot be drawn on. Used only in debug asserts.*/
-    bool             dummy = false;    /** If true, we ignore any drawing operations silently. */
-    double           fake_dash_offset; /** Status of fake dashes */
-    EmbedChartData   iTXt_data;        /** The chart data (language, source text, etc.) to be added as an iTXt chunk to PNG files.*/
+    string           fileName;         ///< The name of the output file (if any) in UTF-8
+    FILE*            outFile;          ///< The opened output file (if opened in constructor). If we have to add iTXt chunks to a PNG file, this is a temp file.
+    FILE*            finalOutFile;     ///< The eventual filename. Used if we add iTXt chunks to a PNG file.
+    cairo_surface_t *surface;          ///< The cairo surface we draw to
+    cairo_t         *cr;               ///< The cairo context we draw to
+    PptFile          ppt;              ///< The PPT file, in case of PPT output
+    std::vector<std::vector<SimpleTransform>> tr_stack; ///< In case of non-graphics (PPT) output this is the transform stack.
+    std::string      error;            ///< Empty if the Canvas is OK, else contains an error.
+    bool             fatal = true;     ///< If set, any error above will be reported as fatal.
+    bool             candraw;          ///< False, if Canvas cannot be drawn on. Used only in debug asserts.
+    bool             dummy = false;    ///< If true, we ignore any drawing operations silently.
+    double           fake_dash_offset; ///< Status of fake dashes
+    EmbedChartData   iTXt_data;        ///< The chart data (language, source text, etc.) to be added as an iTXt chunk to PNG files.
     /** @}*/
 
     /** @name Values recorded at construction
@@ -304,10 +308,13 @@ protected:
 
     /** @name Logic
      * @{ */
-    void SetLowLevelParams();
-    static void GetPagePosition(const Block &total, const PBDataVector *pageBreakData,
-                                unsigned page, XY &offset, XY &size,
-                                const PageBreakData **relevant = nullptr);
+    void SetLowLevelParams(const XY &size = XY(0, 0));
+    struct PagePosition {
+        XY size;
+        XY offset;
+        const PageBreakData* page_break_data;
+    };
+    static PagePosition GetPagePosition(const Block &total, const PBDataVector *pageBreakData, unsigned page) noexcept;
     void CreateSurface(const XY &size);
     void CreateContext(const XY &origSize, const XY &origOffset,const XY &headingLeftingSize);
     void CreateNilContext() { return CreateContext(XY(0, 0), XY(0, 0), XY(0, 0)); }
