@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -595,7 +595,7 @@ public:
         if (dirty)
         { CreateAnArrowHead(endArrowHead, scale, endType, gvEndType); dirty = false; }
     }
-    const ArrowHead& GetArrowHead(bool bidir, EArrowEnd which) const noexcept { _ASSERT(!dirty); return which==EArrowEnd::END ? endArrowHead : ArrowHead::noneArrowHead; }
+    const ArrowHead& GetArrowHead(bool /*bidir*/, EArrowEnd which) const noexcept { _ASSERT(!dirty); return which==EArrowEnd::END ? endArrowHead : ArrowHead::noneArrowHead; }
 
     /** Returns what area we cover from the target at the end (e.g., an entity line).
      * (Used only for symmetric empty arrowheads, others return empty.)
