@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -20,8 +20,6 @@
 /** @file cgen_attribute.cpp The implementation of Attribute and line, fill, shadow and width attributes.
 * @ingroup libcgencommon_files */
 
-#define _CRT_NONSTDC_NO_DEPRECATE //For visual studio to allow strdup
-
 #include <cassert>
 #include <cstring>
 #include "cgen_attribute.h"
@@ -63,7 +61,7 @@ bool CaseInsensitiveEndsWith(std::string_view base, std::string_view a) noexcept
     return i2<0 || base[i2]=='.';
 }
 
-std::optional<size_t>  CaseInsensitiveContains(std::string_view base, std::string_view a) noexcept {
+std::optional<size_t> CaseInsensitiveContains(std::string_view base, std::string_view a) noexcept {
     if (base.length() < a.length()) return {};
     for (size_t i = 0; i<=base.length()-a.length(); i++)
         if (CaseInsensitiveCommonPrefixLen(base.substr(i), a)==a.length())
@@ -72,19 +70,6 @@ std::optional<size_t>  CaseInsensitiveContains(std::string_view base, std::strin
 }
 
 
-std::string PrintStrings(const std::vector<std::string> &ls)
-{
-    string ret;
-    for (auto &s : ls) {
-        if (s.length()==0) continue;
-        if (ret.length()) ret += "', '";
-        else ret = "'";
-        ret += s;
-    }
-    ret += "'";
-    return ret;
-}
-
 Attribute::Attribute(std::string_view a, std::string_view s, const FileLineColRange & l, const FileLineColRange & v,
                      bool force_string) :
     type(EAttrType::STRING), name(a), value(s),
@@ -399,7 +384,7 @@ bool LineAttr::AddAttribute(const Attribute &a, Chart *chart, EStyleType t)
         if (ELineType t;  a.type == EAttrType::STRING && Convert(a.value, t)) {
             type = t;
             return true;
-        } else 
+        } else
             a.InvalidValueError(CandidatesFor(t), chart->Error);
         return true;
     }
@@ -427,7 +412,7 @@ bool LineAttr::AddAttribute(const Attribute &a, Chart *chart, EStyleType t)
             if (!radius || *radius<=0)
                 radius = 10;
             return true;
-        } else 
+        } else
             a.InvalidValueError(CandidatesFor(c), chart->Error);
         return true;
     }
@@ -459,7 +444,7 @@ void LineAttr::AttributeNames(Csh &csh, std::string_view prefix)
     "type", "The style of the line, such as dotted, dashed or double",
     "width", "The width of the line",
     "radius", "How wide the line turns in corners (only for some corner types)",
-    "corner", "The style of tunrning at corners.",
+    "corner", "The style of turning at corners.",
     ""};
     csh.AddToHints(names_descriptions, csh.HintPrefix(COLOR_ATTRNAME).append(prefix), EHintType::ATTR_NAME);
 }
@@ -533,7 +518,7 @@ string LineAttr::Print(int) const
     return ss + ")";
 }
 
-//This one does not assume anything about wether the resulting rectange should be the
+//This one does not assume anything about wither the resulting rectangle should be the
 //outer edge or inner edge of the line - just uses the radius value and coordinates
 //as they are.
 //"this->radius" corresponds to the radius at the middle of the line
@@ -736,9 +721,9 @@ Contour LineAttr::CreateRectangle_InnerEdge(double x1, double x2, double y1, dou
    /                     \
    |                     |
    @endverbatim
- * Here we see the top of a bevelled rectangle with a label 'AAAAAAAAA' in it.
+ * Here we see the top of a beveled rectangle with a label 'AAAAAAAAA' in it.
  * We can see that the vertical sides of the rectangle are spaced further apart than
- * the length of the label - this is because of the bevelled corners.
+ * the length of the label - this is because of the beveled corners.
  *
  * In this function the label is already laid out on the surface - its extent is captured in `textCover`.
  * We want the very top of the rectangle (the outer edge of its upper horizontal line)
@@ -749,7 +734,7 @@ Contour LineAttr::CreateRectangle_InnerEdge(double x1, double x2, double y1, dou
  * This one assumes that the
  * @param [in] textCover The extents of the label
  * @param [in] rect_top The y coordinate of the outer edge of the upper horizontal line of the rectangle.
- * @returns The left and right margin to be used from the leftmost and rightmost point of `textCover`, resp
+ * @returns The left and right margin to be used from the leftmost and rightmost point of `textCover`, resp.
  *          till the *outer* edge of the vertical sides of the rectangle.
  *          This margin is at least as much as the line width then. We return (0,0) if `textCover` is empty.
  */
@@ -993,7 +978,7 @@ string FillAttr::Print(int) const
 /** If this fill is applied to a rectangle, what will be the typical fill
  * at a horizontal line of a rectangle. Call only if fully specified.
  * @param [in] pos If 0 we show what is the fill at the top of the rectangle
- *             If 1, it will be the botom of the rectangle. [0..1] is OK.*/
+ *             If 1, it will be the bottom of the rectangle. [0..1] is OK.*/
 FillAttr FillAttr::ApproxColorHorizontal(double pos) const noexcept {
     switch (gradient.value_or(EIntGradType::NONE)) {
     default:
@@ -1439,7 +1424,7 @@ void OneArrowAttr::MakeComplete()
 OneArrowAttr & OneArrowAttr::operator += (const OneArrowAttr &toadd)
 {
     //here we always set dirty - but maybe we do not change the attribute
-    //potential for improvement - but little, since we usually dont set the attribute
+    //potential for improvement - but little, since we usually don't set the attribute
     //after CreateArrowHeads anyway.
     line += toadd.line;
     ApplyArrowType(endType, toadd.endType, gvEndType, toadd.gvEndType, toadd.type);
@@ -1526,7 +1511,7 @@ void OneArrowAttr::AttributeNames(Csh &csh, std::string_view prefix)
         "color", "The color of the arrowheads",
         "type", "The arrowhead type to use at the destination of the arrow.",
         "line.width", "The line with of line-like arrowheads.",
-        "line.color", "The color of the line of the arrwhead.",
+        "line.color", "The color of the line of the arrowhead.",
         "line.type", "The line style of the line-like arrowheads, like dotted, dashed, etc.",
         "line.corner", "Determines, how to join arrowheads when on a bent line.",
         "xmul", "A size multiplier applied to the width arrowheads. To increase width, for example, specify a number greater than 1.",
@@ -1685,7 +1670,8 @@ void FullArrowAttr::MakeComplete()
 {
     TwoArrowAttr::MakeComplete();
     if (!midType && !gvMidType) { midType = EArrowType::SOLID; dirty = true; }
-    if (!skipType&& !gvSkipType) { skipType = EArrowType::NONE; dirty = true; }
+    if (type != EArcArrowType::BIGARROW)
+        if (!skipType && !gvSkipType) { skipType = EArrowType::NONE; dirty = true; }
 }
 
 FullArrowAttr & FullArrowAttr::operator += (const FullArrowAttr &toadd)
@@ -1693,7 +1679,7 @@ FullArrowAttr & FullArrowAttr::operator += (const FullArrowAttr &toadd)
     TwoArrowAttr::operator+=(toadd);
     //in all ernesty we strive to keep skipType NONE for bigarrows.
     ApplyArrowType(midType, toadd.midType, gvMidType, toadd.gvMidType, toadd.type);
-    if (type != EArcArrowType::BIGARROW)
+    if (type != EArcArrowType::BIGARROW && toadd.type != EArcArrowType::BIGARROW)
         ApplyArrowType(skipType, toadd.skipType, gvSkipType, toadd.gvSkipType, toadd.type);
     return *this;
 }
@@ -1769,7 +1755,7 @@ bool FullArrowAttr::AttributeValues(std::string_view attr, Csh &csh, EArcArrowTy
 
 ////////////////////////////////////////////////////////////
 
-bool WidthAttr::operator == (const WidthAttr &a) const
+bool WidthAttr::operator == (const WidthAttr &a) const noexcept
 {
     if (a.is_set != is_set) return false;
     if (a.is_set && (a.value != value || (value<0 && a.str != str))) return false;
