@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -58,34 +58,33 @@ using contour::ECloseType;
 using std::string;
 
 /** Helper class, holds two doubles.
- * Typically used to store left/right marging values.
+ * Typically used to store left/right margin values.
  * Sometimes during computation the values may be reversed or
  * mean something different. */
 struct DoublePair
 {
     double left;   ///<The margin typically on the left side
     double right;  ///<The margin typically on the right side
-    DoublePair() = default;
-    DoublePair(double a, double b) : left(a), right(b) {}
     /** Swaps the two sides. */
     void swap() {std::swap(left, right);}
 };
 
-/** Supply string with a good append ("<<") operator.*/
-template <class T>
-string &operator <<(string&s, T t) {
-    std::ostringstream ss;
-    ss << t;
-    s.append(ss.str());
-    return s;
-}
-
-/** Supply a string view append operator to string.*/
-inline string &operator <<(string&s, std::string_view t)
-{
-    s.append(t);
-    return s;
-}
+/** Supply string with a good append ("<<") operator.
+ * When a lot of small strings are to be added and/or the result is expected
+ * to be longer than 16 bytes (what small string optimization can handle),
+ * use StrCat() instead.*/
+template <class T> requires std::is_arithmetic_v<T>
+string &operator <<(string &s, T t) { return s.append(std::to_string(t)); }
+template <class T> requires std::is_arithmetic_v<T>
+string&& operator <<(string&& s, T t) { return std::move(s.append(std::to_string(t))); }
+string& operator <<(string& s, bool t) = delete;
+string&& operator <<(string&& s, bool t) = delete;
+inline string& operator <<(string& s, const char *t) { return s.append(t); } //needed or it converts to bool
+inline string&& operator <<(string&& s, const char* t) { return std::move(s.append(t)); }
+inline string &operator <<(string&s, std::string_view t) { return s.append(t); }
+inline string&& operator <<(string&& s, std::string_view t) { return std::move(s.append(t)); }
+inline string& operator <<(string& s, std::string&& t) { return s.append(std::move(t)); }
+inline string&& operator <<(string&& s, std::string&& t) { return std::move(s.append(std::move(t))); }
 
 /** Return length the common prefix in two strings (case insensitive). */
 unsigned CaseInsensitiveCommonPrefixLen(std::string_view a, std::string_view b) noexcept;
@@ -144,8 +143,7 @@ void CopyTo(std::string::iterator to, char a, StrViews ... views);
 template <typename ... StrViews>
 inline void CopyTo(std::string::iterator to, std::string_view a, StrViews ... views)
 {
-    std::copy(a.begin(), a.end(), to);
-    CopyTo(to+a.length(), views...);
+    CopyTo(std::copy(a.begin(), a.end(), to), views...);
 }
 
 template <typename ... StrViews>
@@ -186,14 +184,27 @@ struct StringWithPos
 };
 using StringWithPosList = std::vector<StringWithPos>;
 
-/** Converts a list of strings to a single string enclosed in 's.*/
-std::string PrintStrings(const std::vector<std::string> &ls);
+/** Converts a range of strings to a single string enclosed in 's.*/
+template <std::ranges::range R> requires(requires(R r) { std::string() += *r.begin(); })
+std::string PrintStrings(R &&r)
+{
+    string ret;
+    for (auto& s : r) {
+        if (s.length() == 0) continue;
+        if (ret.length()) ret += "', '";
+        else ret = "'";
+        ret += s;
+    }
+    ret += "'";
+    return ret;
+}
 
-/** Converts 's' to a flotaing point or signed or unsigned integer
- * value. Returns true on error including the case when not all 
+
+/** Converts 's' to a floating point or signed or unsigned integer
+ * value. Returns true on error including the case when not all
  * characters of s can be consumed.*/
 template <typename T> requires(std::is_arithmetic_v<T> && !std::is_const_v<T>)
-inline bool from_chars(std::string_view s, T& value)
+inline bool from_chars(std::string_view s, T& value) noexcept
 {
     if (s.length()==0) return true;
     if (s.front()=='+') s.remove_prefix(1);
@@ -201,6 +212,10 @@ inline bool from_chars(std::string_view s, T& value)
     return res.ec!=std::errc{} || res.ptr!=&s.back()+1;
 }
 
+//Returns the number converted to double or zero on error.
+inline double to_double(std::string_view s) noexcept { double d; return from_chars(s, d) ? 0 : d; }
+inline int to_int(std::string_view s) noexcept { int d; return from_chars(s, d) ? 0 : d; }
+
 ///////////////////////////////////////////////////////////////////////
 
 
@@ -242,7 +257,7 @@ public:
  * It destroys its objects at deletion.
  * It is almost as cheap as a simple list of pointers, fully owns
  * the objects added to it - and hence does not support copy operations.
- * It comes from "list" so iterators to it remain valid even across manipulatioos. */
+ * It comes from "list" so iterators to it remain valid even across manipulations. */
 template <class Object>
 class UPtrList : public std::list<std::unique_ptr<Object>>
 {
@@ -292,7 +307,7 @@ std::unique_ptr<UPtrList<Object>> Duplicate(const UPtrList<Object> &l)
  * It uses shared pointers, so others can point to these objects,
  * It is almost as cheap as a simple list of pointers, fully owns
  * the objects added to it - and hence does not support copy operations.
- * It comes from "list" so iterators to it remain valid even across manipulatioos. */
+ * It comes from "list" so iterators to it remain valid even across manipulations. */
 template <class Object>
 class SPtrList : public std::list<std::shared_ptr<Object>>
 {
@@ -422,7 +437,7 @@ enum class EAttrType {
 /** Describes the context of the use of a style or attribute.*/
 enum class EStyleType {
     STYLE,   ///<The attribute is to be applied to a style definition or the style is a user defined style
-    DEFAULT, ///<The attribute is to be applied to a default style or the style is a default stlye such as "box" or "arrow" and MUST be complete.
+    DEFAULT, ///<The attribute is to be applied to a default style or the style is a default style such as "box" or "arrow" and MUST be complete.
     DEF_ADD, ///<The style is an additional default style, such as "->" and does not have to be complete.
     ELEMENT, ///<The attribute is to be applied to an element or the style object is part of an element describing it
     OPTION   ///<The attribute is indeed a chart option
@@ -598,8 +613,8 @@ public:
      * the inner edge a somewhat smaller one).*/
     /** @{ */
 
-    /** Creates the midline of the rectange according to corner style.
-     * This one does not assume anything about wether the resulting rectange should be the
+    /** Creates the midline of the rectangle according to corner style.
+     * This one does not assume anything about whether the resulting rectangle should be the
      * outer edge or inner edge of the line - just uses the radius value and coordinates
      * as they are. (So if 'b' is the middle of the line, the result will also be at the
      * middle of the line.) Here we simply add round and bevel corners if applicable.
@@ -607,15 +622,15 @@ public:
      * You can specify a radius different from the one stored in us. Leaving that out
      * defaults to the radius stored in `this`*/
     Contour CreateRectangle_Midline(double x1, double x2, double y1, double y2, double r = -1) const;
-    /** Creates the midline of the rectange according to corner style.
-     * This one does not assume anything about wether the resulting rectange should be the
+    /** Creates the midline of the rectangle according to corner style.
+     * This one does not assume anything about whether the resulting rectangle should be the
      * outer edge or inner edge of the line - just uses the radius value and coordinates
      * as they are. (So if 'b' is the middle of the line, the result will also be at the
      * middle of the line.) Here we simply add round and bevel corners if applicable.
      * For ECornerType::NOTE it creates the outer line only.*/
     Contour CreateRectangle_Midline(const XY &s, const XY &d) const {return CreateRectangle_Midline(s.x, d.x, s.y, d.y);}
-    /** Creates the midline of the rectange according to corner style.
-     * This one does not assume anything about wether the resulting rectange should be the
+    /** Creates the midline of the rectangle according to corner style.
+     * This one does not assume anything about whether the resulting rectangle should be the
      * outer edge or inner edge of the line - just uses the radius value and coordinates
      * as they are. (So if 'b' is the middle of the line, the result will also be at the
      * middle of the line.) Here we simply add round and bevel corners if applicable.
@@ -712,7 +727,7 @@ public:
     OptAttr<ColorType> color;        ///<The color of the fill.
     OptAttr<ColorType> color2;       ///<The secondary color of the fill if we use a gradient.
     OptAttr<double> gradient_angle;  ///<The gradient angle of the fill. For radiant gradients, if negative we do 'in'.
-    OptAttr<EIntGradType> gradient;  ///<The type of gradiant to use
+    OptAttr<EIntGradType> gradient;  ///<The type of gradient to use
     OptAttr<double> frac;            ///<[0..1] the fraction of the first color from the gradient
     constexpr FillAttr() noexcept : color(ColorType(255,255,255)), color2(ColorType(255,255,255)), gradient_angle(-1), gradient(EIntGradType::NONE), frac(0.5) {}
     constexpr explicit FillAttr(ColorType c) noexcept : color(c) {} ///<Creates an empty style, where only the color is set to `c`.
@@ -731,8 +746,8 @@ public:
     constexpr bool IsComplete() const noexcept {return color && gradient_angle && gradient && frac;} ///<True if all of the line attributes are set. (But color2 is not needed.)
     FillAttr &operator +=(const FillAttr&a) noexcept; ///<Applies `a` to us: sets all our attributes, which are set in `a` to the value in `a`; leaves the rest unchanged.
     constexpr bool operator == (const FillAttr& a) const noexcept = default;
-    static constexpr FillAttr None() noexcept { return FillAttr(ColorType::none(), EGradientType::NONE); } ///<Creates no fill, fully specified 
-    static constexpr FillAttr Solid(ColorType c) noexcept { return FillAttr(c, EGradientType::NONE); }     ///<Creates a solid color fill, fully specified 
+    static constexpr FillAttr None() noexcept { return FillAttr(ColorType::none(), EGradientType::NONE); } ///<Creates no fill, fully specified
+    static constexpr FillAttr Solid(ColorType c) noexcept { return FillAttr(c, EGradientType::NONE); }     ///<Creates a solid color fill, fully specified
 
     virtual bool AddAttribute(const Attribute &a, Chart*chart, EStyleType t);
     static void AttributeNames(Csh &csh, std::string_view prefix);
@@ -753,8 +768,9 @@ public:
      * Black color, no offset (effectively no shadow) and no blur.*/
     constexpr ShadowAttr() noexcept = default;
     constexpr explicit ShadowAttr(ColorType c) noexcept : color{c}, offset{}, blur{} {} ///<Creates an empty style, where only the color is set to `c`.
-    constexpr bool IsEmpty() const noexcept { _ASSERT(IsComplete()); return !color || color->IsFullyTransparent() || offset==0; }
+    constexpr bool IsNone() const noexcept { _ASSERT(IsComplete()); return !color || color->IsFullyTransparent() || offset==0; }
     void Empty() noexcept { color.reset();  offset.reset();  blur.reset(); } ///<Clear all content from the shadow style.
+    bool IsEmpty() const noexcept { return !color && !offset && !blur; }  ///<Returns true if none of the elements are set
     void MakeComplete() noexcept;
     constexpr bool IsComplete() const noexcept {return color && offset && blur;} ///<True if all of the shadow attributes are set.
     ShadowAttr &operator +=(const ShadowAttr&a) noexcept; ///<Applies `a` to us: sets all our attributes, which are set in `a` to the value in `a`; leaves the rest unchanged.
@@ -791,7 +807,7 @@ enum class EArrowType {
     LINE,         ///<Line: just two lines
     HALF,    ///<Line: just one line
     DIAMOND,      ///<A filled diamond shape
-    DIAMOND_EMPTY,///<An emmpty diamond shape
+    DIAMOND_EMPTY,///<An empty diamond shape
     DOT,          ///<A filled circle
     DOT_EMPTY,    ///<An empty circle
     SHARP,        ///<A filled triangle with miters
@@ -800,7 +816,7 @@ enum class EArrowType {
     VEE,          ///<Like a '<' sign (from graphviz)
     VEE_EMPTY,    ///<Like a '<' sign, but empty inside (from graphviz)
     NSDIAMOND,    ///<A filled diamond shape, that is not symmetric
-    NSDIAMOND_EMPTY,///<An emmpty diamond shape, that is not symmetric
+    NSDIAMOND_EMPTY,///<An empty diamond shape, that is not symmetric
     NSDOT,        ///<A filled circle, that is not symmetric
     NSDOT_EMPTY,  ///<An empty circle, that is not symmetric
     JUMPOVER,     ///<Line only: jump over line: used for skip
@@ -813,7 +829,7 @@ enum class EArrowType {
     TRIPLE_LINE,  ///<Line only: Three set of lines
     TRIPLE_HALF,  ///<Line only: Three single lines
     //Block arrows only
-    EMPTY_INV,    ///<Block only: Inward trinagle as wide as body
+    EMPTY_INV,    ///<Block only: Inward triangle as wide as body
     STRIPES,      ///<Block only: Three rectangles of increasing size
     TRIANGLE_STRIPES ///<Block only: Three triangles of increasing size
 };
@@ -830,7 +846,7 @@ constexpr bool IsDouble(EArrowType t) noexcept {return t>=EArrowType::DOUBLE &&
 constexpr bool IsTriple(EArrowType t) noexcept {return t>=EArrowType::TRIPLE && t<=EArrowType::TRIPLE_HALF;}
 /** Returns 2 for double, 3 for triple and 1 for single types.*/
 constexpr unsigned Duplicity(EArrowType t) noexcept { return IsTriple(t) ? 3 : IsDouble(t) ? 2 : 1; }
-/** Returns the simple varian of double and triple arrows or the argument if not such.*/
+/** Returns the simple variant of double and triple arrows or the argument if not such.*/
 constexpr EArrowType SingleVariant(EArrowType t) noexcept { return IsTriple(t) ? EArrowType(unsigned(t)+unsigned(EArrowType::SOLID)-unsigned(EArrowType::TRIPLE)) : IsDouble(t) ? EArrowType(unsigned(t)+unsigned(EArrowType::SOLID)-unsigned(EArrowType::DOUBLE)) : t; }
 /** True if type is a single line sort (and is good for arrows)*/
 constexpr bool IsHalf(EArrowType t) noexcept {return t==EArrowType::HALF || t==EArrowType::DOUBLE_HALF || t==EArrowType::TRIPLE_HALF;}
@@ -930,8 +946,8 @@ protected:
     EArcArrowType       type;     ///<What kind of arrows do we apply to
     LineAttr            line;     ///<The line style for the arrowhead
     OptAttr<EArrowSize> size;     ///<The size of the arrowhead. Defaults to SMALL.
-    OptAttr<double>     xmul;     ///<The x-size multipler of the arrowhead. Defaults to 1.
-    OptAttr<double>     ymul;     ///<The y-size multipler of the arrowhead. Defaults to 1.
+    OptAttr<double>     xmul;     ///<The x-size multiplier of the arrowhead. Defaults to 1.
+    OptAttr<double>     ymul;     ///<The y-size multiplier of the arrowhead. Defaults to 1.
     OptAttr<EArrowType> endType;  ///<The type of the arrowhead at the end of the arrow (also start for bidirectional arrows).
     OptAttr<std::string>gvEndType;///<The type of the arrowhead expressed as a gv style string
 
@@ -967,6 +983,8 @@ public:
     void SetSize(EArrowSize s) { size = s; dirty = true; }
     /** Deletes all content, clears all attributes. */
     void Empty();
+    /** Returns true if no attribute is set. */
+    bool IsEmpty() const noexcept { return line.IsEmpty() && !size && !endType && !gvEndType && !xmul && !ymul; }
     /** True if all attributes are set*/
     bool IsComplete() const { return line.IsComplete() && size && (endType || gvEndType) && xmul && ymul; }
     /** Set attributes that are not yet set (use default values)*/
@@ -1015,6 +1033,8 @@ public:
     bool SetGvType(EArrowEnd which, const OptAttr<std::string> &t) { auto a = GetArrowAttr(which); if (a.second) *a.second = t; if (a.first && t) a.first->reset(); dirty = true; return a.second!=nullptr; }
     /** Deletes all content, by clearing all attributes. */
     void Empty();
+    /** Returns true if no attribute is set. */
+    bool IsEmpty() const noexcept { return !startType && !gvStartType && OneArrowAttr::IsEmpty(); }
     /** True if all attributes are set*/
     bool IsComplete() const { return (startType || gvStartType) && OneArrowAttr::IsComplete(); }
     /** Set attributes that are not yet set (use default values)*/
@@ -1066,6 +1086,8 @@ public:
     /** Read the graphviz arrow type attribute (and resets the old one if t is true). Returns false, if we do not have that kind of arrow end. */
     bool SetGvType(EArrowEnd which, const OptAttr<std::string> &t) { auto a = GetArrowAttr(which); if (a.second) *a.second = t; if (a.first && t) a.first->reset(); dirty = true; return a.second!=nullptr; }
     void Empty();
+    /** Returns true if no attribute is set. */
+    bool IsEmpty() const noexcept { return !midType && !gvMidType && ((!skipType && !gvSkipType) || type == EArcArrowType::BIGARROW) && TwoArrowAttr::IsEmpty(); }
     bool IsComplete() const { return (midType || gvMidType) && (skipType || gvSkipType || type==EArcArrowType::BIGARROW) && TwoArrowAttr::IsComplete(); }
     void MakeComplete();
     FullArrowAttr & operator += (const FullArrowAttr &);
@@ -1083,11 +1105,12 @@ class WidthAttr
     mutable double value;
     std::string str;
 public:
-    WidthAttr() { Empty(); MakeComplete(); }
-    void Empty() { is_set = false; }
-    void MakeComplete() { if (!is_set) { is_set = true; value = 0; str.clear(); } }
+    WidthAttr() noexcept { Empty(); MakeComplete(); }
+    void Empty() noexcept { is_set = false; }
+    bool IsEmpty() const noexcept { return !is_set; }
+    void MakeComplete() noexcept { if (!is_set) { is_set = true; value = 0; str.clear(); } }
     WidthAttr &operator +=(const WidthAttr&a) { if (a.is_set) *this = a; return *this; }///<Applies `a` to us: sets all our attributes, which are set in `a` to the value in `a`; leaves the rest unchanged.
-    bool operator == (const WidthAttr &a) const;
+    bool operator == (const WidthAttr &a) const noexcept;
     bool AddAttribute(const Attribute &a, Chart* chart, EStyleType t);
     static bool AttributeValues(std::string_view attr, Csh &csh);
     double GetWidth(Canvas &canvas, const ShapeCollection &shapes, const StringFormat &format) const;
