@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -23,7 +23,6 @@
 
  //to kill sscanf errors in Visual Studio
 #define _CRT_SECURE_NO_WARNINGS 
-#define _CRT_NONSTDC_NO_DEPRECATE //For visual studio to allow strdup
 
 #include <sstream>
 #include "cgen_color.h"
@@ -87,7 +86,7 @@ string ColorType::Print(void) const
     ss<<(int)r<<","<<(int)g<<","<<(int)b;
 	if (a!=255) ss<<","<<int(a);
 	ss<<")";
-    return ss.str();
+    return std::move(ss).str();
 }
 
 ColorType ColorType::operator +(const ColorType &o) const noexcept
@@ -128,8 +127,7 @@ bool ColorSet::AddColor(std::string_view alias, std::string_view colordef,
         this->operator[](std::string(alias)) = c;
         return true;
     }
-    std::string ss = "Invalid color definition for ";
-    ss << alias << ": '" << colordef << "'. Ignoring color alias definition.";
+    std::string ss = StrCat("Invalid color definition for ", alias, ": '", colordef, "'. Ignoring color alias definition.");
     error.Error(linenum.start, ss);
     return false;
 }
@@ -154,7 +152,7 @@ inline std::string_view remove_spaces(std::string_view s)
  * 4. a color name, followed by a comma and an alpha value,  followed by {+-} and a percentage of lightness
  * 5. three int value separated by commas (rgb)
  * 6. four int values separated by commas (rgba)
- * Either can be preceeded by '++' to indicate a color to overlay.
+ * Either can be preceded by '++' to indicate a color to overlay.
  *
  * In the first 4 case we consult the collection in the last case we return
  * a value irrespective of the collection content.
@@ -187,7 +185,7 @@ ColorType ColorSet::GetColor(std::string_view s_original) const
             ret.type = type;
         return ret;
     }
-    //if first element in collection, see if followidered by an alpha value and/or lightness
+    //if first element in collection, see if followed by an alpha value and/or lightness
     ColorType color = i->second;
     color.type = type;
     double num;
