@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -20,7 +20,6 @@
  * @ingroup libcgencommon_files  */
 
 #define _CRT_SECURE_NO_DEPRECATE //For visual studio to allow sscanf
-#define _CRT_NONSTDC_NO_DEPRECATE //For visual studio to allow strdup
 #include "cgen_shapes.h"
 
 const char ShapeElement::act_code[] = "LMCEHTSSSP";
@@ -157,7 +156,7 @@ void Shape::Add(ShapeElement &&e)
         current_section_point = XY(e.x, e.y);
         break;
     case ShapeElement::MOVE_TO:
-        ProcessSubPath(false); //If we already have data, this is an open segment, dont close
+        ProcessSubPath(false); //If we already have data, this is an open segment, don't close
         max += XY(e.x, e.y);
         current_section_point = XY(e.x, e.y);
         break;
@@ -191,7 +190,7 @@ Block Shape::GetLabelPos(const Block &o) const noexcept
 }
 
 /** Returns the coordinates of a port if the shape is  drawn into box 'o'.
- * If the port name is not found, we dont return a value.*/
+ * If the port name is not found, we don't return a value.*/
 std::optional<Shape::Port> Shape::GetPort(const std::string &port, const Block & o) const
 {
     auto i = ports.find(port);
@@ -391,7 +390,7 @@ bool ShapeCollection::Add(std::string_view shape_text, MscError &Error)
                 case '1': shapes.back().Add(ShapeElement(ShapeElement::SECTION_FG_FILL, 0, 0, 0, 0, 0, 0)); break;
                 case '2': shapes.back().Add(ShapeElement(ShapeElement::SECTION_FG_LINE, 0, 0, 0, 0, 0, 0)); break;
                 } else
-                Error.Error(fpos, "'S' should be followed by a space and an intener between 0 and 2. Ignoring this line.");
+                Error.Error(fpos, "'S' should be followed by a space and an integer between 0 and 2. Ignoring this line.");
                 break;
         case 'T':
             if (block) break;
