@@ -1,6 +1,6 @@
 /*
 This file is part of Msc-generator.
-Copyright (C) 2008-2022 Zoltan Turanyi
+Copyright (C) 2008-2023 Zoltan Turanyi
 Distributed under GNU Affero General Public License.
 
 Msc-generator is free software: you can redistribute it and/or modify
@@ -23,8 +23,6 @@ along with Msc-generator.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef CGENCOMMON_H
 #define CGENCOMMON_H
 
-#define _CRT_NONSTDC_NO_DEPRECATE //For visual studio to allow strdup
-
 #include <string>
 #include "csh.h"
 #include "chartbase.h"
@@ -48,7 +46,7 @@ public:
     //Contract is that e, c and f are not empty
     LanguageData(std::vector<std::string> e, std::string_view desc, std::string_view ent, std::string_view d,
                  std::unique_ptr<Csh> &&c, bool ah, bool ec, Chart::ChartFactoryFunction f):
-        description(desc), extensions(e), 
+        description(desc), extensions(e),
         has_autoheading(ah), has_element_controls(ec), entityname(ent),
         default_text(d), pCsh(std::move(c)), create_chart(f)
         { pCsh->pShapes = &shapes; }
@@ -76,9 +74,9 @@ public:
  * coloring parse pass.
  * Charts are supposed to override a set of abstract virtual functions of class Chart,
  * which provide more information about this language (description, name, file extensions,
- * etc.). 
+ * etc.).
  * There is also the concept of 'design libraries', which are definitions in a language
- * defined in one file - but usable in all charts of that language. They typically 
+ * defined in one file - but usable in all charts of that language. They typically
  * contain chart designs and shapes. Every chart has a function to export the collected
  * designs and shapes so far (along with the errors happened during their parsing). These
  * functions are used to fill in LanguageData::shapes, LanguageData::designs and
@@ -89,23 +87,25 @@ public:
 class LanguageCollection
 {
 protected:
-    std::list<LanguageData> languages; ///<Stores our supported languages.
-public:
-    /** The csh parameter set used by all languages in this collection */
-    std::shared_ptr<CshParams> cshParams = std::make_shared<CshParams>();  
+    std::list<LanguageData> languages; ///<Stores our supported languages. Must be a list as LanguageData is neither copyable, nor movable.
     /** Libraries used in the system (with version number), mapping to their license text. */
     std::map<std::string, std::string> libraries;
+public:
+    /** The csh parameter set used by all languages in this collection */
+    std::shared_ptr<CshParams> cshParams = std::make_shared<CshParams>();
     /** Adds a language. Returns false, if the language already added.*/
-    bool AddLanguage(Chart::ChartFactoryFunction f, Csh::FileListProc proc, void *param);
+    bool AddLanguage(Chart::ChartFactoryFunction f, Csh::FileListProc proc);
     const LanguageData *GetLanguage(std::string_view language) const noexcept { auto i = std::find_if(languages.begin(), languages.end(), [language](const auto &l) {return l.GetName() == language; }); return i==languages.end() ? nullptr : &*i; } ///<Get the Language object based on language name.
     LanguageData *GetLanguageToModify(std::string_view language) noexcept { auto i = std::find_if(languages.begin(), languages.end(), [language](const auto &l) {return l.GetName() == language; }); return i==languages.end() ? nullptr : &*i; } ///<Get the Language object based on language name.
     const LanguageData* GetLanguageByExtension(std::string_view e) const noexcept { for (auto& l : languages) if (l.HasExtension(e)) return &l; return nullptr; } ///<Return the name of the language handling this extension. "" if none.
     const LanguageData *GetLanguageByDescr(std::string_view d) const noexcept { auto i = std::find_if(languages.begin(), languages.end(), [d](const auto &l) {return l.description == d; }); return i==languages.end() ? nullptr : &*i; } ///<Get the Language object based on language's short description.
     std::string LanguageForExtension(std::string_view e) const { for (auto &l : languages) if (l.HasExtension(e)) return l.GetName();  return std::string(); } ///<Return the name of the language handling this extension. "" if none.
-    std::vector<std::string> GetLanguages() const { std::vector<std::string> ret; for (auto &l : languages) ret.push_back(l.GetName()); return ret; } ///<Get a list of supported languages.
-    std::vector<std::string> GetExtensions() const { { std::vector<std::string> ret; for (auto &l : languages) ret.insert(ret.end(), l.extensions.begin(), l.extensions.end()); return ret; } } ///<Get a list of supported extensions.
-    std::string GetLibraries() const { std::string libs; for (auto &[s,_] : libraries) libs += s + ", "; libs.pop_back(); libs.pop_back(); return libs; } ///<Provides a comma separated list of the libraries used.
-    MscError CompileDesignLibs(const std::vector<std::pair<std::string, std::string>> &design_files);
+    std::vector<std::string_view> GetLanguages() const { std::vector<std::string_view> ret; for (auto &l : languages) ret.push_back(l.GetName()); return ret; } ///<Get a list of supported languages.
+    std::vector<std::string_view> GetExtensions() const { { std::vector<std::string_view> ret; for (auto &l : languages) ret.insert(ret.end(), l.extensions.begin(), l.extensions.end()); return ret; } } ///<Get a list of supported extensions.
+    void AddLibraries(std::map<std::string, std::string>&& l) { for (auto& [name, license] : l) libraries.try_emplace(std::move(name), std::move(license)); }
+    const std::map<std::string, std::string>& GetLibraries() const noexcept { return libraries; }
+    std::string GetLibraryNames() const { std::string libs; for (auto &[s,_] : libraries) libs += s + ", "; libs.pop_back(); libs.pop_back(); return libs; } ///<Provides a comma separated list of the libraries used.
+    MscError CompileDesignLibs(std::vector<std::pair<std::string, std::string>> &&design_files);
 };
 
 
