@@ -273,8 +273,8 @@ Chart::ParseCompassPoint(std::string_view n, const FileLineCol &l,
         if (CaseInsensitiveEqual(n, compass_points[u]))
             return compass_point_degree[u];
     if (report_error)
-        Error.Error(l, StrCat("Did not recognize '", n, "' as a compass point. Ignoring it.",
-                                    "Use one of 'nw', 'n', 'ne', 'e', 'se', 's', 'sw', 'w', 'c' or 'perp'."));
+        Error.Error(l, StrCat("Did not recognize '", n, "' as a compass point. Ignoring it."),
+                    "Use one of 'nw', 'n', 'ne', 'e', 'se', 's', 'sw', 'w', 'c' or 'perp'.");
     return {};
 }
 
@@ -300,6 +300,8 @@ Chart::ParseCompassPoint(std::string_view n, const FileLineCol &l,
 *                               ignoring page breaks.
 * @param [in] include_chart_text If supplied and the image is a png image, we place the chart text
 *                                into an iTXt chunk.
+* @param [in] only_page If nonzero, we only print this page into the file specified by 'fn' (no
+*                       page number added). In this case 'ignore_pagebreaks' is ignored.
 * @param [in] pageSize If non-zero, a fixed page size is used and a single multi-page
 *                      file will be created (only with PDF).
 * @param [in] margins For fixed size pages, these are the margins
@@ -310,23 +312,23 @@ Chart::ParseCompassPoint(std::string_view n, const FileLineCol &l,
 * @returns False on error (but not on warning) irrespective of `generateErrors`. */
 bool Chart::DrawToFile(Canvas::EOutputType ot, const XY &scale,
                        std::string_view fn, bool bPageBreak, bool ignore_pagebreaks,
-                       const char *include_chart_text,
+                       const char *include_chart_text, unsigned only_page,
                        const XY &pageSize, const double margins[4],
                        int ha, int va, bool generateErrors)
 {
     const PBDataVector pageBreakData = GetPageVector();
-    const unsigned from = pageBreakData.size()<=1 || ignore_pagebreaks ? 0 : 1;
-    const unsigned till = pageBreakData.size()<=1 || ignore_pagebreaks ? 0 : unsigned(pageBreakData.size());
+    const unsigned from = only_page ? only_page : pageBreakData.size()<=1 || ignore_pagebreaks ? 0 : 1;
+    const unsigned till = only_page ? only_page : pageBreakData.size()<=1 || ignore_pagebreaks ? 0 : unsigned(pageBreakData.size());
     if (pageSize.x<=0 || pageSize.y<0)
         for (unsigned page = from; page<=till; page++) {
+            std::string fileName(fn);
+            if (page > 0 && only_page == 0) {  //append page number if not all pages written
+                char num[3] = "00";
+                num[0] = char('0' + page / 10);
+                num[1] = char('0' + page % 10);
+                fileName.insert(fn.find_last_of('.'), num);
+            }
             if (ot==Canvas::ISMAP) {
-                std::string fileName(fn);
-                if (page>0) {  //append page number if not all pages written
-                    char num[3] = "00";
-                    num[0] = char('0'+page/10);
-                    num[1] = char('0'+page%10);
-                    fileName.insert(fn.find_last_of('.'), num);
-                }
                 FILE *fout = fopen(fileName.c_str(), "wt");
                 if (fout==nullptr) {
                     Error.FatalError(FileLineCol(0, 0), "Could not open output file '" + fileName + "'.");
@@ -343,7 +345,7 @@ bool Chart::DrawToFile(Canvas::EOutputType ot, const XY &scale,
                 }
                 fclose(fout);
             } else {
-                Canvas canvas(ot, total, copyrightTextHeight, fn, scale, &pageBreakData, page);
+                Canvas canvas(ot, total, copyrightTextHeight, fileName, scale, &pageBreakData, page);
                 if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData)) return false;
                 if (include_chart_text)
                     canvas.AddiTXtChunk(EmbedChartData{ .chart_type = GetLanguageExtensions().front(), .chart_text = include_chart_text,
@@ -534,10 +536,8 @@ Chart::Include(std::string_view filename, const FileLineCol &linenum_command)
 }
 
 XY Chart::GetCanvasSize(unsigned page) const noexcept {
-    XY origSize, origOffset;
-    PBDataVector pages = GetPageVector();
-    const PageBreakData* pbData;
-    Canvas::GetPagePosition(total, &pages, page, origOffset, origSize, &pbData);
+    const PBDataVector pages = GetPageVector();
+    auto [origSize, origOffset, pbData] = Canvas::GetPagePosition(total, &pages, page);
     XY size = origSize + (pbData ? pbData->headingLeftingSize : XY(0, 0));
     size.y += copyrightTextHeight;
     return size;
