@@ -373,9 +373,9 @@ public:
     * @param [in] page The page to draw. Zero means the whole chart. */
     virtual void DrawComplete(Canvas &canvas, bool pageBreaks, unsigned page) = 0;
 
-    virtual bool DrawToFile(Canvas::EOutputType, const XY &scale,
+    virtual bool DrawToFile(Canvas::EOutputType, const XY& scale,
                             std::string_view fn, bool bPageBreak, bool ignore_pagebreaks,
-                            const char *include_chart_text = nullptr,
+                            const char* include_chart_text = nullptr, unsigned only_page = 0,
                             const XY &pageSize=XY(0,0), const double margins[4]=nullptr,
                             int ha=-1, int va=-1, bool generateErrors=false);
 #ifdef CAIRO_HAS_WIN32_SURFACE
