@@ -448,6 +448,10 @@ static void usage()
 " -x=<width>  Specifies chart width (in pixels). Only for bitmap output.\n"
 " -y=<height> Specifies chart height (in pixels). If only one of -x or -y\n"
 "             is specified, the aspect ratio is kept. Only for bitmap output.\n"
+" --fps=<fps> Can be used to select the maximum frames per second for the GUI.\n"
+"             Silently ignored when not using the GUI. Ignored on Windows, as\n"
+"             there frame presentation is synchronized to the refresh of the\n"
+"             screen maxing the FPS to the screen FPS.\n"
 "\n"
 "Signalling chart specific options:\n"
 " --force-mscgen\n"
@@ -473,6 +477,10 @@ static void usage()
 "  MSCGEN_NOFONT: When set (to any value), Msc-generator draws no text to any\n"
 "                 output (but considers the place of the text at layout as\n"
 "                 normally. Useful for testing.\n"
+"  MSCGEN_FPS: When set to a number, the GUI will use it as maximum frames per\n"
+"              second. Can be overridden by the --fps option. Ignored on\n"
+"              Windows, as there frame presentation is synchronized to the\n"
+"              refresh of the screen maxing the FPS to the screen FPS.\n"
 "\n");
 }
 
@@ -583,6 +591,7 @@ do_main(const std::vector<std::string> &args,
     std::optional<bool>     oPedantic;
     OUnsignedPair           oEmbed;   //If the user says -T embed:slide:chart, these will contain 'slide' and 'chart' (both default to 0 as in 'insert')
     OUnsignedPair           oExtract; //If the user specified a filename ending in ":<num>:<num> this contains the two numbers.
+    double                  oFPS = 60;
 
 
     if (const char* const font1 = getenv("MSCGEN_FONT")) oFont = font1;
@@ -596,6 +605,12 @@ do_main(const std::vector<std::string> &args,
     bool show_usage = false;
     std::vector<std::string_view> tmp_langs;
 
+    if (const char* const fps = getenv("MSCGEN_FPS"))
+        if (from_chars(fps, oFPS) || oFPS < 1 || 240 < oFPS) {
+            Error.Error(opt_pos, StrCat("The MSCGEN_FPS environment variable is supposed be a number between [1..240] and not '", fps, "'. Ignoring it."));
+            oFPS = 60;
+        }
+
     for (auto i=args.begin(); i!=args.end(); i++) {
         if (i->length()==0) continue;
         else if (i->length()>=2 && i->at(0) == '-' && i->at(1) == 'x') {
@@ -906,6 +921,17 @@ do_main(const std::vector<std::string> &args,
             oChartOptions.push_back(i->substr(6));
         } else if (i->starts_with("--design=")) {
             oForcedDesign = i->substr(9);
+        } else if (i->starts_with("--fps")) {
+            if (i->length()<6 || i->at(5)!='=')
+                Error.Error(opt_pos, "Missing '=' after '--fps'. (Apply no spaces.)");
+            else if (i->length()==6)
+                Error.Error(opt_pos, "Missing number after '--fps='. (Apply no spaces.)");
+            else if (from_chars(i->substr(6), oFPS))
+                Error.Error(opt_pos, StrCat("I need a number after '--fps=' and not '", i->substr(6), "'."));
+            else if (oFPS < 1 || 240 < oFPS) {
+                Error.Error(opt_pos, StrCat("FPS should be a number between [1..240] and not '", i->substr(6), "'. Using max FPS of 60."));
+                oFPS = 60;
+            }
         } else if (i->length()>=2 && i->at(0) == '-' &&
                    (tmp_langs = IsValidChartSpecificCommandLineArg(languages, *i)).size()) { //may be a chart-specific attribute, test if we accept it
             oChartArguments[std::move(*i)] = std::move(tmp_langs);
@@ -1301,7 +1327,7 @@ do_main(const std::vector<std::string> &args,
         }
         MscInitializeCshAppearanceList();
         return GUIInit{std::move(languages), file_read_proc, file_read_proc_param,
-                       OpenNamedFile, CopyrightText, oInputFile, oInputFile, type, load_data};
+                       OpenNamedFile, CopyrightText, oInputFile, oInputFile, type, load_data, oFPS};
     }
 
     _ASSERT(type.length());
@@ -1397,6 +1423,8 @@ do_main(const std::vector<std::string> &args,
             oOutputFile.append(".svg"); break;
         case Canvas::EMF:
             oOutputFile.append(".emf"); break;
+        case Canvas::WMF:
+            oOutputFile.append(".wmf"); break;
         case Canvas::ISMAP:
             oOutputFile.append(".map"); break;
         case Canvas::PPT:
@@ -1871,7 +1899,7 @@ do_main(const std::vector<std::string> &args,
                 const bool ignore_pagebreaks = oEmbed.has_value();
                 const bool show_pagebreaks = oEmbed.has_value();
                 chart->DrawToFile(oOutType, scale_to_use, oOutputFile, show_pagebreaks, ignore_pagebreaks,
-                                  oIncludeChartText ? input.c_str() : nullptr,
+                                  oIncludeChartText ? input.c_str() : nullptr, 0,
                                   GetPhysicalPageSize(oPageSize.value_or(EPageSize::NO_PAGE)),
                                   margins, oHA, oVA, true);
             }
