@@ -22,7 +22,7 @@
  *
  * The contour library contains functionality for handling 2D vector shapes. Intersection,
  * union, expansion, area/circumference calculation, distance computation are the key
- * fetures.
+ * features.
  *
  * What is a *contour*?
  * --------------------
@@ -35,7 +35,7 @@
  * disjoint parts (even inside a hole). We always try to normalize contours
  * such that their edges do not cross any other edge.
  * Edges are ordered in the contour - *clockwise* for positive contour outlines,
- * while counerclockwise for outlines representing holes. Clockwiseness is defined
+ * while counterclockwise for outlines representing holes. Clockwiseness is defined
  * such that the y coordinate grows _downward_. This convention is used everywhere
  * as throughout the comments.
  *
@@ -43,14 +43,14 @@
  * --------
  *
  * - The main feature of the library is to be able to create union and intersection
- * of contours; substraction and xor operation is also possible.
+ * of contours; subtraction and xor operation is also possible.
  * - A number of auxiliary operations
  * are defined, such as area, centroid location and circumference length calculation,
  * tangent fitting and various cuts.
  * - Naturally linear transformations (shift, rotate, scale, etc.) are supported too.
- * - The library uses the C++11 move schemantics extensively.
- * - The library can *expand* a contory by a given pixels, that is generate
- * a contour that is this many pixels wider (or narrrower for negative values),
+ * - The library uses the C++11 move schematics extensively.
+ * - The library can *expand* a contour by a given pixels, that is generate
+ * a contour that is this many pixels wider (or narrower for negative values),
  * such that the resulting contour is sane - its edges do not cross each other.
  * (In general this is called curve offsetting, but for closed curves this really
  * becomes expansion or shrinking.)
@@ -59,8 +59,8 @@
  * in between at any given place. In other words, placing a 2*x - 2*y rectangle
  * centered anywhere outside the expanded shape will not overlap with the original.
  * - The distance of two contours (along with the closest points) can be determined.
- * - Especially important for layout tipography, the library can determine, how much a
- * one of two shapes has to be moved up or down to touch anothe shape.
+ * - Especially important for layout typography, the library can determine, how much a
+ * one of two shapes has to be moved up or down to touch another shape.
  *
  * See the definition of the Contour class for a detailed description of the
  * features.
@@ -88,7 +88,7 @@
  *   We calculate all crosspoints and start walking around one of the contours. During the walk
  *   we stop by each crosspoint and see if we have to switch to the other contour. Finally we get
  *   back to the staring point. Then we can start an additional walk if relevant crosspoints have
- *   not been visited - in which case the resulting contour will not be contigous, that is it will
+ *   not been visited - in which case the resulting contour will not be contiguous, that is it will
  *   contain multiple disjoint shapes. (Some of which may turn out to be holes inside other
  *   shapes.)
  * - A *tangent* for our purposes is a line that touches a shape (or edge) only at a single point
@@ -105,7 +105,7 @@
  *       lien on the straight edge.
  *     - If we take a tangent at a vertex, which is not smooth (has an angle), there are actually two
  *       different tangent lines, one for each edge. In this case the forward and backward tangent points
- *       are returned form the edge subsequent or preceeding to the vertex, respectively.
+ *       are returned form the edge subsequent or preceding to the vertex, respectively.
  * - *Untangling* means the process of creating a shape, whose edges do not cross each other. Normally
  *   contours in class Contour have no edges crossing. There are two rules of what part of a tangled
  *   list of edges shall we keep during untangling, called winding rules.
@@ -325,6 +325,7 @@ struct XY {
     constexpr double &operator[](unsigned i) noexcept { return i ? y : x; }
     constexpr double &operator[](bool i) noexcept { return i ? y : x; }
     std::string Dump(bool precise) const;
+    bool IsSane() const noexcept { return !std::isnan(x) && !std::isnan(y); } //we consider infinite and subnormal values OK
 };
 
 constexpr XY operator*(double a, const XY &xy) noexcept {return xy*a;} ///< Return a scaled version of the point `xy`. Enables writing the scalar before the point.
@@ -359,8 +360,11 @@ struct Range {
     constexpr Range(double s, double d) noexcept : from(s), till(d) {}
     void MakeInvalid() noexcept {from = MaxVal(from); till = -MaxVal(till);}
     constexpr bool IsInvalid() const noexcept {return from == CONTOUR_INFINITY && till == -CONTOUR_INFINITY;}
-    constexpr bool Overlaps(const struct Range &r, double gap=0) const  noexcept  ///< True if the two ranges at least touch.
+    bool IsSane() const noexcept { return !std::isnan(from) && !std::isnan(till); } //we consider infinite and subnormal values OK
+    constexpr bool Overlaps(const struct Range &r, double gap) const  noexcept  ///< True if the two ranges at least touch.
         {return from<=r.till+gap && r.from <= till+gap;}
+    constexpr bool Overlaps(const struct Range &r) const  noexcept  ///< True if the two ranges at least touch.
+        {return from<=r.till && r.from <= till;}
     constexpr bool HasZeroOverlap(const struct Range& r) const  noexcept  ///< True if the two ranges at most touch, but have zero intersection
         { return from>=r.till || r.from>=till; }
     Range &operator+=(double a) noexcept
@@ -448,9 +452,10 @@ struct Block {
     constexpr Block(const Block &) noexcept = default;
     void MakeInvalid() noexcept {x.MakeInvalid(); y.MakeInvalid();}           ///< Empty the Block.
     constexpr bool IsInvalid() const noexcept {return x.IsInvalid() || y.IsInvalid();}  ///< Check if Block is empty or not.
+    bool IsSane() const noexcept { return x.IsSane() && y.IsSane(); } //we consider infinite and subnormal values OK
     /** Comparison operator required for set ordering.
      *
-     * Blocks whoose y coordinate is larger are counted as larger here.
+     * Blocks whose y coordinate is larger are counted as larger here.
      * Note that this definition improves performance when checking overlaps
      * between a large number of Blocks (lower blocks come later).
      */
@@ -461,8 +466,10 @@ struct Block {
                 y.from < b.y.from;
     }
     constexpr bool operator == (const struct Block &b) const noexcept {return x==b.x && y==b.y;}
-    constexpr bool Overlaps(const struct Block &b, double gap=0) const noexcept
+    constexpr bool Overlaps(const struct Block &b, double gap) const noexcept
         {return x.Overlaps(b.x, gap) && y.Overlaps(b.y, gap);} ///< Sees if the two blocks have at least `gap` vertical and horizontal distance between them.
+    constexpr bool Overlaps(const struct Block &b) const noexcept
+        {return x.Overlaps(b.x) && y.Overlaps(b.y);} ///< Sees if the two blocks have at least `gap` vertical and horizontal distance between them.
     constexpr bool HasZeroOverlap(const struct Block &b) const noexcept
         {return x.HasZeroOverlap(b.x) || y.HasZeroOverlap(b.y);} ///< True if the two blocks have an empty intersection. They may touch.
     constexpr XY UpperLeft(void) const noexcept
@@ -592,7 +599,7 @@ struct TRMatrix
     XY a0;
     TRMatrix() = default; ///<Uninitialized matrix
     constexpr TRMatrix(const XY &_ab, const XY &_a1, const XY & _a0) : ab(_ab), a1(_a1), a0(_a0) {} ///<Fully initialized matrix
-    constexpr TRMatrix(bool) : ab(0, 0), a1(1, 1), a0(0, 0) {} ///<Matrix initialized to the identity matrix
+    constexpr TRMatrix(bool) : ab{0, 0}, a1{1, 1}, a0{0, 0} {} ///<Matrix initialized to the identity matrix
     /**Return an identity matrix*/
     static constexpr TRMatrix CreateIdentity() { return TRMatrix(XY(0,0), XY(1,1), XY(0, 0)); }
     /** Return a matrix rotating 'x' degrees around the origin.
