@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -72,7 +72,7 @@ void contour::debug::DebugCanvas::DrawLabel(std::string s, const XY &xy, unsigne
     l.Draw(*canvas, shapes, xy.x, xy.x, xy.y);
 }
 
-void contour::debug::DebugCanvas::Draw(const Edge & e, bool cont1, bool cont2, std::string s, std::string en, unsigned n)
+void contour::debug::DebugCanvas::Draw(const Edge & e, bool, bool cont2, std::string s, std::string en, unsigned n)
 {
     canvas->Line(e, line);
     DrawX(e.GetStart());
