@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -52,7 +52,7 @@ class DebugCanvas
         std::ostringstream out;
         std::fixed(out);
         out << std::setprecision(n) << a_value;
-        return out.str();
+        return std::move(out).str();
     }
 
 public:
@@ -220,22 +220,22 @@ template <>
 inline Block GetBB(const XY &xy) { return Block(xy, xy); }
 
 template <>
-inline Block GetBB(const ColorType &c) { return Block(false); } //color
+inline Block GetBB(const ColorType&) { return Block(false); } //color
 
 template <>
-inline Block GetBB(const std::string_view& s) { return Block(false); } //vertex or edge text
+inline Block GetBB(const std::string_view&) { return Block(false); } //vertex or edge text
 
 template <>
-inline Block GetBB(const std::string& s) { return Block(false); } //vertex or edge text
+inline Block GetBB(const std::string&) { return Block(false); } //vertex or edge text
 
 template <>
-inline Block GetBB(const char * const &s) { return Block(false); } //vertex or edge text
+inline Block GetBB(const char * const&) { return Block(false); } //vertex or edge text
 
 template <>
-inline Block GetBB(const double &d) { return Block(false); } //line/marker size width
+inline Block GetBB(const double&) { return Block(false); } //line/marker size width
 
 template <>
-inline Block GetBB(const EC &e) { return Block(false); } //command
+inline Block GetBB(const EC&) { return Block(false); } //command
 
 constexpr bool HasTitle(const EC& e) { return e == EC::TITLE; }
 constexpr bool HasTitle(const char*) { return false; }
