@@ -1094,12 +1094,11 @@ bool SimpleContour::IsConcaveJoin(size_t u) const
     //test if we cross each other
     XY r[9];
     double pos_my[9], pos_other[9];
-    bool overlap=false;
-    unsigned num = at(u).Crossing(at_next(u), true, r, pos_my, pos_other, &overlap);
-    if (overlap) return true; //overlapping edges are concave by default.
+    const int num = at(u).CrossingWithOverlap(at_next(u), true, r, pos_my, pos_other); //num<0 if the two edges overlap
+    if (num<0) return true; //overlapping edges are concave by default.
     if (num) {
         //if we have crosspoints, take the one closes to the vertex, halven the positions
-        double * pm = std::max_element(pos_my, pos_my+num);
+        const double * pm = std::max_element(pos_my, pos_my+abs(num));
         p1 = at(u).Pos2Point(*pm/2+0.5);
         p2 = at_next(u).Pos2Point(pos_other[pm-pos_my]/2);
     } else {
@@ -1503,19 +1502,20 @@ std::vector<Path> SimpleContour::SplitAt(std::vector<PathPos> pos, bool remove_e
 {
     std::vector<Path> ret;
     if (pos.size()<2) return ret;
-    for (auto &p : pos)
-        if (p.pos == 1) { p.pos = 0; p.edge = (p.edge+size()+1)%size(); }
+    for (auto& p : pos)
+        if (p.pos == 1)
+            p = {.edge = next(p.edge), .pos = 0};
     std::sort(pos.begin(), pos.end());
     ret.reserve(pos.size());
     for (unsigned u = 0; u<pos.size(); u++) {
         ret.emplace_back();
-        const unsigned uu = (u+1)%pos.size();
+        const unsigned uu = pos.size()<=(u+1) ? 0 : u+1;
         if (pos[u].edge==pos[uu].edge) {
             ret.back().emplace_back(at(pos[u].edge), pos[u].pos, pos[uu].pos);
         } else {
             if (pos[u].pos<1)
                 ret.back().emplace_back(at(pos[u].edge), pos[u].pos, 1);
-            for (size_t v = (pos[u].edge+1)%size(); v!=pos[uu].edge; v = (v+1)%size())
+            for (size_t v = next(pos[u].edge); v!=pos[uu].edge; v = next(v))
                 ret.back().push_back(at(v));
             if (pos[uu].pos>0)
                 ret.back().emplace_back(at(pos[uu].edge), 0, pos[uu].pos);
