@@ -243,9 +243,9 @@ public:
     const Edge *operator[](const XY &p) const noexcept {size_t edge; const EPointRelationType r = IsWithin(p, &edge); return r==WI_ON_EDGE||r==WI_ON_VERTEX ? &edges[edge] : nullptr;} ///<Returns const pointer to edge `p` is at, nullptr if `p` is not on the contour.
     size_t      next(size_t vertex) const noexcept { return edges.next(vertex); }         ///<Returns index of subsequent edge, warps around to 0 at last one.
     size_t      prev(size_t vertex) const noexcept { return edges.prev(vertex); }  ///<Returns index of preceeding edge, warps around at 0 to last edge.
-    const Edge &at(size_t i) const { return edges.at(i); }      ///<Returns reference to edge at index `i`.
-    const Edge &at_next(size_t i) const { return edges.at_next(i); } ///<Returns reference to edge after index `i`. Wraps around at last edge.
-    const Edge &at_prev(size_t i) const { return edges.at_prev(i); } ///<Returns reference to edge before index `i`. Wraps around at first edge.
+    const Edge &at(size_t i) const noexcept { return edges[i]; }      ///<Returns reference to edge at index `i`.
+    const Edge &at_next(size_t i) const noexcept { return edges.at_next(i); } ///<Returns reference to edge after index `i`. Wraps around at last edge.
+    const Edge &at_prev(size_t i) const noexcept { return edges.at_prev(i); } ///<Returns reference to edge before index `i`. Wraps around at first edge.
 
     void swap(SimpleContour &b) noexcept;
     void clear() noexcept { edges.clear(); clockwise_fresh = true; area_fresh = true; boundingBox_fresh = true;  boundingBox.MakeInvalid(); area = 0.; clockwise = true; } ///<Empty the shape by deleting all edges.
@@ -401,7 +401,7 @@ bool EdgeVector<Edgeish>::CrossPoints(std::vector<CPData> &ret, bool me_closed,
 /** Simple comparison operator for container ordering */
 inline std::strong_ordering SimpleContour::operator <=>(const SimpleContour &b) const noexcept
 {
-    //Do not use bounding box, clockwiseness or aread, which may possibly not
+    //Do not use bounding box, clockwiseness or area, which may possibly not
     //be computed, as we want this ordering to remain unchanged even if we compute a
     //formerly not computed value.
     if (size() != b.size()) return size() <=> b.size();
