@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -42,7 +42,7 @@ struct CshPos
     int last_pos;  ///<The index of the last character of the range.
     CshPos operator + (const CshPos &a) const noexcept { return CshPos(std::min(first_pos, a.first_pos), std::max(last_pos, a.last_pos)); }
     CshPos &operator += (const CshPos &a) noexcept { first_pos = std::min(first_pos, a.first_pos); last_pos = std::max(last_pos, a.last_pos); return *this; }
-    bool operator ==(const CshPos &a) const noexcept { return first_pos==a.first_pos && last_pos==a.last_pos; }
+    auto operator <=>(const CshPos&) const noexcept = default;
     bool IsWithin(int p) const noexcept { return first_pos<=p && last_pos>=p; } ///<Returns true if positin p is inside the range.
     bool IsWithin(const CshPos &p) const noexcept {return IsWithin(p.first_pos) && IsWithin(p.last_pos);} ///<Returns true if another range is completely within the range.
     bool Overlaps(const CshPos &p) const noexcept { return p.first_pos<=last_pos && first_pos<=p.last_pos; }
@@ -66,8 +66,8 @@ enum EColorSyntaxType : uint8_t {
     COLOR_PARENTHESIS,     ///<An opening or closing parenthesis ()
     COLOR_DESIGNNAME,      ///<The name of a design
     COLOR_STYLENAME,       ///<The name of a style
-    COLOR_ENTITYNAME,      ///<The name of an entity
-    COLOR_ENTITYNAME_FIRST,///<The name of an entity when first occures in the file
+    COLOR_ENTITYNAME,      ///<The name of an entity (we also have partial, see below)
+    COLOR_ENTITYNAME_FIRST,///<The name of an entity when first occurs in the file
     COLOR_COLORNAME,       ///<The name of a color
     COLOR_COLORDEF,        ///<The description of a color other than a name
     COLOR_MARKERNAME,      ///<The name of a marker
@@ -84,10 +84,11 @@ enum EColorSyntaxType : uint8_t {
     COLOR_ATTRVALUE,       ///<The value of an attribute
     COLOR_ATTRVALUE_EMPH,  ///<Emphasized part of the attribute value
     COLOR_LABEL_TEXT,      ///<The text of a label, either after a colon or between quotation marks
-    COLOR_LABEL_ESCAPE,    ///<The text of a lanel escape, such as '\n'
+    COLOR_LABEL_ESCAPE,    ///<The text of a label escape, such as '\n'
     COLOR_ERROR,           ///<A place where an error was detected.
     COLOR_NO_ERROR,        ///<The inverse of error formatting
     COLOR_COMMENT,         ///<A comment
+    COLOR_ENTITYNAME_PARTIAL,///<The name of an entity partially written
     COLOR_MAX              ///<Maximum value. Note: The Windows GUI also uses this value to mark places (in CshErrorList) that are no longer in error and shall be re-colored.
 };
 
@@ -144,6 +145,14 @@ struct CshErrorList {
     void swap(CshErrorList &a) { error_ranges.swap(a.error_ranges); }
 };
 
+struct MultiEntry : CshPos {
+    bool is_param = false;
+    std::string_view content;
+    MultiEntry(const CshPos& pos, std::string_view content, bool p = false) noexcept : CshPos(pos), is_param(p), content(content) {}
+};
+
+using MultiList = std::vector<MultiEntry>;
+
 /** Checks if all elements are sorted and non-overlapping.
  *  Used in debug mode only.*/
 template <typename PosList>
@@ -312,7 +321,7 @@ public:
      * @{  */
     mutable std::string plain;  ///<Plain text of the hint for sorting
     mutable std::string replaceto;///<When selected, insert this to the chart text. When empty, use 'plain'.
-    mutable bool keep = false;  ///<wether after insertion we shall keep the hint box up
+    mutable bool keep = false;  ///<whether after insertion we shall keep the hint box up
     mutable int x_size = 0;     ///<calculated size of hint text in the hint listbox. NOT scaled for high DPI screens.
     mutable int y_size = 0;     ///<calculated size of hint text in the hint listbox. NOT scaled for high DPI screens.
     mutable std::pair<uint16_t, uint16_t> highlight = {0,0}; ///<The character indexes (begin,end) in 'plain' to highlight. Used with fuzzy or starts_with search.
@@ -321,10 +330,10 @@ public:
     /** @name Derived values used by the Windows MFC GUI. They should not have been here
      *  but easier this way. TODO: refactor these to an optional.
      * @{  */
-    mutable int ul_x = 0;       ///<Size of the rectange shown in list box
-    mutable int ul_y = 0;       ///<Size of the rectange shown in list box
-    mutable int br_x = 0;       ///<Size of the rectange shown in list box
-    mutable int br_y = 0;       ///<Size of the rectange shown in list box
+    mutable int ul_x = 0;       ///<Size of the rectangle shown in list box
+    mutable int ul_y = 0;       ///<Size of the rectangle shown in list box
+    mutable int br_y = 0;       ///<Size of the rectangle shown in list box
+    mutable int br_x = 0;       ///<Size of the rectangle shown in list box
     mutable EHintItemSelectionState state = HINT_ITEM_NOT_SELECTED; ///<Will show if this hint is selected or not
     /** @} */
     CshHint(std::string &&d, const char *desc, EHintType t, bool s = true, CshHintGraphicCallback c = nullptr, CshHintGraphicParam p = 0, int so = 0) :
@@ -369,8 +378,8 @@ enum class EHintFilter {
  * In this rule we can realize that the cursor is inside value (or we match to a rule saying
  * "name =" and realize that the cursor is just after). Here we can conclude that the hint
  * will be a sort of attribute value. (So we pick EHintSourceType::ATTR_VALUE.) But we do not know
- * what values the attribute can take, because that depens on what element we are parsing.
- * So after we macth
+ * what values the attribute can take, because that depends on what element we are parsing.
+ * So after we match
  * the low level rule, we set the hint type only and when processing the higher level rule for the
  * full element, do we fill in the actual hints. In the meantime we set the hintStatus member to
  * HINT_LOCATED. Some cases we cannot fill in all the hints in one go and must go to an even higher
@@ -437,7 +446,7 @@ protected:
     virtual void FillNamesHints() = 0;
     /** @} */
     virtual bool CalculateIsCursorAtLineBegin() const;
-    virtual void BeforeYaccParse(const std::string& input, int cursor_p);
+    virtual void BeforeYaccParse(std::string&& input, int cursor_p);
     virtual void AfterYaccParse();
     virtual bool DoHintLocated(EHintSourceType hsource, std::string_view a_name);
 public:
@@ -457,16 +466,14 @@ public:
     * @param prefix the part of the filename already typed, we return all filenames not
     *        only the ones prefixed by this.
     *        May include a path component, in that case, we return files from that dir.
-    * @param included_from the path of the current file (for relative paths in prefix).
-    * @param param arbitrary parameter passed.*/
-    typedef std::vector<std::string>(*FileListProc)(std::string_view prefix, std::string_view included_from, void*param);
+    * @param included_from the path of the current file (for relative paths in prefix).*/
+    typedef std::vector<std::string>(*FileListProc)(std::string_view prefix, std::string_view included_from);
 
     StyleNameSet          ForbiddenStyles; ///<Styles we never offer as hints (e.g., ->)
     int                   cursor_pos;      ///<The location of the cursor now (used to identify partial keyword names & hint list). The zero-based index of the *byte* (and not UTF-8 character) the cursor is before. If ==input_text.length(), we are after the last character. Larger or negative values are invalid.
     std::shared_ptr<CshParams> params;     ///<Parameters to use
     std::string           FileName;        ///<The disk path of the file we parse - to provide correct include filename hints. May be empty if we are sure to ignore hints or if no disk file.
     FileListProc const    file_list_proc;  ///<The callback to call for a list of files to "include"
-    void* const           file_list_proc_param; ///<The parameter to pass to the callback fn.
     /** @} */
     /** @name Running variables during csh parsing
      * @{  */
@@ -482,12 +489,12 @@ public:
     std::string            input_text;       ///<The text we parse
     /** @}*/
     /** Type to store designs in. (These designs are just to maintain info needed for coloring and hints, not full attribute sets.)*/
-    using CshContextMap = std::map<std::string, CshContext>;
+    using CshContextMap = std::map<std::string, CshContext, std::less<>>;
     /** @name The collected CSH info
      * @{ */
     CshListType       CshList;        ///<The collected color syntax info
     CshPosList        ColonLabels;    ///<The position of colon labels
-    CshPosList        QuotedStrings;  ///<The position of quoted strings
+    MultiList         MultiElements;  ///<The recent fragments of multi-strings (string concatenated by ~). See multi_segment_string for more.
     CshPosList        Instructions;   ///<The position of all things that end in a semicolon (arcs, commands, chart options)
     CshPosList        BracePairs;     ///<The position of all {} pairs (the trailing '}' may be in fact missing).
     CshPosList        SqBracketPairs; ///<The position of all [] pairs (the trailing ']' may be in fact missing).
@@ -499,11 +506,11 @@ public:
 
     /** Initializes the Csh Object.
      * @param [in] defaultDesign Specifies a ContextBase to collect
-     *   forbidden and default style names, color names and definitons to learn
+     *   forbidden and default style names, color names and definitions to learn
      *   from.
      * @param [in] proc A procedure to call when we need to list file candidates to the 'include' command.
      * @param [in] param An opaque value to pass to the 'proc'.*/
-    Csh(const Context &defaultDesign, FileListProc proc, void*param);
+    explicit Csh(const Context &defaultDesign, FileListProc proc);
     /** Create a copy of the current Csh object (full copy of a descendant).
      * It is used in GUIs to clone a Csh containing the parsed designs and
      * keywords, options, etc. before each parsing of the chart text.
@@ -517,20 +524,20 @@ public:
     * @param [in] input The chart text
     * @param [in] cursor_p The current position of the cursor.
     * @param [in] pedantic The starting value of 'pedantic' chart option.*/
-    virtual void ParseText(const std::string& input, int cursor_p, bool pedantic) = 0;
+    virtual void ParseText(std::string&& input, int cursor_p, bool pedantic) = 0;
     /** Parse design library text to extract designs and shapes
      * By default it is equivalent to ParseText(), but may tweak settings
      * (restoring them afterwards).
      * @param [in] input The chart text
      * @param [in] cursor_p The current position of the cursor.*/
-    virtual void ParseDesignText(const std::string& input, int cursor_p) { ParseText(input, cursor_p, true); }
+    virtual void ParseDesignText(std::string&& input, int cursor_p) { ParseText(std::move(input), cursor_p, true); }
 
     //fill in size, plain and filter/compact if needed
     virtual void ProcessHints(Canvas &canvas, StringFormat *format, std::string_view uc, EHintFilter filter_by_uc, bool compact_same);
 
 
-    /**Add a name of a shape and its port names (whithout the actual shape)
-     * returns false if a shape with this name is already added (in that case we dont add it)*/
+    /**Add a name of a shape and its port names (without the actual shape)
+     * returns false if a shape with this name is already added (in that case we don't add it)*/
     bool AddShapeName(std::string_view n, std::vector<std::string> *ports = nullptr);
     /** Test if a shape name is valid or not.*/
     bool IsShapeName(std::string_view n) const { return GetShapeNum(n)>=0; }
@@ -545,10 +552,12 @@ public:
     virtual void AddCSH_ColonString_CheckAndAddEscapeHint(const CshPos& pos, std::string_view value, bool unquoted, bool two_colons=false);
     virtual void AddCSH_AttrName(const CshPos&, std::string_view name, EColorSyntaxType); ///<At pos there is either an option or attribute name (specified by the type). Search and color.
     virtual void AddCSH_AttrValue_CheckAndAddEscapeHint(const CshPos& pos, std::string_view value, std::string_view name);
+            void AddCSH_AttrValue_CheckAndAddEscapeHint_Solo(const CshPos& pos, std::string_view value);
     virtual void AddCSH_AttrColorValue(const CshPos& pos); ///<At pos there is an attribute value that looks like a color definition (with commas and all).
     virtual void AddCSH_StyleOrAttrName(const CshPos&pos, std::string_view name); ///<At pos there is either an attribute name or a style. Decide and color.
     virtual void AddCSH_EntityName(const CshPos&pos, std::string_view name); ///<At pos there is an entity name. Search and color.
-    virtual void AddCSH_AllCommentBeyond(const CshPos&pos); ///<Mark anything beyond the end of 'pos' as comment
+    virtual void AddCSH_AllCommentBeyond(const CshPos& pos) { AddCSH(CshPos(pos.last_pos + 1, int(input_text.length())), COLOR_COMMENT); } ///<Mark anything beyond the end of 'pos' as comment
+            void AddCSH_ParamOrCond(const CshPos& pos, const multi_segment_string& str); //Color parameter values on proc replay or ifthenelse conditions
     void ReplaceCSH(const CshPos &pos, EColorSyntaxType from, EColorSyntaxType to); ///<Replace any csh entry colored 'from' and *fully within* 'pos' to 'to'
     void RemoveAllTouchingCSH(const CshPos &pos);
     bool IsAnyColoringIn(const CshPos &pos) const { return CshList.CheckIfOverlap(pos); }
@@ -560,17 +569,22 @@ public:
     /** @name Functions to register info (mostly for precise indenting)
     * @{  */
     /** Adds the range to the list of instructions. Used for auto-indenting to the head of it.*/
-    void AddInstruction(const CshPos &pos) { Instructions.emplace_back(std::max(1, pos.first_pos), std::max(1, pos.last_pos)); } //to avoid positions of zero (can be if an instruction starts with a potentially empty rule as with DOT
+    void AddInstruction(const CshPos& pos) { Instructions.emplace_back(std::max(1, pos.first_pos), std::max(1, pos.last_pos)); ClearMulti(); } //to avoid positions of zero (can be if an instruction starts with a potentially empty rule as with DOT
     /** Adds the range to the list of instructions, if this range does not represent a curly brace pair.
      * Used in ifthenelse. There we do not want braces to become their own instruction.*/
-    void AddInstructionIfNotBrace(const CshPos &pos);
+    void AddInstructionIfNotBrace(const CshPos &pos) { if (std::find(BracePairs.rbegin(), BracePairs.rend(), pos) == BracePairs.rend()) AddInstruction(pos); }
     /** Adds a brace pair that is missing its closing element.
      * We substitute a char beyond the end-of-file as the closing element, this enables
      * auto-indent to assume we are inside a brace pair even when cursor is at EOF.*/
     void AddOpenBracePair(const CshPos &pos) { BracePairs.emplace_back(pos.first_pos, int(input_text.length()+2)); }
     /** Adds a range to the list of colon labels. Used for indenting. */
-    void AddColonLabel(const CshPos&pos, std::string_view text);
-    void AddQuotedString(const CshPos&pos) { QuotedStrings.push_back(pos); } ///<Marks a range as a quoted string
+    void AddColonLabel(const CshPos &pos, std::string_view /*text*/) { ColonLabels.push_back(pos); }
+    void AddQuotedString(const CshPos&) { } ///<Marks a range as a quoted string
+    //bool IsQuotedString(const CshPos& pos, std::string_view value) const noexcept { _ASSERT(input_text.data() <= value.data() && value.data() <= input_text.data() + input_text.size()); return input_text.data() < value.data() && value.data()[-1] == '"'; }
+    bool IsQuotedString(const CshPos&, std::string_view value) const noexcept { return value.data() && value.data()[-1]=='"'; }
+    void StoreMulti(const str_view& s, const CshPos& pos) { if (std::ranges::find(MultiElements, pos)==MultiElements.end()) MultiElements.emplace_back(pos, s); }
+    void StoreMulti(const multi_segment_string& s, const CshPos& pos) { if (!s.had_error && !s.multi && std::ranges::find(MultiElements, pos)==MultiElements.end()) MultiElements.emplace_back(pos, s.view(), s.had_param); }
+    void ClearMulti() noexcept { MultiElements.clear(); }
     const CshPos *IsInRange(const CshPosList &range, int pos, bool include_after) const;
     const CshPos *IsInsideRange(const CshPosList &range, int pos) const;
     const CshPos *IsAtEndOfRange(const CshPosList &range, int pos) const; ///<After parsing return the range if character 'pos' is the last of one of the 'range' elements. (Used for smart ident.)
@@ -597,7 +611,7 @@ public:
     /** Push the context stack  with an empty context.*/
     void PushContext(bool f, EContextParse p) { Contexts.emplace_back(f, p); }
     void PopContext() { Contexts.pop_back(); }  ///<Pop the context stack.
-    std::string SetDesignTo(const std::string&design, bool full);
+    std::string SetDesignTo(std::string_view design, bool full);
     /** Returns the relation of the cursor pos to a range in the input file.*/
     ECursorRelPosType CursorIn(const CshPos &p) const noexcept { return CursorIn(p.first_pos, p.last_pos); }
     /** Returns the relation of the cursor pos to a range in the input file.*/
@@ -677,18 +691,18 @@ public:
     virtual void AddLineBeginToHints();
     /** Add chart options to hints. */
     virtual void AddOptionsToHints() { AddDesignOptionsToHints();  }
-    /** Add keywords and option names applicable inside a design definiton to hint list.*/
+    /** Add keywords and option names applicable inside a design definition to hint list.*/
     virtual void AddDesignLineBeginToHints();
-    /** Add chart options applicable inside a design definiton to hints. */
+    /** Add chart options applicable inside a design definition to hints. */
     virtual void AddDesignOptionsToHints() { }
-    virtual void AttributeNamesForStyle(const std::string &style);
-    virtual bool AttributeValuesForStyle(const std::string &attr, const std::string &style);
+    virtual void AttributeNamesForStyle(std::string_view style);
+    virtual bool AttributeValuesForStyle(std::string_view attr, const std::string &style);
     /** @}*/
 
     /** A function to clean up elements after parsing a design library.
      * Only elements are kept that need to be carried over from a design library to the actual charts.
      * These include the designs parsed, the shape library (set after parsing the design library
-     * language-wise, not just for coloring as we do in this class), the font name and cheme to use
+     * language-wise, not just for coloring as we do in this class), the font name and scheme to use
      * pointer and the forbidden styles. */
     virtual void CleanupAfterDesignlib();
 
@@ -739,7 +753,7 @@ unsigned Csh::FindPrefix(const char set[][STRING_LEN], std::string_view text) no
 * We cancel entries that are entirely in the deleted region (if offset < 0) by setting their
 * last position to a value smaller than their first pos.
 * @param list The list to adjust. Any container, shall contain CshPos (or derivative) elements.
-*             As for normal csh entires, the first character of the file is 1.
+*             As for normal csh entries, the first character of the file is 1.
 *             Also, if the first and last position of a csh entry equals, that means
 *             that the csh entry corresponds to a single character of the file.
 * @param [in] start The first character of the change. Again assuming the first character of
@@ -849,8 +863,7 @@ public:
     ~DummyCsh() override = default;
     std::unique_ptr<Csh> Clone() const override { return std::make_unique<DummyCsh>(*this); }
     void FillNamesHints() override {};
-    void ParseText(const std::string& input, int /*cursor_p*/, bool /*pedantic*/) override {}
-
+    void ParseText(std::string&&, int, bool) override {}
 };
 
 #endif
