@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -135,10 +135,17 @@ void Element::PostPosProcess(Canvas &, Chart *chart)
         controls.clear();
     } else {
         //Determine, where the controls shall be shown
-        control_location.x = Range(area.GetBoundingBox().x.till,
-            area.GetBoundingBox().x.till + control_size.x);
-        control_location.y = Range(area.GetBoundingBox().y.from,
-            area.GetBoundingBox().y.from + control_size.y*controls.size());
+        //In case the area is non-rectangular (e.g., a large circle), we must position the controls
+        //to a place where they are reachable by the mouse from the area.
+        //Take the upper region of the area (as high as the controls would be) and place the
+        //controls right of it.
+        Block upper_region = area.GetBoundingBox();
+        upper_region.y.till = std::min(upper_region.y.till, upper_region.y.from + control_size.y*controls.size());
+        const double X = (area*upper_region).GetBoundingBox().x.till;
+        const double Y = upper_region.y.from;
+        control_location.x = Range(X, X + control_size.x);
+        control_location.y = Range(Y, Y + control_size.y*controls.size());
+        //Expand the area if the chart wants it. => For tracking we use a slightly larger area.
         if (chart->trackExpandBy) {
             area.Simplify();
             //If expand cannot do it,/we still keep the original stuff.
@@ -146,11 +153,8 @@ void Element::PostPosProcess(Canvas &, Chart *chart)
                                                    contour::EXPAND_MITER_ROUND,
                                                    contour::EXPAND_MITER_ROUND,
                                                    2, 2);
-            if (!expanded_area.IsEmpty()) {
-                area = expanded_area;
-            } else {
-                _ASSERT(0);
-            }
+            _ASSERT(!expanded_area.IsEmpty());
+            if (!expanded_area.IsEmpty()) area = std::move(expanded_area);
         }
     }
     if (draw_is_different && !area_draw.IsEmpty() && !area_draw_is_frame && chart->trackExpandBy) {
@@ -253,3 +257,54 @@ EGUIControlType Element::WhichControl(const XY &xy) const noexcept
 }
 
 
+/** Return the outer Edge of indicators.
+ * @param [in] pos The middle of the top edge of the indicator shall be here.
+ * @param [in] line The line style of the indicator
+ * @param [in] fill The fill style of the indicator
+ * @param [in] shadow The shadow style of the indicator
+ * @returns The outer edge.*/
+Block Element::GetIndicatorCover(const XY& pos, const LineAttr& line, 
+                                 const FillAttr&, const ShadowAttr&) {
+    Block b(pos.x-indicator_size.x/2, pos.x+indicator_size.x/2,
+            pos.y, pos.y+indicator_size.y);
+    return b.Expand(line.LineWidth());
+}
+
+/** Draw an indicator.
+ * @param [in] pos The middle of the top edge of the indicator shall be here.
+ * @param canvas The canvas to draw on.
+ * @param [in] line The line style of the indicator
+ * @param [in] fill The fill style of the indicator
+ * @param [in] shadow The shadow style of the indicator*/
+void Element::DrawIndicator(XY pos, Canvas* canvas, const LineAttr &line, 
+                            const FillAttr &fill, const ShadowAttr &shadow) {
+    if (!canvas) return;
+
+    const Block area = GetIndicatorCover(pos, line, fill, shadow);
+    LineAttr line_dot(line);
+    line_dot.width = area.y.Spans()/4;
+    line_dot.type = ELineType::SOLID;
+    pos.y += indicator_size.y/2;
+    const XY offset(area.x.Spans()/4, 0);
+    if (canvas->does_graphics()) {
+        if (canvas->cannot_draw()) return;
+        canvas->Shadow(area, line, shadow);
+        canvas->Fill(area, line, fill);
+        canvas->Line(area, line);
+
+        cairo_save(canvas->GetContext());
+        cairo_set_line_cap(canvas->GetContext(), CAIRO_LINE_CAP_ROUND);
+        canvas->Line(pos,        pos,        line_dot);
+        canvas->Line(pos-offset, pos-offset, line_dot);
+        canvas->Line(pos+offset, pos+offset, line_dot);
+        cairo_restore(canvas->GetContext());
+    } else {
+        canvas->Add(GSBox(area, line, fill, shadow));
+        const Contour dot(pos, line.LineWidth());
+        canvas->Add(GSShape(dot.CreateShifted(-offset) + dot + dot.CreateShifted(offset),
+                            LineAttr::None(), FillAttr::Solid(*line.color)));
+    }
+}
+
+
+
