@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -41,6 +41,9 @@ enum class EGUIControlType {
 };
 
 class Chart;
+class LineAttr;
+struct FillAttr;
+struct ShadowAttr;
 
 /** The base object for elements on the chart (entity headings, arcs, boxes, commands, etc.)
  * This object contains a lot of members useful for the Windows GUI and not needed
@@ -63,6 +66,7 @@ protected:
     mutable std::vector<EGUIControlType>
                    controls;           ///<GUI controls of this element.
     mutable Block  control_location;   ///<The area the GUI controls occupy.
+    static XY GetIndicatorSize(const LineAttr& line, const FillAttr&, const ShadowAttr& shadow) { const double a = *shadow.offset + line.LineWidth()*2; return indicator_size+XY(a, a); }
 
 public:
     FileLineColRange file_pos;  ///<The location of the element in the source file.
@@ -76,20 +80,20 @@ public:
     /** Add the attribute names we take to `csh`.*/
     static void AttributeNames(Csh &) {}
     /** Add a list of possible attribute value names to `csh` for attribute `attr`.*/
-    static bool AttributeValues(const std::string /*attr*/, Csh &/*csh*/) { return false; }
+    static bool AttributeValues(std::string_view /*attr*/, Csh &/*csh*/) { return false; }
     virtual void ShiftBy(double y);
     virtual void ShiftBy(const XY &);
     /** An area over which if the mouse hoovers, we highlight the Element*/
-    const Area &GetAreaToSearch() const   
+    const Area &GetAreaToSearch() const
         {return area;};
     /** An area to highlight when the mouse moves over the Element*/
-    const Contour &GetAreaToDraw() const  
+    const Contour &GetAreaToDraw() const
         {return draw_is_different ? area_draw : area;}
     /** An area which shall not get covered by notes, if possible. */
-    const Contour &GetAreaImportant() const     
+    const Contour &GetAreaImportant() const
         {return area_important;}
     /** An area to the edge of which notes made to this element will point to*/
-    const Contour &GetAreaToNote() const  
+    const Contour &GetAreaToNote() const
         {return area_to_note.IsEmpty() ? area : area_to_note;}
     /** An second try for an area to the edge of which notes with an 'at' clause will point to*/
     const Contour &GetAreaToNote2() const {return area_to_note2;}
@@ -108,6 +112,12 @@ public:
     void DrawControls(cairo_t*, double size) const;
     static void DrawControl(cairo_t*, EGUIControlType type, double size);
     EGUIControlType WhichControl(const XY &xy) const noexcept;
+
+    static Block GetIndicatorCover(const XY& pos, const LineAttr& line,
+                                   const FillAttr& fill, const ShadowAttr& shadow);
+    static void DrawIndicator(XY pos, Canvas* canvas, const LineAttr& line,
+                              const FillAttr& fill, const ShadowAttr& shadow);
+
 };
 
 
