@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -21,7 +21,6 @@
  * @ingroup libcgencommon_files */
 
 #define _CRT_SECURE_NO_WARNINGS
-#define _CRT_NONSTDC_NO_DEPRECATE //For visual studio to allow strdup
 
 #include <algorithm>
 #include <cstring>
@@ -56,7 +55,7 @@ bool FileLineCol::Pop() noexcept
 }
 
 /** Advance the position (creating a new position object) with the UTF8 text's length.
- * This is best called if the string is the same that starts at the origial location 
+ * This is best called if the string is the same that starts at the origial location
  * in the input file to get the next char position after the string.
  * If a '\n', '\0xa\0xd' or a is found we bump line and reset col.
  * We do not test if a character is valid utf8 or not. We ignore truncated
@@ -75,7 +74,7 @@ single_char_newline:
             ret.line++;
             ret.col = 1;
             continue;
-        } 
+        }
         if (*i=='\x0a') goto single_char_newline;
         if (*i==0) return ret;
         int length = UTF8TrailingBytes(*i)+1;
@@ -98,7 +97,7 @@ std::string FileLineCol::Print() const
     return ret;
 }
 
-/** Reads a printed file location. 
+/** Reads a printed file location.
  * On success we set our value to that location, set 's' to the first char
  * after us and return true. On failure, we leave us and 's' unchanged and
  * return false. */
@@ -156,17 +155,17 @@ unsigned MscError::AddFile(std::string_view filename)
     return (unsigned)Files.size()-1;
 }
 
-/** Formulate an error element - used internally. 
+/** Formulate an error element - used internally.
  * @param [in] linenum The line number to show. We do not consider the series (the next member), just the file, line and col members.
  * @param [in] linenum_ord The line number to use at ordering. Here we consider all parts.
- * @param [in] t The type of the message (error, warnig, info, etc.)
+ * @param [in] t The type of the message (error, warning, info, etc.)
  * @param [in] is_once True if the message is to be displayed only once.
- * @param [in] add_para Add parenthesis around 
+ * @param [in] add_para Add parenthesis around
  * @param [in] omit_message_type If true, we do not label the message as 'warning' or 'error'.
  * @param [in] lt If this is the main message or explanatory context for a previous main message.
  * @param [in] msg The message to display
  * @returns The error element. */
-ErrorElement MscError::FormulateElement(FileLineCol linenum, FileLineCol linenum_ord, ErrorElement::EType t, 
+ErrorElement MscError::FormulateElement(const FileLineCol& linenum, const FileLineCol& linenum_ord, ErrorElement::EType t,
                                         bool is_once, bool add_para, bool omit_message_type, ErrorElement::ELineType lt,
                                         std::string_view msg) const
 {
@@ -209,7 +208,7 @@ ErrorElement MscError::FormulateElement(FileLineCol linenum, FileLineCol linenum
  * @param [in] atValue True if the problem is with the value of the attr, false if with its name.
  * @param [in] s The message to display
  * @param [in] once Auxiliary info, a type of hint on what to do. Added as a separate element, but only once.
- * @param [in] t The type of the message (error, warnig, info, etc.) */
+ * @param [in] t The type of the message (error, warning, info, etc.) */
  void MscError::Add(const Attribute &a, bool atValue, std::string_view s, std::string_view once, ErrorElement::EType t)
 {
     if (a.error) return;
@@ -226,15 +225,15 @@ ErrorElement MscError::FormulateElement(FileLineCol linenum, FileLineCol linenum
  * @param [in] linenum_ord The line number to use at ordering.
  * @param [in] s The message to display
  * @param [in] once Auxiliary info, a type of hint on what to do. Added as a separate element, but only once.
- * @param [in] t The type of the message (error, warnig, info, etc.)
+ * @param [in] t The type of the message (error, warning, info, etc.)
  * @param [in] omit_error_warning If true, we do not label the message as 'warning' or 'error'.
  * @param [in] lt How to interpret this message. This allows adding a Context or Substitution line manually.*/
-void MscError::Add(FileLineCol linenum, FileLineCol linenum_ord, std::string_view s, std::string_view once,
+void MscError::Add(const FileLineCol& linenum, const FileLineCol& linenum_ord, std::string_view s, std::string_view once,
                    ErrorElement::EType t, bool omit_error_warning, ErrorElement::ELineType lt)
 {
     bool inserted = AddOne(linenum, linenum_ord, s, t, false, false, omit_error_warning, lt);
-    if (inserted && once.length()>0) 
-        AddOne(linenum, linenum_ord, once, t, true, true, omit_error_warning, 
+    if (inserted && once.length()>0)
+        AddOne(linenum, linenum_ord, once, t, true, true, omit_error_warning,
                lt == ErrorElement::ELineType::Main ? ErrorElement::ELineType::Context : lt);
 }
 
@@ -243,18 +242,18 @@ void MscError::Add(FileLineCol linenum, FileLineCol linenum_ord, std::string_vie
  * @param [in] linenum The line number to show.
  * @param [in] linenum_ord The line number to use at ordering.
  * @param [in] s The message to display
- * @param [in] t The type of the message (error, warnig, info, etc.)
+ * @param [in] t The type of the message (error, warning, info, etc.)
  * @param [in] is_once True if the message is to be displayed only once.
  * @param [in] add_parenthesis If set this message is enclosed in parenthesis.
  * @param [in] omit_error_warning If true, we do not label the message as 'warning' or 'error'.
  * @param [in] lt If this is the main message or explanatory context for a previous main message.
  * @returns True if we have actually inserted something.*/
-bool MscError::AddOne(FileLineCol linenum, FileLineCol linenum_ord, std::string_view s, ErrorElement::EType t,
+bool MscError::AddOne(const FileLineCol& linenum, const FileLineCol& linenum_ord, std::string_view s, ErrorElement::EType t,
                       bool is_once, bool add_parenthesis, bool omit_error_warning, ErrorElement::ELineType lt)
 {
     ErrorElement e1 = FormulateElement(linenum, linenum_ord, t, is_once, add_parenthesis, omit_error_warning, lt, s);
     if (!unique_push_back(std::move(e1))) return false;
-    bool was_param = false; 
+    bool was_param = false;
     for (const FileLineCol *p = &linenum; p->GetNext(); p = p->GetNext()) {
         switch (p->reason) {
         default:
@@ -292,7 +291,7 @@ bool MscError::AddOne(FileLineCol linenum, FileLineCol linenum_ord, std::string_
 /** Check if an error is in one of the error stores.*/
 bool MscError::IsInStore(const ErrorElement & e) const
 {
-    return Messages.end()!=std::find_if(Messages.begin(), Messages.end(), 
+    return Messages.end()!=std::find_if(Messages.begin(), Messages.end(),
         [&e](const auto &a) {return a.AppearsSame(e); });
 }
 
@@ -311,7 +310,7 @@ void MscError::Sort()
 
 unsigned MscError::GetErrorNum(bool oWarnings, bool oTechnicalInfo) const
 {
-    return (unsigned) std::count_if(Messages.begin(), Messages.end(), 
+    return (unsigned) std::count_if(Messages.begin(), Messages.end(),
                                     [oWarnings, oTechnicalInfo](auto& m)
                                     {return m.Fits(oWarnings, oTechnicalInfo); });
 }
@@ -326,7 +325,7 @@ const ErrorElement *MscError::GetError(unsigned num, bool oWarnings, bool oTechn
 }
 
 void MscError::WarnMscgen(FileLineCol linenum, std::string_view s, std::string_view once)
-{ 
+{
     if (warn_mscgen)
         Add(linenum, linenum, StrCat(s, " (This is an mscgen syntax.)"), once, ErrorElement::EType::Warning, false);
 }
@@ -335,8 +334,8 @@ void MscError::WarnMscgenAttr(const Attribute &a, bool isOption, std::string_vie
 {
     string ao = isOption ? "Option" : "Attribute";
     if (warn_mscgen)
-        Add(a.linenum_attr.start, a.linenum_attr.start, 
-            ao << " is according to mscgen syntax. Use '" << name << "' instead.", 
+        Add(a.linenum_attr.start, a.linenum_attr.start,
+            ao << " is according to mscgen syntax. Use '" << name << "' instead.",
             "", ErrorElement::EType::Warning, false);
 }
 
@@ -369,14 +368,9 @@ MscError &MscError::operator += (const MscError &o)
  * remove its path portion.*/
 void MscError::RemovePathFromErrorMessages()
 {
-    for (auto &fn : Files) {
-        std::string fn_short = fn;
-        size_t p = fn_short.find_last_of("\\/");
-        if (p!=std::string::npos)
-            fn_short.erase(0, p+1);
-        for (auto &ee: Messages) {
-            if (ee.text.compare(0, fn.length(), fn)==0)
-                ee.text.replace(0, fn.length(), fn_short);
-        }
-    }
+    for (auto &fn : Files)
+        if (const size_t p = fn.find_last_of("\\/"); p!=std::string::npos)
+            for (auto& ee: Messages)
+                if (ee.text.starts_with(fn))
+                    ee.text.erase(0, p+1);
 }
