@@ -40,35 +40,48 @@
 #define NDEBUG
 #endif
 
-inline std::string my_strerror(int errno_) {
-    char buff[512] = {'\0'};
-    //strerror() is not thread safe and results in all kinds of
-    //compiler warnings. There are safe variants, but...
+//strerror() is not thread safe and results in all kinds of
+//compiler warnings. There are safe variants, but...
 #ifdef _WIN32
+inline std::string my_strerror(int errno_) {
     //Well, windows has strerror_s(), which is C11 standard
+    char buff[512] = {'\0'};
     strerror_s(buff, sizeof(buff), errno_);
-#else
-    //POSIX and GCC each have strerror_r(), but what a mess: https://linux.die.net/man/3/strerror_r
-    //The precondition macros listed there do not work on MacOS, so we do this instead.
-    using strerror_r_ret_type = decltype(strerror_r(errno_, buff, sizeof(buff)));
-    constexpr bool strerror_r_rets_int = std::is_same_v<strerror_r_ret_type, int>;
-    constexpr bool strerror_r_rets_charp = std::is_same_v<strerror_r_ret_type, char *>;
-    //Note: "Outside a template, a discarded statement is fully checked. if constexpr is not a
-    //substitute for the #if preprocessing directive" So both branches must be valid C++.
-    if constexpr (strerror_r_rets_int) {
-        if (strerror_r(errno_, buff, sizeof(buff)))
-            snprintf(buff, sizeof(buff), "Could not get error string for errno=%d", errno_);
-    } else if constexpr (strerror_r_rets_charp) {
-        char* b = (char*)strerror_r(errno_, buff, sizeof(buff));
-        if (b!=buff) {
-            strncpy(buff, b, sizeof(buff)-1);
-            buff[sizeof(buff)-1] = 0;
-        }
-    } else
-        static_assert(strerror_r_rets_charp || strerror_r_rets_int, "strerror_r() returns neither char* nor int.");
-#endif
     return buff;
 }
+#else
+//POSIX and GCC each have strerror_r(), but what a mess: https://linux.die.net/man/3/strerror_r
+//The precondition macros listed there do not work on MacOS, so we do this instead.
+//Note: "Outside a template, a discarded statement is fully checked. if constexpr is not a
+//substitute for the #if preprocessing directive" So both branches must be valid C++.
+//Even so, on Mac integers may be smaller than pointers, so we use a template to avoid
+//checking the other branch.
+template <typename strerror_r_ret_type>
+struct errno_printer {
+    static std::string print(int errno_) {
+        char buff[512] = {'\0'};
+        constexpr bool strerror_r_rets_int = std::is_same_v<strerror_r_ret_type, int>;
+        constexpr bool strerror_r_rets_charp = std::is_same_v<strerror_r_ret_type, char*>;
+        if constexpr (strerror_r_rets_int) {
+            if (strerror_r(errno_, buff, sizeof(buff)))
+                snprintf(buff, sizeof(buff), "Could not get error string for errno=%d", errno_);
+        } else if constexpr (strerror_r_rets_charp) {
+            char* b = (char*)strerror_r(errno_, buff, sizeof(buff));
+            if (b != buff) {
+                strncpy(buff, b, sizeof(buff) - 1);
+                buff[sizeof(buff) - 1] = 0;
+            }
+        } else
+            static_assert(strerror_r_rets_charp || strerror_r_rets_int, "strerror_r() returns neither char* nor int.");
+        return buff;
+    }
+};
+inline std::string my_strerror(int errno_) {
+    char buff[1];
+    using strerror_r_ret_type = decltype(strerror_r(errno_, buff, sizeof(buff)));
+    return errno_printer<strerror_r_ret_type>::print(errno_);
+}
+#endif
 
 template<typename T, typename... U> requires (std::equality_comparable_with<T, U> && ...)
 bool is_any_of(T t, U... u) { return ((t == u) || ...); }
