@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -19,8 +19,6 @@
 /** @file numbering.cpp The implementation of basic classes for numbering styles.
  * @ingroup libcgencommon_files */
 
-#define _CRT_SECURE_NO_DEPRECATE //For visual studio to allow sscanf
-#define _CRT_NONSTDC_NO_DEPRECATE //For visual studio to allow strdup
 #include <cstdlib>
 #include "chartbase.h"
 
