@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -142,8 +142,8 @@ bool CshHintGraphicCallbackForVIdent(Canvas *canvas, CshHintGraphicParam p, CshH
  * This object can do operations on a text fragment. A *fragment* is a piece
  * of a label containing no line breaks and having the same formatting,
  * thus a fragment contains no escape sequences.
- *  - It calculate height or width of a fragment using a drawing context.
- *  - It draw a fragment using a drawing context. */
+ *  - It calculates height or width of a fragment using a drawing context.
+ *  - It draws a fragment using a drawing context. */
 class StringFormat {
 public:
     /** Parses and described the content of a \ S(name,height,fillcolor) escape*/
@@ -238,9 +238,9 @@ public:
     /** Deletes all values, clearing all attributes. */
     void Empty();
     /** True if all attributes of text formatting is specified.*/
-    bool IsComplete() const;
+    bool IsComplete() const noexcept;
     /** True if none of the attributes of text formatting are specified.*/
-    bool IsEmpty() const;
+    bool IsEmpty() const noexcept;
     void Default();
     void UnsetWordWrap() {word_wrap = false;} ///<Unset the wordwrap attribute
     bool IsWordWrap() const {_ASSERT(word_wrap); return word_wrap.value_or(false); } ///<True if wordwrap is on.
