@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -19,8 +19,6 @@
 /** @file style.cpp The implementation of styles (Style and SimpleStyle) and contexts (Context).
  * @ingroup libcgencommon_files */
 
-#define _CRT_NONSTDC_NO_DEPRECATE //For visual studio to allow strdup
-
 #include "style.h"
 #include "chartbase.h"
 
@@ -40,7 +38,7 @@ bool Style::AddAttribute(const Attribute &a, Chart*chart)
 
 
 /** Create an empty style that contains all possible attributes.*/
-SimpleStyle::SimpleStyle(EStyleType tt, EColorMeaning cm) : 
+SimpleStyle::SimpleStyle(EStyleType tt, EColorMeaning cm) :
     Style(tt)
 {
     color_meaning = cm;
@@ -60,10 +58,10 @@ SimpleStyle::SimpleStyle(EStyleType tt, EColorMeaning cm) :
  * @param [in] nu True if the style shall contain the `number` attribute.
  * @param [in] shp True if the style shall contain entity related attributes (shape, shape.size)
  */
-SimpleStyle::SimpleStyle(EStyleType tt, EColorMeaning cm, 
+SimpleStyle::SimpleStyle(EStyleType tt, EColorMeaning cm,
              bool t, bool l, bool f, bool s, bool nu, bool shp) :
     Style(tt), color_meaning(cm),
-    f_line(l), f_fill(f), f_shadow(s), 
+    f_line(l), f_fill(f), f_shadow(s),
     f_text(t), f_numbering(nu), f_shape(shp)
 {
     Empty();
@@ -72,7 +70,7 @@ SimpleStyle::SimpleStyle(EStyleType tt, EColorMeaning cm,
 /** Make a style complete by setting the default values - but leave text & lost attributes empty.
  * Default attributes are half solid, right side, no numbering, no compress, indicator yes,
  * makeroom yes and the default of other attribute classes (specified there).
- * We skip text styles, since we have a global text attribute per chart and 
+ * We skip text styles, since we have a global text attribute per chart and
  * we set the default there. We also skip loss attributes as they do not have to be
  * fully specified.
  */
@@ -104,10 +102,21 @@ void SimpleStyle::Empty()
     shape_size.reset();
 }
 
+bool SimpleStyle::IsEmpty() const noexcept {
+    return Style::IsEmpty()
+        && (!f_line || line.IsEmpty())
+        && (!f_fill || fill.IsEmpty())
+        && (!f_text || text.IsEmpty())
+        && (!f_shadow || shadow.IsEmpty())
+        && (!f_numbering || !numbering)
+        && (!f_shape || (!shape && !shape_size));
+}
+
+
 /** Merge another style to us by copying the value of those attributes which are contained by both and are set in `toadd`.*/
 Style & SimpleStyle::operator +=(const Style &toadd)
 {
-    Style::operator+(toadd);
+    Style::operator+=(toadd);
     const SimpleStyle *p = dynamic_cast<const SimpleStyle*>(&toadd);
     if (p==nullptr)
         return *this;
@@ -210,13 +219,13 @@ bool SimpleStyle::AddAttribute(const Attribute &a, Chart*chart)
                 return true;
             }
             if (a.type == EAttrType::CLEAR) {
-                shape_size.reset(); 
+                shape_size.reset();
                 return true;
             }
             if (EArrowSize s; a.type == EAttrType::STRING && Convert(a.value, s)) {
                 shape_size = s;
                 return true;
-            } else 
+            } else
                 a.InvalidValueError(CandidatesFor<EArrowSize>(), chart->Error);
             return true;
         }
@@ -228,7 +237,7 @@ bool SimpleStyle::DoIAcceptUnqualifiedColorAttr() const
 {
     switch (color_meaning) {
     case EColorMeaning::FILL: return f_fill;
-    case EColorMeaning::ARROW_TEXT: 
+    case EColorMeaning::ARROW_TEXT:
     case EColorMeaning::TEXT: return f_text;
     case EColorMeaning::LINE_ARROW_TEXT:
         return f_text || f_line;
@@ -248,7 +257,7 @@ void SimpleStyle::AttributeNames(Csh &csh) const
 {
     if (DoIAcceptUnqualifiedColorAttr())
         csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRNAME)+"color",
-          "Set the color of the element.", 
+          "Set the color of the element.",
           EHintType::ATTR_NAME));
     if (f_line) {
         LineAttr::AttributeNames(csh, "line.");
@@ -269,11 +278,11 @@ void SimpleStyle::AttributeNames(Csh &csh) const
             "Adjust the format of the label text.",
             EHintType::ATTR_NAME));
     }
-    if (f_numbering) csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRNAME)+"number", 
-        "Turn numberin off or specify a concrete number.", 
+    if (f_numbering) csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRNAME)+"number",
+        "Turn numberin off or specify a concrete number.",
         EHintType::ATTR_NAME));
     if (f_shape && csh.pShapes && *csh.pShapes) {
-        csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRNAME) + "shape", 
+        csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRNAME) + "shape",
             "Set the shape of the entity.",
             EHintType::ATTR_NAME));
         csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRNAME) + "shape.size",
@@ -300,13 +309,13 @@ bool SimpleStyle::AttributeValues(std::string_view attr, Csh &csh) const
     if (CaseInsensitiveBeginsWith(attr, "shadow") && f_shadow)
         return shadow.AttributeValues(attr, csh);
     if (CaseInsensitiveEqual(attr, "number") && f_numbering) {
-        csh.AddToHints(CshHint(csh.HintPrefixNonSelectable() + "<number>", 
+        csh.AddToHints(CshHint(csh.HintPrefixNonSelectable() + "<number>",
             "Specify a concrete number to set the number of the element.",
             EHintType::ATTR_VALUE, false));
-        csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRVALUE)+"yes", 
+        csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRVALUE)+"yes",
             "Turn on auto-numbering for this element.",
             EHintType::ATTR_VALUE, true, CshHintGraphicCallbackForYesNo, CshHintGraphicParam(1)));
-        csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRVALUE)+"no", 
+        csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRVALUE)+"no",
             "Exempt this element from auto-numbering and have no number here.",
             EHintType::ATTR_VALUE, true, CshHintGraphicCallbackForYesNo, CshHintGraphicParam(0)));
         return true;
@@ -324,6 +333,19 @@ bool SimpleStyle::AttributeValues(std::string_view attr, Csh &csh) const
     return Style::AttributeValues(attr, csh);
 }
 
+/** Apply our text attributes on top of 'sf' and make that
+ * our text attribute. In other words insert the content of
+ * 'sf' *before* the attrs in our text.*/
+void SimpleStyle::ApplyTextBefore(const StringFormat& sf) {
+    if (text.IsEmpty())
+        text = sf;
+    else {
+        StringFormat sf_ = sf;
+        sf_ += text;
+        text = sf_;
+    }
+}
+
 /////////////////////////////////////////////////////////////
 
 /** Returns the first matching parameter pair, null-null, if none*/
@@ -343,26 +365,27 @@ const ProcParamDef * Procedure::GetProcParamDef(std::string_view name) const
 }
 
 /** Consume a parameter list from procedure invocation and match them to our
- * parameter list. Report errors to chart & return true on error.
+ * parameter list. Report errors to chart & return empty on error.
  * Also adds a $$ parameter with the location of invocation.*/
-std::pair<bool, ParameterContext> Procedure::MatchParameters(const ProcParamInvocationList &pl, 
-                                                             const FileLineCol &invocation, Chart * chart) const
+std::optional<ParameterContext> Procedure::MatchParameters(const ProcParamInvocationList& pl,
+                                                           const FileLineCol& invocation, Chart* chart) const
 {
-    std::pair<bool, ParameterContext> ret;
-    ret.first = false;
+    std::optional<ParameterContext> ret{std::in_place};
     const bool has_params_in_invocation = pl.size();
     auto my_i = parameters.begin();
     auto i = pl.begin();
     for (; i!=pl.end() && my_i != parameters.end(); i++, my_i++) {
         if ((*i)->has_value) {
-            ret.second.emplace((*my_i)->name, ProcParamResolved((*i)->value, (*i)->linenum_value, true));
+            if (ret)
+                ret->emplace((*my_i)->name, ProcParamResolved((*i)->value, (*i)->linenum_value, true));
         } else {
             if ((*my_i)->has_default_value) {
-                ret.second.emplace((*my_i)->name, ProcParamResolved((*my_i)->default_value.c_str(), (*my_i)->linenum_default_value, true));
+                if (ret)
+                    ret->emplace((*my_i)->name, ProcParamResolved { (*my_i)->default_value, (*my_i)->linenum_default_value, true });
             } else {
                 chart->Error.Error((*i)->linenum_value, "This parameter has no default and requires a value. Ignoring procedure call.");
                 chart->Error.Error((*my_i)->linenum_name, (*i)->linenum_value, "Here is the definition of this parameter.");
-                ret.first = true;
+                ret.reset();
             }
         }
     }
@@ -371,38 +394,36 @@ std::pair<bool, ParameterContext> Procedure::MatchParameters(const ProcParamInvo
             chart->Error.Error((*i)->linenum_value, "Procedure "+name+" takes only " + std::to_string(parameters.size()) +" parameters. Ignoring procedure call.");
         else
             chart->Error.Error((*i)->linenum_value, "Procedure "+name+" takes no parameters. Ignoring procedure call.");
-        ret.first = true;
+        ret.reset();
     }
     for (; my_i != parameters.end(); my_i++) {
         if ((*my_i)->has_default_value) {
-            ret.second.emplace((*my_i)->name, ProcParamResolved((*my_i)->default_value.c_str(), (*my_i)->linenum_default_value, true));
+            if (ret)
+                ret->emplace((*my_i)->name, ProcParamResolved { (*my_i)->default_value, (*my_i)->linenum_default_value, true });
         } else if (has_params_in_invocation) {
             chart->Error.Error(invocation, "Procedure "+name+" has parameters that have no default and therefore require values. Ignoring procedure call.");
             chart->Error.Error((*my_i)->linenum_name, invocation, "Here is the definition of such a parameter.");
-            ret.first = true;
+            ret.reset();
             break; //report only once
         } else {
-            chart->Error.Error(invocation, "Parameter "+(*my_i)->name+ " has no default and requires a value. Ignoring procedure call.");
+            chart->Error.Error(invocation, "Parameter "+(*my_i)->name+" has no default and requires a value. Ignoring procedure call.");
             chart->Error.Error((*my_i)->linenum_name, invocation, "Here is the definition of this parameter.");
-            ret.first = true;
+            ret.reset();
         }
     }
-    if (ret.first) {
-        //if error, do not return a matched set
-        ret.second.clear();
-        return ret;
+    if (ret) {
+        //if OK, expand any references in the parameter values - but keep the hidden ones so that $$ notion is kept
+        //This is not strictly OK - we cannot know if this value will be used as a label, formatting or number format.
+        //(if it is neither, then it will contain no formatting escapes, so the below function does nothing.)
+        //But using it as a number format is probably something never happens...
+        for (auto& p : *ret)
+            StringFormat::ExpandReferences(p.second.value, chart, p.second.linenum_value, nullptr,
+                                           false, true, StringFormat::ETextType::LABEL, false);
+        //add $$ parameter as a unique value to this invocation (in a HIDDEN escape)
+        std::string v = invocation.Print();
+        v[1] = ESCAPE_CHAR_HIDDEN;
+        ret->emplace(std::piecewise_construct, std::forward_as_tuple("$$"), std::forward_as_tuple(v, invocation, true));
     }
-    //if OK, expand any references in the parameter values - but keep the hidden ones so that $$ notion is kept
-    //This is not strictly OK - we cannot know if this value will be used as a label, formatting or number format.
-    //(if it is neither, then it will contain no formatting escapes, so the below function does nothing.)
-    //But using it as a number format is probably something never happens...
-    for (auto &p : ret.second)
-        StringFormat::ExpandReferences(p.second.value, chart, p.second.linenum_value,nullptr, 
-                                       false, true, StringFormat::ETextType::LABEL, false);
-    //add $$ parameter as a unique value to this invocation (in a HIDDEN escape)
-    std::string v = invocation.Print();
-    v[1] = ESCAPE_CHAR_HIDDEN;
-    ret.second.emplace(std::piecewise_construct, std::forward_as_tuple("$$"), std::forward_as_tuple(v, invocation, true));
     return ret;
 }
 
