@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -101,10 +101,9 @@ public:
     Style &operator=(Style&&) = default;
     virtual ~Style() = default;
     virtual void Empty() {} ///<Clears all attributes from the style.
+    virtual bool IsEmpty() const noexcept { return true; } ///<Tells if we have any attriubute set or not
     virtual void MakeCompleteButText() {} ///<Sets all unset attribute to the default, except the text formatting related ones.
     virtual Style &operator +=(const Style &/*toadd*/) { return *this; }
-    virtual Style operator +(const Style &toadd) const
-    { return Style(*this) += toadd; }
     virtual bool AddAttribute(const Attribute &a, Chart *chart); ///<Adds an attribute. Returns false and emits error if not applicable.
     virtual bool DoIAcceptUnqualifiedColorAttr() const { return false; } ///<True if the style accepts the 'color' attribute.
     virtual void AttributeNames(Csh &csh) const { csh.AddStylesToHints(false, false); } ///<Adds all applicable attribute names to 'csh'
@@ -151,12 +150,15 @@ public:
     SimpleStyle &operator=(SimpleStyle &&) = default;
     ~SimpleStyle() override = default;
     void Empty() override;
+    bool IsEmpty() const noexcept override;
     void MakeCompleteButText() override;
     Style &operator +=(const Style &toadd) override;
     bool AddAttribute(const Attribute &a, Chart *chart) override;
     bool DoIAcceptUnqualifiedColorAttr() const override;
     void AttributeNames(Csh &csh) const override;
     bool AttributeValues(std::string_view attr, Csh &csh) const override;
+    void ApplyTextBefore(const StringFormat& sf);
+
 };
 
 /** An attribute with an added reference counter - works with AttrCoW.*/
@@ -251,6 +253,7 @@ public:
     SimpleStyleWithArrow &operator=(SimpleStyleWithArrow<Arrow> &&) = default;
     ~SimpleStyleWithArrow() override = default;
     void Empty() override;
+    bool IsEmpty() const noexcept override;
     void MakeCompleteButText() override;
     Style &operator +=(const Style &toadd) override;
     bool AddAttribute(const Attribute &a, Chart *chart) override;
@@ -263,7 +266,7 @@ template <typename Arrow>
 void SimpleStyleWithArrow<Arrow>::MakeCompleteButText()
 {
     SimpleStyle::MakeCompleteButText();
-    arrow.MakeComplete();
+    if (f_arrow!=EArcArrowType::NONE) arrow.MakeComplete();
 }
 
 template <typename Arrow>
@@ -273,6 +276,13 @@ void SimpleStyleWithArrow<Arrow>::Empty()
     arrow.Empty();
 }
 
+template <typename Arrow>
+bool SimpleStyleWithArrow<Arrow>::IsEmpty() const noexcept
+{
+    return SimpleStyle::IsEmpty()
+        && (f_arrow==EArcArrowType::NONE || arrow.read().IsEmpty());
+}
+
 template <typename Arrow>
 Style & SimpleStyleWithArrow<Arrow>::operator +=(const Style &toadd)
 {
@@ -288,8 +298,8 @@ template <typename Arrow>
 bool SimpleStyleWithArrow<Arrow>::AddAttribute(const Attribute &a, Chart*chart)
 {
     if (a.Is("color")) {
-        if ((color_meaning==EColorMeaning::LINE_ARROW_TEXT || 
-             color_meaning==EColorMeaning::ARROW_TEXT) && 
+        if ((color_meaning==EColorMeaning::LINE_ARROW_TEXT ||
+             color_meaning==EColorMeaning::ARROW_TEXT) &&
             f_arrow!=EArcArrowType::NONE) {
             arrow.AddAttribute(a, chart, type);
             SimpleStyle::AddAttribute(a, chart);
@@ -311,7 +321,7 @@ template <typename Arrow>
 bool SimpleStyleWithArrow<Arrow>::DoIAcceptUnqualifiedColorAttr() const
 {
     if ((color_meaning==EColorMeaning::LINE_ARROW_TEXT ||
-         color_meaning==EColorMeaning::ARROW_TEXT) && 
+         color_meaning==EColorMeaning::ARROW_TEXT) &&
         f_arrow!=EArcArrowType::NONE)
         return true;
     return SimpleStyle::DoIAcceptUnqualifiedColorAttr();
@@ -356,17 +366,21 @@ protected:
     void Dereference() {if (p==nullptr || --p->ref_count) return; delete p; p=nullptr;}
     StyleCoW(St *q) : p(q) {} //Only for descendants
 public:
-    template <class A> explicit StyleCoW(A a) : p(new St(a)) {}
-    template <class A, class B> explicit StyleCoW(A a, B b) : p(new St(a, b)) {}
-    template <class A, class B, class C> explicit StyleCoW(A a, B b, C c) : p(new St(a, b, c)) {}
+    template <class A> explicit StyleCoW(A a) : p(new St(std::move(a))) {}
+    template <class A, class B> explicit StyleCoW(A a, B b) : p(new St(std::move(a), std::move(b))) {}
+    template <class A, class B, class C> explicit StyleCoW(A a, B b, C c) : p(new St(std::move(a), std::move(b), std::move(c))) {}
     explicit StyleCoW(EStyleType tt=EStyleType::STYLE) : p(new St(tt)) {}
     explicit StyleCoW(const St &o) : p(new St(o)) { p->ref_count = 1; }
-    explicit StyleCoW(St &&o) : p(new St(std::move(o))) { p->ref_count = 1; }
-    StyleCoW(const StyleCoW &o) : p(o.p) {p->ref_count++;}
+    explicit StyleCoW(St&& o) : p(new St(std::move(o))) { p->ref_count = 1; }
+    explicit StyleCoW(std::unique_ptr<St>&& o) noexcept : p(o.release()) { p->ref_count = 1; }
+    StyleCoW(const StyleCoW& o) noexcept : p(o.p) { p->ref_count++; }
+    StyleCoW(StyleCoW&& o) noexcept : p(o.p) { o.p = nullptr; }
     ~StyleCoW() {Dereference();}
     /** Applies another style to us */
-    StyleCoW &operator +=(const StyleCoW &o) { if (&o!=this) write()+=o.read(); return *this;}
-    StyleCoW& operator = (const StyleCoW& o) { if (&o!=this) { Dereference(); p = o.p; p->ref_count++; } return *this; }
+    StyleCoW& operator +=(const StyleCoW& o) { if (&o != this && !o.read().IsEmpty()) write() += o.read(); return *this; }
+    StyleCoW& operator +=(const St& o) { if (&o != p && !o.IsEmpty()) write() += o; return *this; }
+    StyleCoW& operator = (const StyleCoW& o) noexcept { if (&o != this) { Dereference(); p = o.p; p->ref_count++; } return *this; }
+    StyleCoW& operator = (StyleCoW&& o) noexcept { if (&o != this) { Dereference(); p = o.p; o.p = nullptr; } return *this; }
     StyleCoW &operator = (const St &o) { Dereference(); p = new St(o); p->ref_count = 1;  return *this; }
     StyleCoW &operator = (St &&o) { Dereference(); p = new St(std::move(o)); p->ref_count = 1;  return *this; }
     /** Take a reference to the MscStyle object for reading */
@@ -381,9 +395,9 @@ class StyleSetBase
 {
 public:
     /**Adds the attributes contained in a style to 'csh' for hints*/
-    virtual void AttributeNamesForStyle(Csh& csh, const std::string &style) const = 0;
+    virtual void AttributeNamesForStyle(Csh& csh, std::string_view style) const = 0;
     /** Adds the values of an attribute contained in a style to 'csh' for hints*/
-    virtual bool AttributeValuesForStyle(const std::string &attr, Csh& csh, const std::string &style) const =0 ;
+    virtual bool AttributeValuesForStyle(std::string_view attr, Csh& csh, std::string_view style) const =0 ;
     /**Creates an exact duplicate of the styleset.*/
     virtual std::unique_ptr<StyleSetBase> Duplicate() const = 0;
     virtual ~StyleSetBase() = default;
@@ -394,7 +408,7 @@ public:
  * a style name is not found.
  */
 template <class St>
-class StyleSet : public std::map<std::string, StyleCoW<St>>, public StyleSetBase
+class StyleSet : public std::map<std::string, StyleCoW<St>, std::less<>>, public StyleSetBase
 {
     StyleCoW<St> defaultStyle; ///<The default style.
 public:
@@ -405,9 +419,9 @@ public:
     StyleSet &operator +=(const StyleSet&o);
     /** Merges a set of styles into us by applying members that are set in `o`*/
     StyleSet &operator +=(StyleSet &&o);
-    const StyleCoW<St> &GetStyle(const std::string &) const;
-    void AttributeNamesForStyle(Csh& csh, const std::string &style) const override;
-    bool AttributeValuesForStyle(const std::string &attr, Csh& csh, const std::string &style) const override;
+    const StyleCoW<St> &GetStyle(std::string_view) const;
+    void AttributeNamesForStyle(Csh& csh, std::string_view style) const override;
+    bool AttributeValuesForStyle(std::string_view attr, Csh& csh, std::string_view style) const override;
     std::unique_ptr<StyleSetBase> Duplicate() const override { return std::make_unique<StyleSet<St>>(*this); }
 };
 
@@ -439,7 +453,7 @@ inline StyleSet<St> &StyleSet<St>::operator+=(StyleSet<St> && o)
 
 /** Look up a style by name. Return the default style if not found.*/
 template <class St>
-inline const StyleCoW<St> &StyleSet<St>::GetStyle(const std::string &s) const
+inline const StyleCoW<St> &StyleSet<St>::GetStyle(std::string_view s) const
 {
     typename StyleSet<St>::const_iterator i = StyleSet<St>::find(s);
     if (i==StyleSet<St>::end()) return defaultStyle;
@@ -447,7 +461,7 @@ inline const StyleCoW<St> &StyleSet<St>::GetStyle(const std::string &s) const
 };
 
 template <class St>
-void StyleSet<St>::AttributeNamesForStyle(Csh& csh, const std::string &style) const
+void StyleSet<St>::AttributeNamesForStyle(Csh& csh, std::string_view style) const
 {
     auto i = this->find(style);
     if (i!=this->end())
@@ -455,7 +469,7 @@ void StyleSet<St>::AttributeNamesForStyle(Csh& csh, const std::string &style) co
 }
 
 template <class St>
-bool StyleSet<St>::AttributeValuesForStyle(const std::string &attr, Csh& csh, const std::string &style) const
+bool StyleSet<St>::AttributeValuesForStyle(std::string_view attr, Csh& csh, std::string_view style) const
 {
     auto i = this->find(style);
     if (i!=this->end())
@@ -486,7 +500,7 @@ struct ProcParamInvocation
     std::string value;         ///<value provided by the caller
     FileLineCol linenum_value; ///<location of the value at the caller site
     ProcParamInvocation(const FileLineCol &lv) : has_value(false), linenum_value(lv) {}
-    ProcParamInvocation(std::string_view v, const FileLineCol &lv, bool is_param=true) : 
+    ProcParamInvocation(std::string_view v, const FileLineCol &lv, bool /*is_param*/ = true) :
         has_value(true), value(v), linenum_value(lv) {}
 };
 
@@ -498,12 +512,9 @@ struct ProcParamResolved
     std::string value;         ///<value (provided by the caller or default)
     FileLineCol linenum_value; ///<location of the value at the caller site
     const bool is_parameter;   ///<True for procedure parameters, false for variables.
-    ProcParamResolved() = default;
-    ProcParamResolved(std::string_view v, const FileLineCol &lv, bool is_param) : 
-        value(v), linenum_value(lv), is_parameter(is_param) {}
 };
 
-using ParameterContext = std::map<std::string, ProcParamResolved>;
+using ParameterContext = std::map<std::string, ProcParamResolved, std::less<>>;
 
 /** Enumerates how a procedure definition can turn out.*/
 enum class EDefProcResult
@@ -525,13 +536,13 @@ struct Procedure
     ProcParamDefList parameters;
     static std::pair<const ProcParamDef*, const ProcParamDef*> AreAllParameterNamesUnique(const ProcParamDefList &);
     const ProcParamDef *GetProcParamDef(std::string_view name) const;
-    std::pair<bool, ParameterContext> MatchParameters(const ProcParamInvocationList &pl, const FileLineCol &invocation, Chart *chart) const;
-    std::pair<bool, ParameterContext> MatchParameters(gsl::owner<ProcParamInvocationList *>pl, const FileLineCol &invocation, Chart *chart) const
-         { if (pl) {auto ret = MatchParameters(*pl, invocation, chart); delete pl; return ret; } return MatchParameters(ProcParamInvocationList(), invocation, chart); }
+    std::optional<ParameterContext> MatchParameters(const ProcParamInvocationList &pl, const FileLineCol &invocation, Chart *chart) const;
+    std::optional<ParameterContext> MatchParameters(gsl::owner<ProcParamInvocationList *>pl, const FileLineCol &invocation, Chart *chart) const
+         { auto ret = MatchParameters(pl ? std::move(*pl): ProcParamInvocationList(), invocation, chart); delete pl; return ret; }
     void AddAttribute(const Attribute &a, Chart &chart);
 };
 
-using ProcedureSet = std::map<std::string, Procedure>;
+using ProcedureSet = std::map<std::string, Procedure, std::less<>>;
 
 
 /** An abstract base class for contexts containing style and color definitions and chart option values.
@@ -540,7 +551,7 @@ using ProcedureSet = std::map<std::string, Procedure>;
  * parse. It includes the current definition of colors, styles and a few global
  * options. Any time you open a brace in the input file, a new context is opened.
  * If you change any value of it, it will get restored at the closing brace.
- * (Individual languages may depart from this behaviour.)
+ * (Individual languages may depart from this behavior.)
  *
  * Contexts are said to be *full* if they contain a setting for all chart
  * options they contain and a complete value sets for all default styles.
@@ -554,27 +565,27 @@ class Context : public ContextParams
 {
 protected:
     /** This constructor is used to create an empty context or one set to plain.
-     * Used an initializing the first context at parse, when starting to record 
-     * a design, storing a procedure or moving a newly defined design to the design 
+     * Used an initializing the first context at parse, when starting to record
+     * a design, storing a procedure or moving a newly defined design to the design
      * store. We will have an empty text format, no colors, no procedures, no parameters.*/
-    Context(bool f, EContextParse p, const FileLineCol &l) : 
-        ContextParams(f, p), file_pos(l), num_error(0), if_condition(2), 
+    Context(bool f, EContextParse p, const FileLineCol &l) :
+        ContextParams(f, p), file_pos(l), num_error(0), if_condition(2),
         export_colors(false), export_styles(false) {}
-    /** This constructor is used when a context needs to be duplicated due to 
+    /** This constructor is used when a context needs to be duplicated due to
      * the opening of a new scope. You can change the parse mode, e.g., when
      * parsing the not-selected branch of an ifthenelse.*/
     Context(const Context &o, EContextParse p, const FileLineCol &l) :
         ContextParams(o.is_full, p), file_pos(l), num_error(o.num_error), if_condition(2),
         text(o.text), colors(o.colors), export_colors(false), export_styles(false) {}
 public:
-    const FileLineCol file_pos;   ///<The location of the first character of this context in the input file (used to unambigously differentiate contexts)
+    const FileLineCol file_pos;   ///<The location of the first character of this context in the input file (used to unambiguously differentiate contexts)
     unsigned          num_error;  ///<The number of errors at the beginning of a procedure in this context. Used during parsing and looses significance after we have finished parsing the procedure.
     unsigned          if_condition;///<If we are after an 'if' clause, what was the result of the condition: 0=false, 1=true, 2=error (already reported, none of the branches will fire)
     ParameterContext  parameters; ///<Parameters if we are inside a procedure definition or invocation (in the latter case all have value, too)
     ProcedureSet      Procedures; ///<The procedures specified in this context
     StringFormat      text;       ///<The default text style
     ColorSet          colors;     ///<The set of named colors defined here
-    bool              export_colors; ///<Whether to lift the clolors defined in this context to the enclosing context on popping this one.
+    bool              export_colors; ///<Whether to lift the colors defined in this context to the enclosing context on popping this one.
     bool              export_styles; ///<Whether to lift the styles defined in this context to the enclosing context on popping this one.
     virtual void Empty() { text.Empty(); colors.clear();  is_full = false; } ///<Make the context empty
     virtual void Plain(); ///<Set the context to the 'plain' design
@@ -586,8 +597,8 @@ public:
     virtual Style *GetStyle4Write(const std::string &name) = 0;  ///<Return a writable copy of a style
     virtual StyleNameSet GetStyleNames() const = 0; ///<Return the names of the styles included
     virtual std::unique_ptr<StyleSetBase> GetACopyOfStyles() const = 0; ///<Return a copy of all styles included
-    virtual void AttributeNamesForStyle(Csh& csh, const std::string &style) const = 0; ///<Adds the attributes contained in a style to 'csh' for hints
-    virtual bool AttributeValuesForStyle(const std::string &attr, Csh& csh, const std::string &style) const = 0; ///<Adds the values of an attribute contained in a style to 'csh' for hints
+    virtual void AttributeNamesForStyle(Csh& csh, std::string_view style) const = 0; ///<Adds the attributes contained in a style to 'csh' for hints
+    virtual bool AttributeValuesForStyle(std::string_view attr, Csh& csh, std::string_view style) const = 0; ///<Adds the values of an attribute contained in a style to 'csh' for hints
 };
 
 /** A named catalog of context pointers used to export and import designs.
@@ -596,7 +607,7 @@ public:
 using ContextPtrCatalog = std::map<std::string, std::shared_ptr<const Context>>;
 
 /** A base class for chart contexts.
- * This calss actually contains styles (and a numbering style)
+ * This class actually contains styles (and a numbering style)
  * 'St' must be a descendant of 'Style'*/
 template <class St>
 class ContextBase : public Context
@@ -605,23 +616,23 @@ public:
     StyleSet<St>            styles;      ///<The set of named styles defined here
     NumberingStyle          numberingStyle;///<The numbering style in use
     /** This constructor is used to create an empty context or one set to plain.
-     * Used an initializing the first context at parse, when starting to record 
-     * a design, storing a procedure or moving a newly defined design to the design 
+     * Used an initializing the first context at parse, when starting to record
+     * a design, storing a procedure or moving a newly defined design to the design
      * store.
      * @param [in] f If true, the context contains a value for all styles and attributes (Full)
      * @param [in] p Tells us what components to observe and how to behave during parsing.
      * @param [in] t Tells us with what content to create the context. It should not be 'COPY
      * @param [in] l The first character of the context in the input file.*/
-    ContextBase(bool f, EContextParse p, EContextCreate t, const FileLineCol &l) : Context(f, p, l) { 
+    ContextBase(bool f, EContextParse p, EContextCreate t, const FileLineCol &l) : Context(f, p, l) {
         switch (t) {
         default: _ASSERT(0); FALLTHROUGH;
         case EContextCreate::PLAIN: Plain(); break;
-        case EContextCreate::EMPTY: Empty(); break; 
+        case EContextCreate::EMPTY: Empty(); break;
         case EContextCreate::CLEAR: numberingStyle.Empty();}}
-    /** This constructor is used when a context needs to be duplicated due to 
+    /** This constructor is used when a context needs to be duplicated due to
      * the opening of a new scope. You can change the parse mode, e.g., when
      * parsing the not-selected branch of an ifthenelse.*/
-    ContextBase(const ContextBase<St> &o, EContextParse p, const FileLineCol &l) : 
+    ContextBase(const ContextBase<St> &o, EContextParse p, const FileLineCol &l) :
         Context(o, p, l), styles(o.styles), numberingStyle(o.numberingStyle) {}
     /** Semi copy operator: copy only user-defined context content. If o is full, we overwrite us, else just merge.*/
     void ApplyContextContent(const ContextBase<St> &o);
@@ -634,9 +645,9 @@ public:
     StyleNameSet GetStyleNames() const override;
     std::unique_ptr<StyleSetBase> GetACopyOfStyles() const override { return styles.Duplicate(); }
     /** Add the attribute names a particular style can take to hints.*/
-    void AttributeNamesForStyle(Csh& csh, const std::string &style) const override { styles.AttributeNamesForStyle(csh, style); }
-    /** Add the possible attribute values for an attribute to hints, if allowed to be set for the particluar style.*/
-    bool AttributeValuesForStyle(const std::string &attr, Csh& csh, const std::string &style) const override { return styles.AttributeValuesForStyle(attr, csh, style); }
+    void AttributeNamesForStyle(Csh& csh, std::string_view style) const override { styles.AttributeNamesForStyle(csh, style); }
+    /** Add the possible attribute values for an attribute to hints, if allowed to be set for the particular style.*/
+    bool AttributeValuesForStyle(std::string_view attr, Csh& csh, std::string_view style) const override { return styles.AttributeValuesForStyle(attr, csh, style); }
 };
 
 
