@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -33,7 +33,8 @@
 #include "canvas.h"
 #include "flowchart.h"
 #include "flowcsh.h"
-#include "steps.h"                                                                    
+#include "steps.h"
+#include "flow_parser_compile.h"
 
 using namespace flow;
 
@@ -48,9 +49,9 @@ std::string FlowChart::GetLanguageDefaultText() const {
         "else goto turn;\n";
 }
 
-std::unique_ptr<Csh> FlowChart::CshFactory(Csh::FileListProc proc, void *param) const
+std::unique_ptr<Csh> FlowChart::CshFactory(Csh::FileListProc proc) const
 {
-    return std::make_unique<FlowCsh>(proc, param);
+    return std::make_unique<FlowCsh>(proc);
 }
 
 FlowChart::FlowChart(FileReadProcedure *p, void *param) : ChartBase(p, param)
@@ -102,7 +103,7 @@ std::string FlowChart::SerializeGUIState() const
     return std::string();
 }
 
-bool FlowChart::ControlClicked(Element *arc, EGUIControlType t)
+bool FlowChart::ControlClicked(Element*, EGUIControlType)
 {
     return false;
 }
@@ -136,13 +137,14 @@ unsigned FlowChart::ParseText(std::string_view input, std::string_view filename)
     unsigned ret = current_file = Error.AddFile(filename);
     //if you support progres::
     //Progress.RegisterParse(len);
-    FlowParse(*this, input.data(), unsigned(input.length()));
+    sv_reader<false> reader(input, current_file);
+    procedure_parse_helper proc_helper;
+    flow_compile_parse(*this, proc_helper, reader); //return value ignored
     return ret;
 }
 
 
-void FlowChart::CompleteParse(bool autoPaginate, bool addHeading,
-                              XY pageSize, bool fitWidth, bool collectLinkInfo)
+void FlowChart::CompleteParse(bool, bool, XY, bool, bool)
 {
     Canvas canvas(Canvas::Empty::Query);
     pageBreakData.clear();
@@ -152,7 +154,7 @@ void FlowChart::CompleteParse(bool autoPaginate, bool addHeading,
     XY crTexSize = Label(copyrightText, canvas, Shapes, sf).getTextWidthHeight().RoundUp();
     copyrightTextHeight = crTexSize.y;
 
-    //Transform to lists 
+    //Transform to lists
     ConvertToSeries();
 
     //Arrange blocks
@@ -177,9 +179,9 @@ void FlowChart::CompleteParse(bool autoPaginate, bool addHeading,
 }
 
 void FlowChart::BestOrdering() {
-    std::vector<size_t> map(series.size()), found; //map[i] tells the current index of column ID 'i'    
+    std::vector<size_t> map(series.size()), found; //map[i] tells the current index of column ID 'i'
     for (size_t u = 0; u<series.size(); u++) map[u] = u;
-    struct Diff { 
+    struct Diff {
         size_t diff=0, sub_diff=0;
         constexpr auto operator <=>(const Diff&) const noexcept = default;
         Diff& operator +=(const Diff& o) noexcept { diff += o.diff; sub_diff += o.sub_diff; return *this; }
@@ -234,10 +236,10 @@ void FlowChart::BestOrdering() {
                 if (a->target.col>=0) a->target.col = found[a->target.col];
             }
         }
-    //Now re-order the columns    
+    //Now re-order the columns
     //start by inverting the mapping in 'found' into 'map'
     for (size_t u = 0; u<series.size(); u++) map[found[u]] = u;
-    //map[i] now tells us the original column ID of the column 
+    //map[i] now tells us the original column ID of the column
     //that goes to the ith index (col) in the new order
     std::vector<StepList> new_series;
     new_series.reserve(series.size());
@@ -249,7 +251,7 @@ void FlowChart::BestOrdering() {
 size_t FlowChart::ConvertToSeries(FElementList &list, bool allocate_new)
 {
     //Go through the list and sanitize branches
-    for (auto i = list.begin(); i!=list.end(); ++i) 
+    for (auto i = list.begin(); i!=list.end(); ++i)
         if (Branch* p = dynamic_cast<Branch*>(i->get()))
             if (p && p->instructions.empty() && std::next(i) != list.end()
                     && dynamic_cast<Branch*>(std::next(i)->get()) == nullptr)
@@ -390,7 +392,7 @@ void FlowChart::Layout() {
         total = Block(0, 0, 0, 0);
         return;
     }
-    _ASSERT(IsHorizontal(sdir)!=IsHorizontal(ndir)); //ndir and sdir must be perpendicular 
+    _ASSERT(IsHorizontal(sdir)!=IsHorizontal(ndir)); //ndir and sdir must be perpendicular
     Canvas canvas(Canvas::Empty::Query);
     //Calculate node sizes
     for (auto& sl : series)
