@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -28,7 +28,7 @@
 
 /** This is the namespace containing flowchart elements, except parsing.*/
 namespace flow {
-              
+
 class FlowElement;
 
 /** The chart class.*/
@@ -37,30 +37,30 @@ class FlowChart : public ChartBase<FlowContext>
 public:
     /** @name Language attributes
     * @{ */
-    std::string GetLanguageDescription() const override { return "Flow Chart"; } ///<Returns the human-readable (UTF-8) short description 
+    std::string GetLanguageDescription() const override { return "Flow Chart"; } ///<Returns the human-readable (UTF-8) short description
     std::vector<std::string> GetLanguageExtensions() const override { return{"flow"}; } ///<Returns a set of extensions in order of preference
     std::string GetLanguageEntityName() const override { return "node"; }
     std::string GetLanguageDefaultText() const override; ///<Returns the default text of the chart in UTF-8
     bool GetLanguageHasAutoheading() const override { return false; } ///<True if the chart has automatic headings
     bool GetLanguageHasElementControls() const override { return false; } ///<True if the elements of the chart support GUI controls
-    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc, void *param) const override;
+    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc) const override;
     std::map<std::string, std::string> RegisterLibraries() const override { return {}; };
     /** @} */
 public:
     const double defaultGap = 20;
     const double arrowGap = 20;
     const EDir sdir = EDir::RIGHT; //the series direction
-    const EDir ndir = EDir::DOWN;  //the nodes direction whithin a series
+    const EDir ndir = EDir::DOWN;  //the nodes direction within a series
     EDir ConvertDir(ELogicalDir ld) const { return flow::ConvertDir(ld, sdir, ndir); }
     FElementList steps;
     std::vector<StepList> series;
 
     FlowChart(FileReadProcedure *p, void *param);
     ~FlowChart() override = default;
-                 
-    std::unique_ptr<FlowElement> PopContext();    ///<Closes a scope 
 
-    static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param) 
+    std::unique_ptr<FlowElement> PopContext();    ///<Closes a scope
+
+    static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param)
         { return std::make_unique<FlowChart>(p, param); }
 
     bool AddCommandLineArg(const std::string & arg) override;
@@ -105,7 +105,4 @@ public:
 
 using flow::FlowChart;
 
-/** Yacc-generated function to parse a graph. defined in gv_lang.cpp. */
-void FlowParse(FlowChart &, const char *buff, unsigned len);
-
 #endif
\ No newline at end of file
