@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -25,12 +25,13 @@
 #include "flowcsh.h"
 #include "flowstyle.h"
 #include "steps.h"
+#include "flow_parser_csh.h"
 
 using namespace flow;
 
 
-FlowCsh::FlowCsh(Csh::FileListProc proc, void *param) :
-    Csh(FlowContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc, param)
+FlowCsh::FlowCsh(Csh::FileListProc proc) :
+    Csh(FlowContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc)
 {
     FillNamesHints();
 }
@@ -54,17 +55,16 @@ void FlowCsh::FillNamesHints()
 /** Parse chart text for color syntax and hint info
  *
  * @param [in] input The chart text
- * @param [in] len The length of the chart text
  * @param [in] cursor_p The current position of the cursor.
- * @param [in] scheme The number of the color syntax scheme to use.
  * @param [in] pedantic The initial value of the pedantic chart option.
    (We ignore it in CSH parsing.)*/
-void FlowCsh::ParseText(const std::string& input, int cursor_p, bool pedantic)
+void FlowCsh::ParseText(std::string&& input, int cursor_p, bool /*pedantic*/)
 {
     //initialize data struct
-    BeforeYaccParse(input, cursor_p);
+    BeforeYaccParse(std::move(input), cursor_p);
     //call parsing
-    FlowCshParse(*this, input_text.c_str(), unsigned(input_text.length()+1)); //include terminating zero
+    sv_reader<true> reader(input_text, 0);
+    flow_csh_parse(*this, reader); //return value ignored
     //Tidy up afterwards
     AfterYaccParse();
 }
