@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -31,19 +31,16 @@ namespace flow {
 class FlowCsh : public Csh
 {
 public:
-    FlowCsh(Csh::FileListProc proc, void *param);
+    explicit FlowCsh(Csh::FileListProc proc);
     ~FlowCsh() override = default;
     std::unique_ptr<Csh> Clone() const override { return std::make_unique<FlowCsh>(*this); }
 
     // Inherited via Csh
     void FillNamesHints() override;
-    void ParseText(const std::string& input, int cursor_p, bool pedantic) override;
+    void ParseText(std::string&& input, int cursor_p, bool pedantic) override;
 
 };
 
 }; //namespace
 
-/** Yacc generated function to do the csh parsing of a graph. defined in gv_csh_lang.cpp */
-extern void FlowCshParse(flow::FlowCsh &csh, const char *buff, unsigned len);
-
 #endif
\ No newline at end of file
