@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -21,7 +21,6 @@
 
 
 #include <cstring>
-#include "parse_tools.h" //for remove_head_tail_whitespace()
 #include "flowstyle.h"
 #include "flowchart.h"
 
@@ -63,7 +62,7 @@ void FlowStyle::MakeCompleteButText()
 
 Style &FlowStyle::operator += (const Style &toadd)
 {
-    SimpleStyleWithArrow<TwoArrowHeads>::operator+(toadd);
+    SimpleStyleWithArrow<TwoArrowHeads>::operator+=(toadd);
     const FlowStyle* p = dynamic_cast<const FlowStyle *>(&toadd);
     if (p) {
         //Write your own code adding an FlowStyle to another
