@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -66,12 +66,13 @@ public:
     FlowStyle(FlowStyle&&) = default;
     FlowStyle &operator=(const FlowStyle&) = default;
     FlowStyle &operator=(FlowStyle&&) = default;
-    virtual void Empty() override;
-    virtual void MakeCompleteButText() override;
-    virtual Style &operator +=(const Style &toadd) override;
-    virtual bool AddAttribute(const Attribute &a, Chart *) override;
-    virtual void AttributeNames(Csh &csh) const override;
-    virtual bool AttributeValues(std::string_view attr, Csh &csh) const override;
+    void Empty() override;
+    bool IsEmpty() const noexcept override { return  SimpleStyleWithArrow<TwoArrowHeads>::IsEmpty(); }
+    void MakeCompleteButText() override;
+    Style &operator +=(const Style &toadd) override;
+    bool AddAttribute(const Attribute &a, Chart *) override;
+    void AttributeNames(Csh &csh) const override;
+    bool AttributeValues(std::string_view attr, Csh &csh) const override;
 };
 
 /** Context for Graphs. Just a styleset and colors, really.*/
