@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -45,7 +45,7 @@ void FlowElement::AttributeNames(Csh &csh)
         EHintType::ATTR_NAME));
 }
 
-bool FlowElement::AttributeValues(const std::string &attr, Csh &csh)
+bool FlowElement::AttributeValues(std::string_view attr, Csh &csh)
 {
     if (CaseInsensitiveEqual(attr, "label")) 
         return true;
@@ -94,7 +94,7 @@ void Arrow::AttributeNames(Csh &csh)
     csh.AttributeNamesForStyle("arrow");
 }
 
-bool Arrow::AttributeValues(const std::string &attr, Csh &csh)
+bool Arrow::AttributeValues(std::string_view attr, Csh &csh)
 {
     if (FlowElement::AttributeValues(attr, csh)) return true;
     return csh.AttributeValuesForStyle(attr, "arrow");
@@ -208,7 +208,7 @@ void Step::AttributeNames(Csh &csh)
     csh.AttributeNamesForStyle("step");
 }
 
-bool Step::AttributeValues(const std::string &attr, Csh &csh)
+bool Step::AttributeValues(std::string_view attr, Csh &csh)
 {
     if (FlowElement::AttributeValues(attr, csh)) return true;
     return csh.AttributeValuesForStyle(attr, "step");
