@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -39,7 +39,7 @@ public:
     Label parsed_label;
     double label_sx, label_dx, label_cx, label_y;
     FlowElement(FlowChart *c) : chart(c) {}
-    virtual void AddRefName(const char *name) = 0;
+    virtual void AddRefName(std::string_view name) = 0;
     /** Applies an Attribute to us.
      * We accept no attributes here.
      * @param [in] a The attribute to add.
@@ -48,13 +48,13 @@ public:
     /** Add the attribute names we take to `csh`.*/
     static void AttributeNames(Csh &);
     /** Add a list of possible attribute value names to `csh` for attribute `attr`.*/
-    static bool AttributeValues(const std::string &/*attr*/, Csh &/*csh*/);
+    static bool AttributeValues(std::string_view /*attr*/, Csh &/*csh*/);
     virtual FlowElement *AddAttributeList(AttributeList *al);
     /** Adds to a steplist, returns true if we continue, false if gotoed away. */
     virtual bool AddToStepList(size_t col, std::unique_ptr<FlowElement> &&self) = 0;
     virtual void PreLayout(Canvas &c);
     virtual void FlowPostPosProcess(Canvas &c);
-    virtual void Draw(Canvas &c) {}
+    virtual void Draw(Canvas&) {}
 };
 
 typedef UPtrList<FlowElement> FElementList;
@@ -115,9 +115,9 @@ struct Pos {
 struct KneePos
 {
     Pos col, element;
-    KneePos(int c, int c_side, int e, int e_side) noexcept 
+    KneePos(int c, int c_side, int e, int e_side) noexcept
         : col{c, c_side}, element{e, e_side} {}
-    KneePos(const NodeRef &r, int c_side=0, int e_side=0) noexcept 
+    KneePos(const NodeRef &r, int c_side=0, int e_side=0) noexcept
         : col{r.col, c_side}, element(r.element, e_side) {}
 };
 
@@ -132,20 +132,20 @@ public:
     Path path;
     TwoArrowHeadsInstance arrowheads;
     Arrow(FlowChart *f);
-    virtual void AddRefName(const char *name) override { _ASSERT(0); }
+    virtual void AddRefName(std::string_view) override { _ASSERT(0); }
     virtual bool IsBranch() const noexcept { return false; };
     FlowElement *AddAttributeList(AttributeList *al) override;
     /** Add the attribute names we take to `csh`.*/
     static void AttributeNames(Csh &);
     /** Add a list of possible attribute value names to `csh` for attribute `attr`.*/
-    static bool AttributeValues(const std::string &/*attr*/, Csh &/*csh*/);
-    bool AddToStepList(size_t col, std::unique_ptr<FlowElement> &&self) override { _ASSERT(0); return false; }
+    static bool AttributeValues(std::string_view/*attr*/, Csh &/*csh*/);
+    bool AddToStepList(size_t, std::unique_ptr<FlowElement>&&) override { _ASSERT(0); return false; }
     void Route();
     void FlowPostPosProcess(Canvas &) override;
     void Draw(Canvas &c) override;
 };
 
-/** Represents one step of a flowchart. 
+/** Represents one step of a flowchart.
  * An action (box) a condition (if) or star/stop*/
 class Step : public FlowElement
 {
@@ -153,7 +153,7 @@ public:
     enum Type {BOX, IF, START, STOP} type;
     struct KeywordHelper {
         Type type;
-        char* keyword;
+        str_view keyword;
         bool must_have_label;
     };
     std::unique_ptr<Arrow> prev_arrow; //The arrow leading to us
@@ -163,11 +163,11 @@ public:
     NodeRef my_location;
     Contour clip_by_arrows;
     Step(FlowChart *f, Type t);
-    void AddRefName(const char *name) override { if (name) refname = name; }
+    void AddRefName(std::string_view name) override { refname = name; }
     /** Add the attribute names we take to `csh`.*/
     static void AttributeNames(Csh &);
     /** Add a list of possible attribute value names to `csh` for attribute `attr`.*/
-    static bool AttributeValues(const std::string &/*attr*/, Csh &/*csh*/);
+    static bool AttributeValues(std::string_view/*attr*/, Csh &/*csh*/);
     bool AddToStepList(size_t col, std::unique_ptr<FlowElement> &&self) override;
     void PreLayout(Canvas & canvas) override;  //creates outline in Area at pos(0,0)
     virtual void ShiftBy(const XY &xy) override { Element::ShiftBy(xy); center += xy; outer_edge.Shift(xy); }
@@ -181,7 +181,7 @@ class Repeat : public Arrow
 public:
     FElementList instructions;
     Repeat(FlowChart *f, FElementList *fl) : Arrow(f) { if (fl) { instructions = std::move(*fl); delete fl; } }
-    void AddRefName(const char *name) override { if (instructions.size())  instructions.front()->AddRefName(name); }
+    void AddRefName(std::string_view name) override { if (instructions.size())  instructions.front()->AddRefName(name); }
     bool AddToStepList(size_t col, std::unique_ptr<FlowElement> &&self) override;
     void PreLayout(Canvas & canvas) override;
 };
@@ -192,7 +192,7 @@ public:
     FElementList instructions;
     Branch(FlowChart *f) : Arrow(f) {}
     bool IsBranch() const noexcept override { return true; }
-    Branch *AddLabel(const char *l) { if (l) label = l; return this; }
+    Branch *AddLabel(std::string_view l) { label = l; return this; }
     Branch *AddSteps(FElementList *l) { if (l) { instructions.Append(l); delete l; } return this; }
     Branch *AddStep(FlowElement *l) { if (l) { instructions.Append(l); } return this; }
     bool AddToStepList(size_t col, std::unique_ptr<FlowElement> &&self) override;
@@ -204,7 +204,7 @@ class Goto : public Arrow
 {
 public:
     const std::string target_str;
-    Goto(FlowChart *f, const char *step) : Arrow(f), target_str(step ? step : "") {}
+    Goto(FlowChart *f, std::string_view step) : Arrow(f), target_str(step) {}
     bool AddToStepList(size_t col, std::unique_ptr<FlowElement> &&self) override;
 };
 
