@@ -112,6 +112,7 @@ void TextEditor::SetPalette(ColorSyntaxAppearance(&P)[COLOR_MAX]) {
 		mPaletteBase[i].color = IM_COL32(P[i].r, P[i].g, P[i].b, 255);
 		mPaletteBase[i].bold = bool(P[i].effects & COLOR_FLAG_BOLD);
 		mPaletteBase[i].italic = bool(P[i].effects & COLOR_FLAG_ITALICS);
+		mPaletteBase[i].underlined = bool(P[i].effects & COLOR_FLAG_UNDERLINE);
 	}
 }
 
@@ -129,6 +130,8 @@ void TextEditor::SetFonts(ImFont *regular, ImFont *bold, ImFont *italic, ImFont
 //Each newline counts as one character
 //TABs are counted as one character in 'pos' but as mTabSize in the ret value
 TextEditor::Coordinates TextEditor::GetCoordinate(int char_pos) noexcept {
+	if (char_pos < 0) return {0,0};
+	if (mLines.empty()) return {0,0};
 	int line = 0;
 	while (line<(int)mLines.size()) {
 		int chars_in_line = mLines[line].num_chars();
@@ -137,7 +140,8 @@ TextEditor::Coordinates TextEditor::GetCoordinate(int char_pos) noexcept {
 		char_pos -= chars_in_line+1; //+1 for newline
 		line++;
 	}
-	return Coordinates::Invalid();
+	//beyond the end - return pos after last char
+	return {int(mLines.size() - 1), int(mLines.back().num_chars())};
 }
 
 
@@ -1038,6 +1042,10 @@ void TextEditor::Render()
 									  newOffset, prevStyle.color, mLineBuffer.c_str());
 					auto textSize = GetFont(prevStyle)
 						->CalcTextSizeA(mFontSize, FLT_MAX, -1.0f, mLineBuffer.c_str(), nullptr, nullptr);
+					const float zoom = mFontSize/16.f;
+					const float y = mFontSize*0.85f;
+					if (prevStyle.underlined)
+						drawList->AddLine(ImVec2(newOffset.x, newOffset.y+y), ImVec2(newOffset.x+textSize.x, newOffset.y+y), prevStyle.color, zoom);
 					bufferOffset.x += textSize.x;
 					mLineBuffer.clear();
 				}
@@ -1100,7 +1108,7 @@ void TextEditor::Render()
 				const float xFrom = TextDistanceToLineStart(errorIt->first);
 				const float xTill = TextDistanceToLineStart({errorIt->first.mLine, errorIt->first.mColumn + errorIt->second.first});
 				const auto hStart = ImVec2(textScreenPos.x+xFrom, textScreenPos.y);
-				const auto hEnd = ImVec2(textScreenPos.x+xTill, textScreenPos.y+mCharAdvance.y);
+				const auto hEnd = ImVec2(textScreenPos.x+xTill + (xTill-xFrom<1 ?  spaceSize : 0), textScreenPos.y+mCharAdvance.y);
 				constexpr float net_thick = 2.0f;
 				const float thick = net_thick*mScale;
 				std::vector<ImVec2> points;
@@ -1201,6 +1209,8 @@ void TextEditor::SetText(const std::string & aText)
 	mUndoBuffer.clear();
 	mUndoIndex = 0;
 
+	SetSelection(mState.mSelectionStart, mState.mSelectionEnd); //normalize selection
+
 	Colorize();
 }
 
@@ -1233,6 +1243,8 @@ void TextEditor::SetTextLines(const std::vector<std::string> & aLines)
 	mUndoBuffer.clear();
 	mUndoIndex = 0;
 
+	SetSelection(mState.mSelectionStart, mState.mSelectionEnd); //normalize selection
+
 	Colorize();
 }
 
