@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -42,6 +42,7 @@
 #include "canvas.h"
 #include "graphchart.h"
 #include "gvcsh.h"
+#include "graph_parser_compile.h"
 
 using namespace graph;
 
@@ -134,9 +135,9 @@ void GraphChart::BeforePushScope()
         bool created;
         SubGraph *s;
         if (parse_stack.size())
-            s = GetCurrentGraph().CreateSubGraph<CollapsableSubGraph>(parse_stack.back(), last_subgraphname.c_str(), cluster, &created);
+            s = GetCurrentGraph().CreateSubGraph<CollapsableSubGraph>(parse_stack.back(), last_subgraphname, cluster, &created);
         else
-            s = GetCurrentGraph().CreateSubGraph<CollapsableSubGraph>(last_subgraphname.c_str(), cluster, &created);
+            s = GetCurrentGraph().CreateSubGraph<CollapsableSubGraph>(last_subgraphname, cluster, &created);
         parse_stack.push_back(s);
         if (cluster) {
             if (created) {
@@ -178,11 +179,11 @@ ClusterCollapseMap GraphChart::GetAllCollapsedGUIState() const
     return ret;
 }
 
-void GraphChart::NewGraph(bool d, bool s, const char *name)
+void GraphChart::NewGraph(bool d, bool s, std::string_view name)
 {
     Graphs.emplace_back(d, s);
     Graph &GG = Graphs.back();
-    if (name) GG.name = name;
+    if (name.size()) GG.name = name;
     else GG.name << "@" << Graphs.size();
     lst_scope_is_graph = true;
     cluser_counter = 0;
@@ -202,7 +203,7 @@ void GraphChart::FinalizeGraph(bool keep)
         Graphs.pop_back();
 }
 
-GraphNode *GraphChart::CreateNode(Graph &g, const char *name)
+GraphNode *GraphChart::CreateNode(Graph &g, std::string_view name)
 {
     bool created;
     GraphNode * p = g.CreateNode<GraphNode>(name, &created);
@@ -215,7 +216,7 @@ GraphNode *GraphChart::CreateNode(Graph &g, const char *name)
     return p;
 }
 
-GraphNode *GraphChart::CreateNode(SubGraph &g, const char *name)
+GraphNode *GraphChart::CreateNode(SubGraph &g, std::string_view name)
 {
     bool created;
     GraphNode * p = g.graph.CreateNode<GraphNode>(&g, name, &created);
@@ -239,7 +240,7 @@ GraphEdge *GraphChart::CreateEdge(GraphNode *_1, GraphNode *_2,
     GraphEdge * p = _1->graph.CreateEdge<GraphEdge>(_1, _2, p1, p2, port_file_pos1, port_file_pos2, &created);
     if (created) {
         p->style = MyCurrentContext().styles["edge"];
-        p->style += MyCurrentContext().styles[p->type.AsText()];
+        p->style += MyCurrentContext().styles[std::string(p->type.AsText())];
     }
     return p;
 }
@@ -327,7 +328,7 @@ void GraphChart::ApplyForcedLayout(const string & name)
 }
 
 void GraphChart::AddAttributeListToStyleList(const GraphStyle * s,
-    const std::list<std::string>* styles)
+    const std::vector<std::string>* styles)
 {
     if (s==nullptr) return;
     AttributeList *al = new AttributeList;
@@ -405,7 +406,9 @@ unsigned GraphChart::ParseText(std::string_view input, std::string_view filename
     parse_stack.clear();
     last_subgraphname.clear();
     unsigned ret = current_file = Error.AddFile(filename);
-    GraphParse(*this, input.data(), unsigned(input.length()));
+    sv_reader<false> reader(input, current_file);
+    gv_parser_helper_compile proc_helper;
+    graph_compile_parse(*this, proc_helper, reader); //return value ignored
     return ret;
 }
 
@@ -439,7 +442,7 @@ void CollapsibleGraph::Collapse(std::set<std::string>& clusters)
         SubGraph *ret = GetSubGraph(c);
         _ASSERT(ret);
         //create replacement node
-        GraphNode *replacement = CreateNode<CollapsableNode>(ret->parent, ret->name.c_str(), nullptr);
+        GraphNode *replacement = CreateNode<CollapsableNode>(ret->parent, ret->name, nullptr);
         replacement->style = ret->style; //should be based on cluster_collapse, see PushContext()
         replacement->file_pos = ret->file_pos;
         //take note of which nodes do we replace to the one created above
@@ -558,8 +561,10 @@ void GraphChart::CompleteParse(bool autoPaginate, bool addHeading,
 }
 
 
-void GraphChart::CollectIsMapElements(Canvas & /*canvas*/)
+void GraphChart::CollectIsMapElements(Canvas& canvas)
 {
+    for (auto& GG : Graphs)
+        GG.GraphCollectIsMapElements(canvas, *this);
 }
 
 void GraphChart::RegisterAllLabels()
@@ -717,9 +722,9 @@ std::string GraphChart::GetLanguageDefaultText() const
         "}\n";
 }
 
-std::unique_ptr<Csh> GraphChart::CshFactory(Csh::FileListProc proc, void *param) const
+std::unique_ptr<Csh> GraphChart::CshFactory(Csh::FileListProc proc) const
 {
-    return std::make_unique<GraphCsh>(proc, param);
+    return std::make_unique<GraphCsh>(proc);
 }
 
 std::map<std::string, std::string> GraphChart::RegisterLibraries() const
