@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -32,15 +32,15 @@ namespace graph {
 class CollapsableNode : public GraphNode
 {
 public:
-    CollapsableNode(Graph &g, const char *name) : GraphNode(g, name) { controls.push_back(EGUIControlType::EXPAND); }
+    CollapsableNode(Graph &g, std::string_view name) : GraphNode(g, name) { controls.push_back(EGUIControlType::EXPAND); }
 };
 
 /** A subgraph that can be collapsed (adds GUI controls)*/
 class CollapsableSubGraph : public SubGraph
 {
 public:
-    CollapsableSubGraph(Graph &g, const char *n, bool cluster) : SubGraph(g, n, cluster) { controls.push_back(EGUIControlType::COLLAPSE); }
-    CollapsableSubGraph(SubGraph *g, const char *n, bool cluster) : SubGraph(g, n, cluster) { controls.push_back(EGUIControlType::COLLAPSE); }
+    CollapsableSubGraph(Graph &g, std::string_view n, bool cluster) : SubGraph(g, n, cluster) { controls.push_back(EGUIControlType::COLLAPSE); }
+    CollapsableSubGraph(SubGraph *g, std::string_view n, bool cluster) : SubGraph(g, n, cluster) { controls.push_back(EGUIControlType::COLLAPSE); }
 };
 
 /** A graph that can contain collapsible subgraphs. */
@@ -74,20 +74,20 @@ class GraphChart : public ChartBase<GraphContext>
 public:
     /** @name Language attributes
     * @{ */
-    std::string GetLanguageDescription() const override { return "Graphviz Graph"; } ///<Returns the human-readable (UTF-8) short description 
-    std::vector<std::string> GetLanguageExtensions() const override { return{"graph", "dot"}; } ///<Returns a set of extensions in order of preference
+    std::string GetLanguageDescription() const override { return "Graphviz Graph"; } ///<Returns the human-readable (UTF-8) short description
+    std::vector<std::string> GetLanguageExtensions() const override { return{"gv", "graph", "dot"}; } ///<Returns a set of extensions in order of preference
     std::string GetLanguageEntityName() const override { return "node"; }
     std::string GetLanguageDefaultText() const override; ///<Returns the default text of the chart in UTF-8
     bool GetLanguageHasAutoheading() const override { return false; } ///<True if the chart has automatic headings
     bool GetLanguageHasElementControls() const override { return true; } ///<True if the elements of the chart support GUI controls
-    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc, void *param) const override;
+    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc) const override;
     std::map<std::string, std::string> RegisterLibraries() const override;
     /** @} */
 protected:
     bool had_layout = false;  ///<True if our graph had been laid out.
     void BeforePushScope();   ///<Creates graph or subgraph before opening the scope. Uses last_** variables to determine what to do
 public:
-    /** @name variables used during parsing 
+    /** @name variables used during parsing
     * @{ */
     std::list<SubGraph*> parse_stack; ///<The subgraphs we are in the middle of defining. Empty if we fill the main graph (just for parsing)
     std::string        last_subgraphname;  ///<The name of the lastly opened subgraph (just for parsing)
@@ -112,14 +112,14 @@ public:
     ClusterCollapseMap GetAllCollapsedGUIState() const;
     ClusterCollapseMap GetAllExpandedGUIState() const { return ClusterCollapseMap(); }
 
-    void NewGraph(bool d, bool s, const char *name);   ///<Starts the parsing of a new graph (called after completing the parse of its header)
+    void NewGraph(bool d, bool s, std::string_view name = {});   ///<Starts the parsing of a new graph (called after completing the parse of its header)
     Graph &GetCurrentGraph() { if (Graphs.size()==0) Graphs.emplace_back(true,false); return Graphs.back(); } ///<The current graph we put stuff into.
     void FinalizeGraph(bool keep); ///<Called after completing the parse of a graph. If 'keep' is false, we discard the graph.
 
     static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param) { return std::make_unique<GraphChart>(p, param); }
 
-    GraphNode * CreateNode(Graph &g, const char *name);        ///<Creates a new node into the top graph.
-    GraphNode * CreateNode(SubGraph &g, const char *name); ///<Creates a new node into a subgraph.
+    GraphNode * CreateNode(Graph &g, std::string_view name);        ///<Creates a new node into the top graph.
+    GraphNode * CreateNode(SubGraph &g, std::string_view name); ///<Creates a new node into a subgraph.
     /** Creates an edge between two nodes. */
     GraphEdge *CreateEdge(GraphNode *_1, GraphNode *_2, const std::string &p1, const std::string &p2,
                               const FileLineColRange &port_file_pos1, const FileLineColRange &port_file_pos2);
@@ -136,7 +136,7 @@ public:
     void SetPedantic(bool pedantic) noexcept override { if (Contexts.size()) Contexts.back().pedantic = pedantic; }
 
     /** Adds an attribute list (in the format of a GraphStyle) to each of a list of styles.*/
-    void AddAttributeListToStyleList(const GraphStyle *s, const std::list<std::string> *styles);
+    void AddAttributeListToStyleList(const GraphStyle *s, const std::vector<std::string> *styles);
     bool AddChartOption(const Attribute &a);
     void DoAttributeInstruction(GraphStyle::EGraphElementType t, gsl::owner<GraphStyle*> al);
     unsigned ParseText(std::string_view input, std::string_view filename) override;
@@ -153,8 +153,4 @@ public:
 
 using graph::GraphChart;
 
-/** Yacc-generated function to parse a graph. defined in gv_lang.cpp. */
-void GraphParse(GraphChart &, const char *buff, unsigned len); 
-
-
 #endif
\ No newline at end of file
