@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 #include <cstring>
 #include "gvcsh.h"
 #include "gvstyle.h"
+#include "graph_parser_csh.h"
 
 using namespace graph;
 
@@ -49,15 +50,15 @@ bool GraphCsh::DoHintLocated(EHintSourceType hsource, std::string_view a_name)
 }
 
 
-GraphCsh::GraphCsh(Csh::FileListProc proc, void *param) :
-    Csh(GraphContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc, param)
+GraphCsh::GraphCsh(Csh::FileListProc proc) :
+    Csh(GraphContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc)
 {
     FillNamesHints();
 }
 
 void GraphCsh::FillNamesHints()
 {
-    //these are used only when node definition is possible in AddCSH_KeywordOrEntity(), so 
+    //these are used only when node definition is possible in AddCSH_KeywordOrEntity(), so
     //we do not add 'strict'
     AddOptionsToHints();
     MoveHintsToOptionNames();
@@ -77,21 +78,23 @@ void GraphCsh::FillNamesHints()
  * @param [in] cursor_p The current position of the cursor.
  * @param [in] pedantic The initial value of the pedantic chart option.
    (We ignore it in CSH parsing.)*/
-void GraphCsh::ParseText(const std::string& input, int cursor_p, bool pedantic)
+void GraphCsh::ParseText(std::string&& input, int cursor_p, bool pedantic)
 {
     //initialize data struct
-    BeforeYaccParse(input, cursor_p);
+    BeforeYaccParse(std::move(input), cursor_p);
     (void)pedantic;
     //call parsing
-    GraphCshParse(*this, input_text.c_str(), unsigned(input_text.length()));
+    sv_reader<true> reader(input_text, 0);
+    gv_parser_helper_csh proc_helper;
+    graph_csh_parse(*this, proc_helper, reader); //return value ignored
     //Tidy up afterwards
     AfterYaccParse();
 }
 
-void GraphCsh::AddCSH_Subgraphname(const CshPos & pos, const char * name)
+void GraphCsh::AddCSH_Subgraphname(const CshPos & pos, std::string_view name)
 {
     //TODO: Check if this is == or !=
-    if (strncmp(name, "cluster", 7)) {
+    if (name== "cluster") {
         AddCSH(CshPos(pos.first_pos, pos.first_pos+6), COLOR_ATTRVALUE_EMPH);
         AddCSH(CshPos(pos.first_pos+8, pos.last_pos), COLOR_ATTRVALUE);
     } else {
@@ -99,11 +102,11 @@ void GraphCsh::AddCSH_Subgraphname(const CshPos & pos, const char * name)
     }
 }
 
-const char compass_values[][3] = {"n", "ne", "e", "se", "s", "sw", "w", "nw", "c", "_", ""};
-void GraphCsh::AddCSH_Compass(const CshPos & pos, const char * name)
+static const std::string_view compass_values[] = {"n", "ne", "e", "se", "s", "sw", "w", "nw", "c", "_", ""};
+void GraphCsh::AddCSH_Compass(const CshPos & pos, std::string_view name)
 {
     for (auto c : compass_values)
-        if (strncmp(name, c, 3)==0) {
+        if (name == c) {
             AddCSH(pos, COLOR_ATTRVALUE);
             return;
         }
@@ -129,16 +132,16 @@ void GraphCsh::AddCSH_AttrName(const CshPos &pos, std::string_view name, EColorS
 }
 
 /** Just color the text to COLOR_ATTRVALUE*/
-void GraphCsh::AddCSH_AttrValue(const CshPos & pos, const char * aname, const char * avalue, EColorSyntaxType t) 
-{ 
-    if (strncmp("label_format", aname, 100)==0) 
+void GraphCsh::AddCSH_AttrValue(const CshPos & pos, std::string_view aname, std::string_view avalue, EColorSyntaxType)
+{
+    if ("label_format"==aname || "link_format"==aname)
         AddCSH_AttrValue_CheckAndAddEscapeHint(pos, avalue, {});
-     else 
+     else
         AddCSH(pos, COLOR_ATTRVALUE);
 }
 
 
-void GraphCsh::AddCSH_LineBeginOutsideGraph(const CshPos & pos, const char * text)
+void GraphCsh::AddCSH_LineBeginOutsideGraph(const CshPos & pos, std::string_view text)
 {
     std::set<std::string> options = {"pedantic"};
     std::set<std::string> keywords = {"strict", "usedesign", "defcolor", "defstyle", "defdesign", "include", "defproc", "set"};
@@ -198,7 +201,7 @@ void GraphCsh::AddLineBeginToHintsInsideGraph()
     };
     AddEntitiesToHints();
     AttributeNames(GraphStyle::GRAPH);
-    AddToHints(keyword_names, HintPrefix(COLOR_KEYWORD), EHintType::KEYWORD, 
+    AddToHints(keyword_names, HintPrefix(COLOR_KEYWORD), EHintType::KEYWORD,
         CshHintGraphicCallbackForKeywords);
     AllowAnything();
 }
@@ -220,7 +223,7 @@ void GraphCsh::AddKeywordsOutsideGraph(bool add_strict, bool add_graph_digraph)
         "include", "Include the text of another file in parsing.",
         ""
     };
-    if (add_strict) 
+    if (add_strict)
         AddToHints(keyword_names2, HintPrefix(COLOR_KEYWORD), EHintType::KEYWORD,
             CshHintGraphicCallbackForKeywords);
     if (add_graph_digraph)
@@ -231,7 +234,7 @@ void GraphCsh::AddKeywordsOutsideGraph(bool add_strict, bool add_graph_digraph)
 
 void GraphCsh::AddLineBeginToHintsOutsideGraph(bool add_strict, bool add_graph_digraph)
 {
-    if (add_strict) 
+    if (add_strict)
         AddOptionsToHints();
     AddKeywordsOutsideGraph(add_strict, add_graph_digraph);
 }
@@ -249,10 +252,10 @@ void GraphCsh::AddOptionsToHints()
 bool GraphCsh::AddOptionsValuesToHints()
 {
     if (CaseInsensitiveEqual(hintAttrName, "pedantic")) {
-        AddToHints(CshHint(HintPrefix(COLOR_ATTRVALUE) + "no", 
-            "Don't generate warnings for undirected edges in directed graphs and vice versa.", 
+        AddToHints(CshHint(HintPrefix(COLOR_ATTRVALUE) + "no",
+            "Don't generate warnings for undirected edges in directed graphs and vice versa.",
             EHintType::ATTR_VALUE, true, CshHintGraphicCallbackForYesNo, 0));
-        AddToHints(CshHint(HintPrefix(COLOR_ATTRVALUE) + "yes", 
+        AddToHints(CshHint(HintPrefix(COLOR_ATTRVALUE) + "yes",
             "Generate warnings for directed edges in undirected graphs and vice versa.",
             EHintType::ATTR_VALUE, true, CshHintGraphicCallbackForYesNo, 1));
         return true;
