@@ -66,7 +66,12 @@ std::set<std::string> GetLayoutMethods()
 {
     std::set<std::string> ret;
     int size;
-    for (char **list = gvPluginList(Gvc, const_cast<char*>("layout"), &size, nullptr); size; size--, list++)
+#if GRAPHVIZ_VER >= 900
+    char** list = gvPluginList(Gvc, const_cast<char*>("layout"), &size);
+#else
+    char** list = gvPluginList(Gvc, const_cast<char*>("layout"), &size, nullptr);
+#endif
+    for (; size; size--, list++)
         ret.emplace(*list);
     return ret;
 }
@@ -1050,7 +1055,7 @@ void canvas_cover_polygon(bool draw, GVJ_t * job, pointf * A, int n, int filled)
 
 /** Callback called by graphviz to draw a curve - we extract it and save as a drawing primitive
  * into Graph::_current or actually draw it on Graph::_current->_canvas, if 'draw' is true.*/
-void canvas_cover_beziercurve(bool draw, GVJ_t * job, pointf * A, int n, int /*arrow_at_start*/, int /*arrow_at_end*/, int filled)
+void canvas_cover_beziercurve(bool draw, GVJ_t * job, pointf * A, int n, int filled)
 {
     if (Graph::_current==nullptr || Graph::_current->_canvas==nullptr) return;
     Path p;
@@ -1160,9 +1165,14 @@ void cover_polygon(GVJ_t * job, pointf * A, int n, int filled)
     canvas_cover_polygon(false, job, A, n, filled);
 }
 
-void cover_beziercurve(GVJ_t * job, pointf * A, int n, int arrow_at_start, int arrow_at_end, int filled)
+#if GRAPHVIZ_VER >= 800
+void cover_beziercurve(GVJ_t* job, pointf* A, int n, int filled)
+#else
+void cover_beziercurve(GVJ_t * job, pointf * A, int n, int /*arrow_at_start*/, int /*arrow_at_end*/, int filled)
+#endif
+
 {
-    canvas_cover_beziercurve(false, job, A, n, arrow_at_start, arrow_at_end, filled);
+    canvas_cover_beziercurve(false, job, A, n, filled);
 }
 
 void cover_polyline(GVJ_t * job, pointf * A, int n)
@@ -1203,7 +1213,7 @@ gvrender_engine_t cover_render_engine = {
     nullptr,               //(*resolve_color) (GVJ_t * job, gvcolor_t * color);
     cover_ellipse,      //(*ellipse) (GVJ_t * job, pointf * A, int filled);
     cover_polygon,      //(*polygon) (GVJ_t * job, pointf * A, int n, int filled);
-    cover_beziercurve,  //(*beziercurve) (GVJ_t * job, pointf * A, int n, int arrow_at_start, int arrow_at_end, int);
+    cover_beziercurve,  //(*beziercurve) (GVJ_t * job, pointf * A, int n, int arrow_at_start, int arrow_at_end, int); after graphviz 7.0: (GVJ_t * job, pointf * A, int n, int arrow_at_start, int arrow_at_end, int);
     cover_polyline,     //(*polyline) (GVJ_t * job, pointf * A, int n);
     nullptr,               //(*comment) (GVJ_t * job, char *comment);
     nullptr,               //(*library_shape) (GVJ_t * job, char *name, pointf * A, int n, int filled);
@@ -1273,9 +1283,13 @@ void canvas_polygon(GVJ_t * job, pointf * A, int n, int filled)
     canvas_cover_polygon(true, job, A, n, filled);
 }
 
-void canvas_beziercurve(GVJ_t * job, pointf * A, int n, int arrow_at_start, int arrow_at_end, int filled)
+#if GRAPHVIZ_VER >= 800
+void canvas_beziercurve(GVJ_t* job, pointf* A, int n, int filled)
+#else
+void canvas_beziercurve(GVJ_t * job, pointf * A, int n, int /*arrow_at_start*/, int /*arrow_at_end*/, int filled)
+#endif
 {
-    canvas_cover_beziercurve(true, job, A, n, arrow_at_start, arrow_at_end, filled);
+    canvas_cover_beziercurve(true, job, A, n, filled);
 }
 
 void canvas_polyline(GVJ_t * job, pointf * A, int n)
