@@ -28,6 +28,10 @@
 #include "gvstyle.h"
 #include "chartbase.h"
 
+#ifndef GRAPHVIZ_VER
+#error Need graphviz!
+#endif
+
 /** @name Types & externs copied from cgraph.h: forward declare cgraph types, so we do not have to include the whole cgraph library here
  ** @{ */
 typedef struct Agraph_s Agraph_t;	///<graph, subgraph (or hyperedge) (copied from cgraph.h of graphviz)
@@ -36,7 +40,7 @@ typedef struct Agedge_s Agedge_t;	///<node pair (copied from cgraph.h of graphvi
 typedef struct Agsym_s Agsym_t; ///<symbol (copied from cgraph.h of graphviz)
 typedef struct GVC_s GVC_t; ///<global gvc context (copied from cgraph.h of graphviz)
 typedef struct obj_state_s obj_state_t; ///<generic object (node, edge, graph, etc.) (copied from cgraph.h of graphviz)
-#ifndef GRAPHVIZ_POST_2_49
+#if GRAPHVIZ_VER < 249
 extern "C" void *agbindrec(void *obj, char *name, unsigned int size, int move_to_front);
 extern "C" int agxset(void *obj, Agsym_t * sym, char *value);
 extern "C" int agset(void *obj, char *name, char *value);
@@ -53,7 +57,7 @@ extern "C" Agedge_t *agedge(Agraph_t * g, Agnode_t * t, Agnode_t * h, char *name
 
 /** @name wrappers for passing const char* parameters to cgraph functions
  ** @{ */
-#ifndef GRAPHVIZ_POST_2_49
+#if GRAPHVIZ_VER < 249
 inline void *agbindrec(void *obj, const char *name, unsigned int size, int move_to_front) { return agbindrec(obj, const_cast<char*>(name), size, move_to_front); }
 inline int agxset(void *obj, Agsym_t * sym, const char *value) {return agxset(obj, sym, const_cast<char*>(value)); }
 #endif
