@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -35,7 +35,7 @@ struct GraphEdgeType
     enum {
         NO_ARROW, FWD, BACK, BIDIR
     } dir; ///<The directionality of the edge
-    const char *AsText() const; 
+    std::string_view AsText() const; 
 };
 
 
@@ -77,9 +77,9 @@ struct ParsedStyle
 {
     std::string line;
     std::string fill;
-    std::set<std::string> others;
+    std::set<std::string, std::less<>> others;
     ParsedStyle() = default;
-    void Parse(const char *s);
+    void Parse(std::string_view);
     operator string() const;
 };
 
@@ -112,14 +112,15 @@ public:
     GraphStyle &operator=(const GraphStyle&) = default;
     GraphStyle &operator=(GraphStyle&&) = default;
     std::map<std::string, GraphAttribute> attributes; ///<List of attributes 
-    virtual void Empty() override { attributes.clear(); }
-    virtual void MakeCompleteButText() override {}
-    virtual Style &operator +=(const Style &toadd) override;
-    virtual bool AddAttribute(const Attribute &a, Chart *) override;
+    void Empty() override { attributes.clear(); }
+    bool IsEmpty() const noexcept override { return attributes.empty() && Style::IsEmpty(); }
+    void MakeCompleteButText() override {}
+    Style &operator +=(const Style &toadd) override;
+    bool AddAttribute(const Attribute &a, Chart *) override;
     const GraphAttribute *FindInErrorText(const std::string &text) const;
-    virtual bool DoIAcceptUnqualifiedColorAttr() const override { return false; }
-    virtual void AttributeNames(Csh &csh) const override { DotAttributeNames(element_type, {}, csh); }
-    virtual bool AttributeValues(std::string_view attr, Csh & csh) const override { return DotAttributeValues(element_type, attr, {}, csh); }
+    bool DoIAcceptUnqualifiedColorAttr() const override { return false; }
+    void AttributeNames(Csh &csh) const override { DotAttributeNames(element_type, {}, csh); }
+    bool AttributeValues(std::string_view attr, Csh & csh) const override { return DotAttributeValues(element_type, attr, {}, csh); }
     static void DotAttributeNames(EGraphElementType type, std::string_view layout, Csh &csh);
     static bool DotAttributeValues(EGraphElementType type, std::string_view attr, std::string_view layout, Csh &csh);
     static bool CheckBoolValue(const char *value, bool &result);
