@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -35,7 +35,7 @@
 namespace msc {
 
 /** Enumerates the type of arrow, box and divider symbols (->, => or <->, etc). */
-enum class EArcSymbol
+enum class EArcSymbol : uint8_t
 {
     INVALID = 0,             ///<Invalid value
     ARC_SOLID,               ///<A solid, unidir arrow: -> or <- (DirArrow, SelfArrow, BlockArrow, ArcVertical)
@@ -90,7 +90,7 @@ enum class EArrowLost {
     AT_DST   ///<The arrow is lost close to the dst entity
 };
 
-/** Collects data about an arrow segments (dashed/solid/dotted/double and 
+/** Collects data about an arrow segments (dashed/solid/dotted/double and
  * whether it indicates a lost message using an asterisk */
 struct ArrowSegmentData {
     EArcSymbol type;           ///<The type of the arrow such as ARC_SOLID or BOX_DASHED
@@ -128,7 +128,7 @@ struct ArrowEnding
     FileLineCol linenum;          ///<The position of the 'start before' or 'end after' clause, if generated from source. In calls to AddStartPos() and AddEndPos() this is just the value specified by the user and not the absolute offset.
     double      offset = 0;       ///<The offset from the middle of the entity line. Up to the end of AddAttributeList() larger value is more towards the end of the arrow; after that positive is towards right, negative is towards left.
     ArrowEnding() noexcept = default;
-    ArrowEnding(MscChart &chart, const char *e, const FileLineColRange &l, double o = 0);
+    ArrowEnding(MscChart &chart, std::string_view e, const FileLineColRange &l, double o = 0);
     explicit ArrowEnding(EntityRef e, double o = 0, const FileLineCol &l = FileLineCol()) : entity(e), linenum(l), offset(o) {}
     ArrowEnding(const ArrowEnding&) = default;
     /** Compares position of two valid endings. */
@@ -189,7 +189,7 @@ struct EndConnection : public ArrowEnding
     JoinableResult CanJoinTo(const EndConnection &next) const;
 };
 
-class DistanceRequirements; //defined in msc.h 
+class DistanceRequirements; //defined in msc.h
 struct BoxSignature; //defined in boxes.h
 
                      /** Enumerates all bulk sections. These are ones that are done only once.*/
@@ -209,7 +209,7 @@ enum class EArcSection
     PLACEWITHMARKERS,///MscChart::PlaceWithMarkers
     NOTES,          ///<Note placement
     POST_POS,       ///<MscChart::PostPosProcess
-    DRAW,           ///<Drawing 
+    DRAW,           ///<Drawing
     MAX_ARC_SECTION///<Not used, the max value
 };
 /** Enumerates all the element categories: each can be processed in each section.*/
@@ -247,17 +247,17 @@ class ArcBase : public MscElement
 private:
     bool had_add_attr_list;///<True if we have called AddAttributeList for this arc. TODO: debug only, remove
 protected:
-    bool   valid;          ///<If false, then construction failed, arc does not exist 
+    bool   valid;          ///<If false, then construction failed, arc does not exist
     double vspacing;       ///<Contains the spacing *above* this arc. DBL_MIN if compress mechanism is on for this arc (compress attribute set)
     bool   parallel;       ///<If true, subsequent arcs can be drawn beside this.
     bool   overlap;        ///<If true, subsequent arcs can be drawn overlaid on this.
     bool   keep_together;  ///<If true, do not split this by automatic pagination.
-    bool   keep_with_next; ///<If true, do not separate this from following element by automatic pagination 
+    bool   keep_with_next; ///<If true, do not separate this from following element by automatic pagination
     bool   stick_to_prev;  ///<If true, we do not allow elements to be laid out between this and the previous element (relevant in parallel layout)
     string refname;        ///<Value of the "refname" attribute, to reference numbers & others. Empty if none.
     mutable double height; ///<Pixel height of the arc, calculated by Layout()
     static void AttributeNamesSet1(Csh &csh);
-    static bool AttributeValuesSet1(const std::string &attr, Csh &csh);
+    static bool AttributeValuesSet1(std::string_view attr, Csh &csh);
     /** Add the current activation status to the last element in 'vdist'. Used by Width().*/
     void AddEntityLineWidths(DistanceRequirements &vdist);
 public:
@@ -304,23 +304,23 @@ public:
      * are NOT allowed to merge them. */
     virtual bool SeparatesIndicators() const { return true; }
     /** If this element is followed by a Note, it can store the note for later re-insertion.
-     * This feature is useful for entity commands and arrow joins: when a series of 
+     * This feature is useful for entity commands and arrow joins: when a series of
      * instrcutions are merged and we may want to note only one of them.
      * @returns true if we have stored the Note internally (and took ownership)*/
     virtual bool StoreNoteTemporarily(Note*) { return false; }
     /** After PostParseProcess, we can re-insert the notes stored above.*/
-    virtual void ReinsertTmpStoredNotes(ArcList& list, ArcList::iterator after) { };
+    virtual void ReinsertTmpStoredNotes(ArcList&, ArcList::iterator after [[maybe_unused]]) { };
     /** @} */
 
     /** @name Geometry & other getters
     * These functions return calcualted parameters of the arc*/
     /** @{ */
     /** Get the Y coordinate of the top of the arc.
-     * The actual visual part may not start here, 
+     * The actual visual part may not start here,
      * if we have gap proscribed above in 'chart'
      * However, VSpacing is above this position.*/
     double GetFormalPos() const {return yPos;}
-    /** Return the height of the element with its side comments - 
+    /** Return the height of the element with its side comments -
      * the this may be more than the visual height as the arc may have
      * gap above and/or below proscribed by the 'chart'.*/
     double GetFormalHeight() const {return std::max(height, comment_height);}
@@ -328,7 +328,7 @@ public:
      * This may be wider than the actual visual effect and is used for
      * layout and to gauge associated verticals.*/
     virtual Range GetFormalYExtent() const {return Range(yPos, yPos+GetFormalHeight());}
-    /** Get the Y coordinate range actually occupied by visual elements of the arc. 
+    /** Get the Y coordinate range actually occupied by visual elements of the arc.
      * This is used to do automatic pagination. */
     virtual Range GetVisualYExtent(bool include_comments) const;
     /** Get an (ordered) list of entities that this arrow or box touches.
@@ -350,7 +350,7 @@ public:
     /** Add attribute names and helpers to csh for hinting */
     static void AttributeNames(Csh &csh);
     /** Add attribute values and helpers to csh for hinting */
-    static bool AttributeValues(const std::string &attr, Csh &csh);
+    static bool AttributeValues(std::string_view attr, Csh &csh);
     /** @} */
 
     /** @name Joining oriented functions
@@ -359,8 +359,8 @@ public:
     /** Returns if this element can be part of a join series at all.
      * If not its join related functions will not be called. */
     virtual bool CanJoin() const { return false; }
-    /** Determines what to do if this element is part of a join series and 
-     * disappears due to entity collapse. If false, we keep any indicators 
+    /** Determines what to do if this element is part of a join series and
+     * disappears due to entity collapse. If false, we keep any indicators
      * this->PostParseProcess() returns, else we drop them.
      * If not its join related functions will not be called. */
     virtual bool RemoveIndicator() const { return true; }
@@ -369,50 +369,50 @@ public:
      * AllEntities and the actual offset may be imprecise. If it is called after Width()
      * it shall return an EntityRef from ActiveEntities and offset shall be final.
      * Only for elements that can take part in join series'*/
-    virtual EndConnection GetConnection(bool is_src) const { _ASSERT(0); return EndConnection(); } //This entity cannot participate in join series - should not have been called
-    /** Get the actual position of the side of an element for the purpose comparing 
-     * ordering for joining and of setting unspecified ends of an arrow that joins to it. 
+    virtual EndConnection GetConnection(bool is_src [[maybe_unused]]) const { _ASSERT(0); return EndConnection(); } //This entity cannot participate in join series - should not have been called
+    /** Get the actual position of the side of an element for the purpose comparing
+     * ordering for joining and of setting unspecified ends of an arrow that joins to it.
      * If this is called before PostParseProcess(), it shall return a non-group EntityRef from
      * AllEntities and the actual offset may be imprecise. If it is called after Width()
      * it shall return an EntityRef from ActiveEntities and offset shall be final.
      * Not called for unspecified ends.
      * Only for elements that can take part in join series'*/
-    virtual ArrowEnding GetSidePos(bool is_src) const { _ASSERT(0); return ArrowEnding(); } //This entity cannot participate in join series - should not have been called
+    virtual ArrowEnding GetSidePos(bool is_src [[maybe_unused]]) const { _ASSERT(0); return ArrowEnding(); } //This entity cannot participate in join series - should not have been called
     /** Tests if we can set the ending so and/or set an end to a position for the purpose of joining.
-     * It is called twice for and only for unspecified endings. Once before PostParseProcess(), with an 
+     * It is called twice for and only for unspecified endings. Once before PostParseProcess(), with an
      * test_only==true. Then it may be called again before Width() with test_only==false.
      * Changes shall be made only at the second call.
      * Only for elements that can take part in join series'*/
-    virtual bool SetEndingDueToJoin(bool is_src, const ArrowEnding &ending, bool test_only) { _ASSERT(0); return false; } //This entity cannot participate in join series - should not have been called
-    /** Makes an arrow curly join to another one. This is called after PostParseProcess(), 
+    virtual bool SetEndingDueToJoin(bool is_src [[maybe_unused]], const ArrowEnding&, bool test_only [[maybe_unused]]) { _ASSERT(0); return false; } //This entity cannot participate in join series - should not have been called
+    /** Makes an arrow curly join to another one. This is called after PostParseProcess(),
      * when we know that the previous arrow do not disappear and a curly join is still needed.
      * Called only for elements that return can_curly_join==true in GetConnection()*/
-    virtual void SetCurlyJoin(bool is_src, ArcBase *prev, bool src_of_prev) { _ASSERT(0); } //This entity cannot participate in join series - should not have been called
+    virtual void SetCurlyJoin(bool is_src [[maybe_unused]], ArcBase*, bool src_of_prev [[maybe_unused]]) { _ASSERT(0); } //This entity cannot participate in join series - should not have been called
     /** Get the y position of the end, for the purposes of joining to it.
     * Shall be called only after Layout()*/
-    virtual double GetCenterline(bool at_src) const { _ASSERT(0); return 0; } //This entity cannot participate in join series - should not have been called
-    /** Get the activation status of the entity at the dst end of the arrow, 
+    virtual double GetCenterline(bool at_src [[maybe_unused]]) const { _ASSERT(0); return 0; } //This entity cannot participate in join series - should not have been called
+    /** Get the activation status of the entity at the dst end of the arrow,
      * for the purposes of joining to it. Shall be called only after Layout(). */
-    virtual double GetActSize(bool at_src) const { _ASSERT(0); return 0; } //This entity cannot participate in join series - should not have been called
+    virtual double GetActSize(bool at_src [[maybe_unused]]) const { _ASSERT(0); return 0; } //This entity cannot participate in join series - should not have been called
     /** @} */
 
     /** @name Recursive processors
      * These functions are called recursively for all arcs in this order*/
     /** @{ */
     /** Calculate things  after parsing.
-     * Called after parsing and adding attributes. 
-     * Entity order and collapse/expand is already known here 
+     * Called after parsing and adding attributes.
+     * Entity order and collapse/expand is already known here
      * See documentation for libmscgen for more info.*/
     virtual ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
                                       Numbering &number, MscElement **note_target, ArcBase *vertical_target);
-    /** Substitute name references in labels & process all escapes 
+    /** Substitute name references in labels & process all escapes
      * See documentation for libmscgen for more info.*/
     virtual void FinalizeLabels(Canvas &canvas);
     /** Collects vertical distances for hscale=auto mechanism.
      * See documentation for libmscgen for more info.*/
     virtual void Width(Canvas &/*canvas*/, DistanceRequirements &/*vdist*/) {}
-    /** Calculate entity layout at y=0 vertical position. 
-     * Calculates the height, and sets up the area at yPos==0, 
+    /** Calculate entity layout at y=0 vertical position.
+     * Calculates the height, and sets up the area at yPos==0,
      * add its cover to use at placement later to `cover`.
      * Cover or area does not include any spacing left around such as chart->emphVGapAbove.
      * See documentation for libmscgen for more info.*/
@@ -424,21 +424,21 @@ public:
     virtual void FinalizeLayout(Canvas &/*canvas*/, AreaList * /*cover*/) {}
     /** Move the arc up or down. This can be called multiple times. */
     void ShiftBy(double y) override {if (valid) {MscElement::ShiftBy(y);}}
-    /** Collect the y position of page breaks into MscChart::pageBreakData. 
+    /** Collect the y position of page breaks into MscChart::pageBreakData.
      * See documentation for libmscgen for more info.*/
     virtual void CollectPageBreak() {}
-    /** Split the element into two by a page break. 
-     * Maintain running_state of entities. 
+    /** Split the element into two by a page break.
+     * Maintain running_state of entities.
      * @returns The amount the element has grown. -1 if we cannot rearrange, -2 if we are to ignore.
      * See documentation for libmscgen for more info.*/
     virtual double SplitByPageBreak(Canvas &/*canvas*/, double /*netPrevPageSize*/,
-                                    double /*pageBreak*/, bool &/*addCommandNewpage*/, 
+                                    double /*pageBreak*/, bool &/*addCommandNewpage*/,
                                     bool /*addHeading*/, ArcList &/*res*/) {return -1;}
-    /** Place verticals. 
-     * All height & pos info final by now, except on verticals & notes 
+    /** Place verticals.
+     * All height & pos info final by now, except on verticals & notes
      * See documentation for libmscgen for more info.*/
     virtual void PlaceWithMarkers(Canvas &/*canvas*/) {}
-    /** Emit warnings that need final layout. 
+    /** Emit warnings that need final layout.
      * See documentation for libmscgen for more info.*/
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
     /** Draw the arc.
@@ -453,7 +453,7 @@ typedef UPtrList<ArcBase> ArcList;
 
 /** An arc representing a small indicator symbol.
  * This arc cannot be generated by the user..
- * it is generated by libmscgen to replace arcs that get hidden due to collapsing 
+ * it is generated by libmscgen to replace arcs that get hidden due to collapsing
  * a group entity.
  * An indicator is either on a specific entity or in a box, which may have two entities
  * setting its location. `src` and `dst` members capture the horizontal location.*/
@@ -472,7 +472,7 @@ public:
     bool IsComplete() const;
     /** Set which entities the iterator is between. */
     void SetEntities(EntityRef s, EntityRef d) {src=s; dst=d;}
-    /** Expand the `file_pos` member with that of another interator if at same location. 
+    /** Expand the `file_pos` member with that of another interator if at same location.
      * @returns false, if `src` or `dst` differ (and then no change to `file_pos`).*/
     bool Combine(const Indicator *o);
     /** Expand the `file_pos` member with that of another hidden element. */
@@ -485,9 +485,9 @@ public:
     bool RemoveIndicator() const override { return false; }
     EndConnection GetConnection(bool is_src) const override;
     ArrowEnding GetSidePos(bool is_src) const override;
-    bool SetEndingDueToJoin(bool is_src, const ArrowEnding &ending, bool test_only)  override { _ASSERT(0); return false; } //Never unspecified
-    void SetCurlyJoin(bool is_src, ArcBase *prev, bool src_of_prev) override { _ASSERT(0); } //never curly
-    double GetCenterline(bool at_src) const override { return GetIndicatorCover(XY(GetXCoord(), yPos)).y.MidPoint(); }
+    bool SetEndingDueToJoin(bool /*is_src*/, const ArrowEnding&, bool /*test_only*/)  override { _ASSERT(0); return false; } //Never unspecified
+    void SetCurlyJoin(bool /*is_src*/, ArcBase*, bool /*src_of_prev*/) override { _ASSERT(0); } //never curly
+    double GetCenterline(bool /*at_src*/) const override { return GetIndicatorCover(XY(GetXCoord(), yPos)).y.MidPoint(); }
     double GetActSize(bool at_src) const override;
 
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
@@ -518,7 +518,7 @@ public:
     LabelledArc(EArcCategory c, MscChart *msc);
     LabelledArc(EArcCategory c, const LabelledArc &al);
     /** Return the style of the arc. */
-    const MscStyleCoW &GetStyle() const {return style;} 
+    const MscStyleCoW &GetStyle() const {return style;}
     bool CanBeNoted() const override { return true; }
     bool CanBeAlignedTo() const override { return true; }
     /** Set style to this name, but combine it with default text style */
@@ -532,12 +532,12 @@ public:
      *                 corresponding message shall be displayed.
      * @param [in] e1 The left entity governing the space available for the label.
      * @param [in] e2 The right entity governing the space available for the label. */
-    void OverflowWarning(double overflow, const string &msg, 
+    void OverflowWarning(double overflow, const string &msg,
                          CEntityRef e1, CEntityRef e2);
     /** Counts non-word-wrapped labels in the chart and also how many has been overflown.
-     * It is used to display a warning at the end if may labels were 
+     * It is used to display a warning at the end if may labels were
      * overflown. Shall be called from Layout();
-     * @param [in] space Says how much vertial space the label has.
+     * @param [in] space Says how much vertical space the label has.
      *                   If the label is wider, we mark it as overflown,
      *                   but only if no word_wrapping is used for the labels. */
     void CountOverflow(double space);
@@ -550,7 +550,7 @@ public:
     void AddAttributeList(gsl::owner<AttributeList*> l) override {SetStyleBeforeAttributes(l); AddAttributeListStep2(AddAttributeListStep1(l)); }
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh);
-    static bool AttributeValues(const std::string &attr, Csh &csh);
+    static bool AttributeValues(std::string_view attr, Csh &csh);
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
                               Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
     /** Return the formatted number of the arc (empty string if none).*/
@@ -600,14 +600,14 @@ struct VertXPos
     EntityRef entity2 = nullptr; ///<The second entity, used only for POS_CENTER
     FileLineColRange e1line;     ///<Location of the mention of the first entity in the input line (inside this vertpos fragment)
     FileLineColRange e2line;     ///<Location of the mention of the second entity in the input line (inside this vertpos fragment)
-    double offset = 0;           ///<An arbitrary user specified horizontal offset on top of the poition defined by `pos`
+    double offset = 0;           ///<An arbitrary user specified horizontal offset on top of the position defined by `pos`
 
     /** Create (yet) invalid object */
     VertXPos() noexcept = default;
     /** Create a position with 2 entities (always POS_CENTER*) */
-    VertXPos(MscChart& m, const char* e1, const FileLineColRange& e1l, const char* e2, const FileLineColRange& e2l, double off = 0);
+    VertXPos(MscChart& m, std::string_view e1, const FileLineColRange& e1l, std::string_view e2, const FileLineColRange& e2l, double off = 0);
     /** Create a position with one entity (must not be POS_CENTER) */
-    VertXPos(MscChart& m, const char* e1, const FileLineColRange& e1l, EPosType p = POS_AT, double off = 0);
+    VertXPos(MscChart& m, std::string_view e1, const FileLineColRange& e1l, EPosType p = POS_AT, double off = 0);
     /** Calculate the x coordinate of the middle of the element we specify the location for (only after entities are placed)
     * @param m The chart
     * @param width The width of the object
@@ -623,7 +623,7 @@ struct VertXPos
 /** The base class for all arrows: Self, Dir, Block and Vertical*/
 class Arrow : public LabelledArc
 {
-protected: 
+protected:
     bool headless_mscgen_arrow;   ///<True if this is an mscgen syntax arc defined without arrowhead via one of (--, ==, .., ::)
     StoredNotes tmp_stored_notes; ///<A set of notes attached to us.
 public:
@@ -634,23 +634,23 @@ public:
     Arrow(bool b, EArcCategory c, const LabelledArc &al) : LabelledArc(c, al), headless_mscgen_arrow(false), bidir(b) {}
     Arrow(const Arrow&) = delete;
     Arrow(Arrow&&) = default;
-    /** Get the refinement style for a given arrow symbol 
+    /** Get the refinement style for a given arrow symbol
      * The Arrow:: version returns the style for self and dir arrows, BlockArrow overrides. */
     virtual const MscStyleCoW *GetRefinementStyle4ArrowSymbol(EArcSymbol t) const;
     /** Add a new arrow segment. Called during processing.
-     * @param [in] data The type of the new arrow segment, indication of amy 
+     * @param [in] data The type of the new arrow segment, indication of amy
      *                  potential loss indication and its location.
      * @param [in] m The name of the entity (can be nullptr when omitted by user)
      * @param [in] ml The location of the name in the input file.
      * @param [in] l The location of the whole added segment (for error messages)*/
-    virtual Arrow *AddSegment(ArrowSegmentData data, const char *m, const FileLineColRange &ml,
-                                 const FileLineColRange &l) = 0;
+    virtual Arrow *AddSegment(ArrowSegmentData data, std::string_view m, const FileLineColRange &ml,
+                              const FileLineColRange& l) = 0;
     /** Add a loss position. */
     virtual Arrow* AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange& l) = 0;
-    /** Add the starting position. If ending is null, we assume src is 
+    /** Add the starting position. If ending is null, we assume src is
      * LSIDE or RSIDE and make it a special ending (used when user specifies pipe symbol)*/
     virtual Arrow *AddStartPos(ArrowEnding *ending=nullptr) = 0;
-    /** Add the ending position. If ending is null, we assume src is 
+    /** Add the ending position. If ending is null, we assume src is
      * LSIDE or RSIDE and make it a special ending (used when user specifies pipe symbol)*/
     virtual Arrow *AddEndPos(ArrowEnding *ending=nullptr) = 0;
     /** Indicate that this arrow was defined via an mscgen headless arrow symbol (--, ==, ., ::)*/
@@ -659,7 +659,7 @@ public:
     void ReinsertTmpStoredNotes(ArcList& list, ArcList::iterator after) override;
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh);
-    static bool AttributeValues(const std::string &attr, Csh &csh);
+    static bool AttributeValues(std::string_view attr, Csh &csh);
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
 };
 
@@ -684,7 +684,7 @@ class SelfArrow : public Arrow
 {
 protected:
     EntityRef src;           ///<The entity of the arrow
-    double    YSize;         ///<The vertical difference between the start and end of the arrow 
+    double    YSize;         ///<The vertical difference between the start and end of the arrow
     const double XSizeUnit;  ///<This is the width of the arrow arc in entity units (=130 pixel).
     bool user_specified_side;///<True if the side attribute was specified by the user.
 
@@ -699,10 +699,10 @@ protected:
 public:
     const EArcSymbol type; ///<What kind of arrow symbol was used to define this arrow.
     /** Standard constructor. 'ys' shall be the height of the drawn arrow. */
-    SelfArrow(EArcSymbol t, const char *s, const FileLineColRange &sl,
+    SelfArrow(EArcSymbol t, std::string_view s, const FileLineColRange &sl,
                  MscChart *msc, double ys);
-    Arrow *AddSegment(ArrowSegmentData data, const char *m, const FileLineColRange &ml, 
-                         const FileLineColRange &l) override;
+    Arrow *AddSegment(ArrowSegmentData data, std::string_view m, const FileLineColRange &ml,
+                      const FileLineColRange &l) override;
     Arrow *AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange &l) override;
     Arrow *AddStartPos(ArrowEnding *ending = nullptr) override;
     Arrow *AddEndPos(ArrowEnding *ending = nullptr) override;
@@ -714,11 +714,11 @@ public:
 
     bool CanJoin() const override { return true; }
     EndConnection GetConnection(bool is_src) const override;
-    ArrowEnding GetSidePos(bool is_src) const override { return ArrowEnding(src, 0); }  //Before PostParseProcess() this is an iterator to AllEntities, after to ActiveEntities - OK
-    bool SetEndingDueToJoin(bool is_src, const ArrowEnding &ending, bool test_only) override { _ASSERT(0); return false; } //Never unspecified
-    void SetCurlyJoin(bool is_src, ArcBase *prev, bool src_of_prev) override { _ASSERT(0); } //never curly
+    ArrowEnding GetSidePos(bool /*is_src*/) const override { return ArrowEnding(src, 0); }  //Before PostParseProcess() this is an iterator to AllEntities, after to ActiveEntities - OK
+    bool SetEndingDueToJoin(bool /*is_src*/, const ArrowEnding&, bool /*test_only*/) override { _ASSERT(0); return false; } //Never unspecified
+    void SetCurlyJoin(bool /*is_src*/, ArcBase*, bool /*src_of_prev*/) override { _ASSERT(0); } //never curly
     double GetCenterline(bool at_src) const override { return yPos + (at_src ? src_height : src_height+wh.y); }
-    double GetActSize(bool at_src) const override { return src_act; }
+    double GetActSize(bool /*at_src*/) const override { return src_act; }
 
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
     void Layout(Canvas &canvas, AreaList *cover) override;
@@ -754,7 +754,7 @@ protected:
     ArcBase                 *curly_join_to = nullptr;///<Set to the arrow above us, we curly join to. nullptr if none.
     bool                     curly_join_to_src = false;    ///<True if we curly join to the 'src' end of 'curly_join_to'
     std::pair<bool, bool>    user_unspecified_ends = {false, false};///<True if the user originally left src and/or dst unspecified. We may set src/dst in SetEndingDueToJoin(), but these show if originally we had them unspecified - so that we can update them. Valid after AddAttributeList.
-    
+
     mutable bool is_forward = false;///<True if the src of the arrow is left of the dst (considering *_ending and *_to_box). Valid only after parsing (specifically after AddAttributeList()). (Before that we have no entity ordering yet.)
     mutable double sin_slant = 0; ///<Pre-calculated value of `slant_angle`
     mutable double cos_slant = 0; ///<Pre-calculated value of `slant_angle`
@@ -780,14 +780,14 @@ protected:
     void InsertSkipEntity(unsigned u, Entity *pEntity);
     double ConsiderLabelWidth(double width_needed);
 public:
-    DirArrow(ArrowSegmentData data, 
-             const char *s, const FileLineColRange &sl, 
-             const char *d, const FileLineColRange &dl, 
+    DirArrow(ArrowSegmentData data,
+             std::string_view s, const FileLineColRange &sl,
+             std::string_view d, const FileLineColRange &dl,
              MscChart *msc, bool fw);
     /** Constructor used while converting a collapsed Box to an BlockArrow*/
     DirArrow(const NEntityList &el, bool bidir, EArcSymbol boxsymbol, const LabelledArc &al);
-    Arrow *AddSegment(ArrowSegmentData data, const char *m, const FileLineColRange &ml,
-                         const FileLineColRange &l) override;
+    Arrow *AddSegment(ArrowSegmentData data, std::string_view m, const FileLineColRange &ml,
+                      const FileLineColRange &l) override;
     Arrow *AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange &l) override;
     Arrow *AddStartPos(ArrowEnding *ending = nullptr) override;
     Arrow *AddEndPos(ArrowEnding *ending = nullptr) override;
@@ -810,13 +810,13 @@ public:
                               Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
     /** Update the stored activation status of entities.
      * Called if a centierlined entity command after us has changed active entities.*/
-    virtual void UpdateActiveSizes(); 
+    virtual void UpdateActiveSizes();
     void FinalizeLabels(Canvas &canvas) override;
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
     void Layout(Canvas &canvas, AreaList *cover) override;
     void FinalizeLayout(Canvas &canvas, AreaList *cover)  override;
     /** Get the Centerline Delta from the top of the arc at horizontal position `x`.
-     * A negative return value indicates the object has no centerline. 
+     * A negative return value indicates the object has no centerline.
      * Shall be called only after Layout()*/
     virtual double GetCenterlineAtXPos(double x) const;
     /** Calculates `area.mainline`. Tricky for slanted arrows.
@@ -824,14 +824,14 @@ public:
     void CalculateMainline(double thickness);
     /** Is this the start or end of the arrow?
      * From the index of `xPos` and `specified_as_forward` give EArrowType::{START,MIDDLE,END}*/
-    EArrowEnd WhichArrow(unsigned i) const; 
+    EArrowEnd WhichArrow(unsigned i) const;
     void ShiftBy(double y) override;
     /** Check if the entities in `src` - `middle[]` - `dst` are all left-to-right or right-to-left*/
     void CheckSegmentOrder(double y);
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
     void RegisterLabels() override;
     void CollectIsMapElements(Canvas &canvas) override;
-    void AddClippedLine(Canvas& canvas, Path path, const LineAttr& line, 
+    void AddClippedLine(Canvas& canvas, Path path, const LineAttr& line,
                         const MscArrowHeads& loc_arrow, unsigned i, const Block* clip_by) const;
     void DrawArrow(Canvas &canvas, const Label &loc_parsed_label,
                    const std::vector<LineAttr>& loc_segment_lines,
@@ -844,7 +844,7 @@ class BlockArrow : public DirArrow
 {
 protected:
     /** Signature of the Box, if we got created as a collapsed Box. nullptr if not*/
-    const BoxSignature * const sig;   
+    const BoxSignature * const sig;
 
     mutable double sy;      ///<The y coordinate of the top of the arrow body (not including arrowheads). Set in Width()
     mutable double dy;      ///<The y coordinate of the bottom of the arrow body (not including arrowheads). Set in Width()
@@ -860,16 +860,16 @@ public:
     /** Used when converting a collapsed Box to an BlockArrow */
     BlockArrow(const NEntityList &, bool bidir, EArcSymbol boxsymbol, const LabelledArc &al, const BoxSignature *s);
     ~BlockArrow() override;
-    bool StoreNoteTemporarily(Note* cn) override { return false; }
+    bool StoreNoteTemporarily(Note*) override { return false; }
     void SetTemporarilyStoredNotes(StoredNotes&&); ///<Used by Box::PostParseProcess(), when collapsing to block arrow.
     void SetStyleBeforeAttributes(AttributeList *l) override;
     const MscStyleCoW *GetRefinementStyle4ArrowSymbol(EArcSymbol t) const override;
     const BoxSignature* GetSignature() const override { return sig; }
     static void AttributeNames(Csh &csh);
-    static bool AttributeValues(const std::string &attr, Csh &csh);
+    static bool AttributeValues(std::string_view attr, Csh &csh);
 
     EndConnection GetConnection(bool is_src) const override { auto ret = DirArrow::GetConnection(is_src); ret.can_curly_join = false; return ret; }
-    void SetCurlyJoin(bool is_src, ArcBase *prev, bool src_of_prev) override { _ASSERT(0); } //We cannot curly join
+    void SetCurlyJoin(bool /*is_src*/, ArcBase*, bool /*src_of_prev*/) override { _ASSERT(0); } //We cannot curly join
 
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
                               Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
@@ -924,9 +924,9 @@ protected:
     mutable std::vector<Contour> outer_contours; ///<Calculated contour (only one element)
 public:
     /** Constructor for arrow or box symbols with two marker names (one can be nullptr)*/
-    Vertical(const ArcTypePlusDir *t, const char *s, const char *d, MscChart *msc);
-    Arrow *AddSegment(ArrowSegmentData data, const char *m, const FileLineColRange &ml,
-                         const FileLineColRange &l) override;
+    Vertical(const ArcTypePlusDir *t, std::string_view s, std::string_view d, MscChart *msc);
+    Arrow *AddSegment(ArrowSegmentData data, std::string_view m, const FileLineColRange &ml,
+                      const FileLineColRange &l) override;
     Arrow *AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange &l) override;
     Arrow *AddStartPos(ArrowEnding *ending = nullptr) override;
     Arrow *AddEndPos(ArrowEnding *ending = nullptr) override;
@@ -936,12 +936,12 @@ public:
     void SetVerticalShape(EVerticalShape sh);
     bool CanBeNoted() const override { return false; }
     bool CanBeAlignedTo() const override { return false; }
-    bool StoreNoteTemporarily(Note* cn) override { return false; }
+    bool StoreNoteTemporarily(Note*) override { return false; }
     void SetStyleBeforeAttributes(AttributeList *l) override;
     const MscStyleCoW *GetMyRefinementStyle() const;
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh);
-    static bool AttributeValues(const std::string &attr, Csh &csh, EVerticalShape shape);
+    static bool AttributeValues(std::string_view attr, Csh &csh, EVerticalShape shape);
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
                               Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
@@ -950,7 +950,7 @@ public:
     void ShiftBy(double y) override;
     Range GetFormalYExtent() const override { return Range(false); }
     double SplitByPageBreak(Canvas &/*canvas*/, double /*netPrevPageSize*/,
-                            double /*pageBreak*/, bool &/*addCommandNewpage*/, 
+                            double /*pageBreak*/, bool &/*addCommandNewpage*/,
                             bool /*addHeading*/, ArcList &/*res*/) override {return -2;}
     void PlaceWithMarkers(Canvas &canvas) override;
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
