@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -90,8 +90,8 @@ protected:
     mutable Contour text_cover;         ///<Area covered by the label
     mutable double sx_tag;              ///<Left boundary of tag label (if any)
     mutable double dx_tag;              ///<Right boundary of tag label (if any)
-    mutable double y_tag;               ///<Top boundary of tag label (if any) 
-    mutable Block tag_outer_edge;       ///<The outer edge of the tag rectange (must be clipped by the inner edge of the box)
+    mutable double y_tag;               ///<Top boundary of tag label (if any)
+    mutable Block tag_outer_edge;       ///<The outer edge of the tag rectangle (must be clipped by the inner edge of the box)
     mutable Contour tag_cover;          ///<Area covered by the tag (not just the label of it) - you can use it to clip
 public:
     /** Enumerating mscgen-style boxes. */
@@ -99,19 +99,19 @@ public:
     {
         MSCGEN_COMPAT_NONE, MSCGEN_COMPAT_BOX, MSCGEN_COMPAT_ABOX, MSCGEN_COMPAT_RBOX, MSCGEN_COMPAT_NOTE
     } mscgen_compat; ///<Set to other than MSCGEN_COMPAT_NONE, if this box was defined as an mscgen box, abox, rbox or note.
-    Box(EArcSymbol t, const FileLineColRange &l, const char *s, const FileLineColRange &sl,
-        const char *d, const FileLineColRange &dl, MscChart *msc);
-    Box(Emscgen_compat c, const FileLineColRange &l, const char *s, const FileLineColRange &sl,
-        const char *d, const FileLineColRange &dl, MscChart *msc);
+    Box(EArcSymbol t, const FileLineColRange &l, std::string_view s, const FileLineColRange &sl,
+        std::string_view d, const FileLineColRange &dl, MscChart *msc);
+    Box(Emscgen_compat c, const FileLineColRange &l, std::string_view s, const FileLineColRange &sl,
+        std::string_view d, const FileLineColRange &dl, MscChart *msc);
     bool CanBeNoted() const override { return true; }
     const BoxSignature* GetSignature() const override { return &sig; }
     const MscStyleCoW *GetRefinementStyle4BoxSymbol(EArcSymbol t) const;
-    void SetStyleBeforeAttributes(AttributeList *l) override {} //we do it in AddAttributeList() below
+    void SetStyleBeforeAttributes(AttributeList*) override {} //we do it in AddAttributeList() below
     Box* AddArcList(gsl::owner<ArcList*>l);
     void AddAttributeList(gsl::owner<AttributeList*>) override;
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh);
-    static bool AttributeValues(const std::string &attr, Csh &csh);
+    static bool AttributeValues(std::string_view attr, Csh &csh);
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
         Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
     void FinalizeLabels(Canvas &canvas) override;
@@ -149,10 +149,10 @@ public:
     bool RemoveIndicator() const override { return false; }
     EndConnection GetConnection(bool is_src) const override;
     ArrowEnding GetSidePos(bool is_src) const override;
-    bool SetEndingDueToJoin(bool is_src, const ArrowEnding &ending, bool test_only)  override { _ASSERT(0); return false; } //Never unspecified
-    void SetCurlyJoin(bool is_src, ArcBase *prev, bool src_of_prev) override { _ASSERT(0); } //never curly
-    double GetCenterline(bool at_src) const override { return yPos+total_height/2; }
-    double GetActSize(bool at_src) const override { _ASSERT(0); return 0; } //This never does curly joins
+    bool SetEndingDueToJoin(bool /*is_src*/, const ArrowEnding&, bool /*test_only*/)  override { _ASSERT(0); return false; } //Never unspecified
+    void SetCurlyJoin(bool /*is_src*/, ArcBase*, bool /*src_of_prev*/) override { _ASSERT(0); } //never curly
+    double GetCenterline(bool /*at_src*/) const override { return yPos+total_height/2; }
+    double GetActSize(bool /*at_src*/) const override { _ASSERT(0); return 0; } //This never does curly joins
 
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
         Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
@@ -194,25 +194,24 @@ protected:
     mutable double dx_text;         ///<Right boundary of label
     mutable double y_text;          ///<Top boundary of label
     mutable Contour text_cover;     ///<Area covered by the label
-    mutable Block pipe_block;       ///<A representative rectangle of the pipe                    
+    mutable Block pipe_block;       ///<A representative rectangle of the pipe
     mutable Contour pipe_shadow;    ///<The outer contour of the pipe used to draw the shadow
     mutable Contour pipe_body_line; ///<The line around the body of the pipe (excluding the hole at the end)
     mutable Contour pipe_whole_line;///<The line around the whole of the pipe (including the hole at the end)
     mutable Contour pipe_body_fill; ///<The fill area of the body of the pipe (excluding the hole at the end)
     mutable Contour pipe_hole_fill; ///<The fill area of the hole at the end
-    mutable Contour pipe_hole_line; ///<The line around the hole at the ehnd   
+    mutable Contour pipe_hole_line; ///<The line around the hole at the end
     mutable Contour pipe_hole_curve;///<The curve at the hole end
 public:
     const EArcSymbol type; ///<What box symbol was used to define this pipe.
-    /** Upgrade a box to a pipe: this is how we create pipes normally.
-    * Deletes 'box'*/
-    Pipe(Box *box);
+    /** Upgrade a box to a pipe: this is how we create pipes normally. Deletes 'box'*/
+    Pipe(gsl::owner<Box*> box);
     bool CanBeNoted() const override { return true; }
     const MscStyleCoW *GetRefinementStyle4PipeSymbol(EArcSymbol t) const;
     void SetStyleBeforeAttributes(AttributeList *l) override;
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh);
-    static bool AttributeValues(const std::string &attr, Csh &csh);
+    static bool AttributeValues(std::string_view attr, Csh &csh);
     void Layout(Canvas &/*canvas*/, AreaList * /*cover*/) override { _ASSERT(0); }
     void ShiftBy(double y) override;
     void RegisterCover(EMscDrawPass /*pass*/) const override { _ASSERT(0); } //will never be called
@@ -282,7 +281,7 @@ public:
     void SetStyleBeforeAttributes(AttributeList *l) override;
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh, bool nudge, bool title);
-    static bool AttributeValues(const std::string &attr, Csh &csh, bool nudge, bool title);
+    static bool AttributeValues(std::string_view attr, Csh &csh, bool nudge, bool title);
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
                               Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
@@ -299,7 +298,7 @@ public:
 class ParallelBlocks : public ArcBase
 {
 public:
-    /** Describes the three layout algoritm for parallel blocks.*/
+    /** Describes the three layout algorithm for parallel blocks.*/
     enum EParallelLayoutType
     {
         INVALID_LAYOUT = 0,
@@ -307,7 +306,7 @@ public:
         ONE_BY_ONE,       ///<Lay out elements from each block one by one and avoid overlap
         ONE_BY_ONE_MERGE, ///<Lay out elements from each block one by one and avoid overlap, merge with parallel block around us
     };
-    /** Specify vertical alignmen. */
+    /** Specify vertical alignment. */
     enum EVerticalIdent
     {
         NOT_SET = 0,
@@ -346,7 +345,7 @@ public:
     ParallelBlocks* AddArcList(ArcList*l, AttributeList *al);
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh, bool first);
-    static bool AttributeValues(const std::string &attr, Csh &csh, bool first);
+    static bool AttributeValues(std::string_view attr, Csh &csh, bool first);
     EDirType GetToucedEntities(NEntityList &el) const override;
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
         Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
