@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -52,14 +52,14 @@ namespace msc {
 /** Name of the virtual entity representing the ultimate right side of the chart (of left comments), but notes may extend rightward*/
 #define END_ENT_STR "(end)"
 /** Name used for entity, when a pipe symbol is specified to the left.
- * Only used in a call to Msc::CreateArrow() or Arrow::AddSegment(). 
+ * Only used in a call to Msc::CreateArrow() or Arrow::AddSegment().
  * Those functions will know what to do with it.*/
 #define LSIDE_PIPE_ENT_STR "(left_side_pipe)"
  /** Name used for entity, when a pipe symbol is specified to the left.
- * Only used in a call to Msc::CreateArrow() or Arrow::AddSegment(). 
+ * Only used in a call to Msc::CreateArrow() or Arrow::AddSegment().
  * Those functions will know what to do with it.*/
 #define RSIDE_PIPE_ENT_STR "(right_side_pipe)"
- 
+
  /** Name of a virtual marker representing the current vertical location. Used in verticals when one end is not specified.*/
 #define MARKER_HERE_STR "\""
 /** Name of the builtin chart.top marker */
@@ -67,10 +67,10 @@ namespace msc {
 /** Name of the builtin chart.bottom marker */
 #define MARKER_BUILTIN_CHART_BOTTOM_STR "chart.bottom"
 
-/** The margin on left and right side, between LNote-LSide and RSide-RNote. 
+/** The margin on left and right side, between LNote-LSide and RSide-RNote.
  * In pos space, will be fed into XCoord().*/
 #define MARGIN 0.1
-/** The default size of the comment area in `pos` space if 
+/** The default size of the comment area in `pos` space if
  * - we have comments
  * - we have no 'hspace comment' commands and
  * - all comments are wrapped */
@@ -88,7 +88,7 @@ namespace msc {
 /** A pair of entity indexes for the purpose of later association with a required distance between them.*/
 class IPair {
 public:
-    unsigned first; ///<Index of the left entity. 
+    unsigned first; ///<Index of the left entity.
     unsigned second;///<Index of the right entity.
     IPair(unsigned e1, unsigned e2) : first(std::min(e1,e2)), second(std::max(e1,e2)) {}
     /** Calculates the difference between the two index values*/
@@ -149,25 +149,25 @@ namespace map_manip {
 }
 
 
-/** Stores three types of horizontal distance requirements between entities. 
- * One such object holds the collected horizontal distance requirements from a part 
+/** Stores three types of horizontal distance requirements between entities.
+ * One such object holds the collected horizontal distance requirements from a part
  * (called 'section') of the chart. A section is a point in the chart, distance requirements
  * are valid from that point downward - till the next section. The top of the section
  * can be either a marker or the top/bottom edges of a chart element.
  * These sections are collected in class DistanceRequirements.
  *
- * Entities are identified here by their index (an enumeration from left to right, with 
- * zero being the StartEntity (the leftmost) and 1 being the virtual entity representing the 
+ * Entities are identified here by their index (an enumeration from left to right, with
+ * zero being the StartEntity (the leftmost) and 1 being the virtual entity representing the
  * line between the leftside comments and the chart body. The largest index shall be
- * of the virtual entity representing the line between the right side comments and the 
+ * of the virtual entity representing the line between the right side comments and the
  * chart body.
  *
  * We store three types of distance requirements
  * - Between pairs of entities (e.g., a label on an arrow between them)
  *   (not necessarily neighbouring entities)
- * - On the left or right side of entities 
+ * - On the left or right side of entities
  *   (e.g., entitiy headings, box/pipe sides, arrowheads expanding beyond the entity line)
- *   Note that all entities shall report side distance requirements that already take entity 
+ *   Note that all entities shall report side distance requirements that already take entity
  *   activation into account.
  *
  * For each section, we also store which was the leftmost and rightmost entities used in that
@@ -198,7 +198,7 @@ public:
     /** Return the set of right distances. Ordered by entities distance.*/
     const std::map<unsigned, double> &GetRight() const { return right; }
 
-    void Insert(unsigned e1, int e2, double d);  
+    void Insert(unsigned e1, int e2, double d);
     bool IncreaseIfNonZero(unsigned e1, int e2, double d);
     double Query(unsigned e1, int e2) const; ///<Return a distance req
     /** Increase an existing distance requirement if already nonzero.
@@ -211,7 +211,7 @@ public:
     void CombineLeftRightToPair_Add(double gap);
 };
 
-/** A collection storing horizontal distance requirements between various 
+/** A collection storing horizontal distance requirements between various
  * entities in the chart. The chart is divided into 'sections' and distance
  * requirements are defined for a given section. Then the final positioning
  * of entities (and boxes and verticals) are composed from merging the
@@ -229,15 +229,15 @@ public:
 class DistanceRequirements
 {
 protected:
-    /** A list containing vertical positions and the horizontal 
+    /** A list containing vertical positions and the horizontal
      * space requirements just after this position, which are valid
      * until the next vertical position in the list.
-     * Vertical positions are not identified by y coordinates, 
+     * Vertical positions are not identified by y coordinates,
      * since we determine horizontal scaling before y layout, and this
      * mechanism is needed to manage horizontal scaling. Instead, we
-     * use markers and chart elements as they appear in the source 
+     * use markers and chart elements as they appear in the source
      * file, so vertical positions are akin to input file locations here.
-     * This is OK without parallel blocks, but may such with them. 
+     * This is OK without parallel blocks, but may such with them.
      * Anyway, this is a complicated enough algoritm already, be my
      * guest to do better.*/
     std::list<DistanceRequirementsSection> elements;
@@ -263,7 +263,7 @@ public:
     void InsertEntity(unsigned e) { elements.back().InsertEntity(e); }
     /** Indicate that an entity is touched after the last marker */
     void InsertEntity(EntityRef e) { elements.back().InsertEntity(e->index); }
-    /** Insert a side distance, which happened after the marker indicated and 
+    /** Insert a side distance, which happened after the marker indicated and
      * applies all the way to the current bottom. */
     void Insert(unsigned e1, int e2, double d, iterator from) { Insert(e1, e2, d, from, elements.end()); }
     /** Insert a side distance, which happened all along between two markers indicated. */
@@ -313,18 +313,18 @@ public:
 struct MscPageBreakData : public PageBreakData {
     const EntityCommand *autoHeading;  ///<A non-owning pointer to the automatically inserted heading command. nullptr if none.
     MscPageBreakData(double _y, double _y_span, bool m, EntityCommand *ce=nullptr, double h=0) :
-        PageBreakData(XY(0,_y), XY(0, _y_span)), autoHeading(ce) 
+        PageBreakData(XY(0,_y), XY(0, _y_span)), autoHeading(ce)
     { manual = m; headingLeftingSize.x = 0; headingLeftingSize.y = (h ? h : ce ? ce->GetFormalHeight() : 0);}
 };
 
-/**Helper struct to manage multiple list of arcs, sorted by 
+/**Helper struct to manage multiple list of arcs, sorted by
  * a criteria so that always the smallest shall be taken an element from.*/
 struct LayoutColumn
 {
-    /**If this list is put in as a nested parallel block, 
+    /**If this list is put in as a nested parallel block,
      * this is the parent list*/
     LayoutColumn * const parent;
-    /**If we have nested parallel blocks under processing 
+    /**If we have nested parallel blocks under processing
      * this is how many are not yet done.*/
     unsigned number_of_children;
     /** This is the actual list of arcs we contain.*/
@@ -335,14 +335,14 @@ struct LayoutColumn
     /** Tie-breaker: in case of everything equal, use the column number
      * This tells which column we were in our parallel block series.*/
     const unsigned column;
-    /** This is where the next (uncompressed) arc should be placed, 
+    /** This is where the next (uncompressed) arc should be placed,
      * the lowest part of any prior element in this column.
      * This is different from y_bottom_all due to elements
-     * marked by 'overlap' and 'parallel'. Those increase y_bottom_all, 
+     * marked by 'overlap' and 'parallel'. Those increase y_bottom_all,
      * but keep y the same.*/
     double y;
     /**we will never shift elements higher than this runnning value
-     * (due to compress). 
+     * (due to compress).
      * (any element marked with "parallel" will set this to its top)*/
     double y_upper_limit;
     /** True if the previous element was marked with parallel */
@@ -351,7 +351,7 @@ struct LayoutColumn
     Range previous_x_extent;
     /**the lowest element's bottom (largest num value) we have seen in 'list'
      * so far. The lowest of these will be returned from LayoutParallelArcLists().
-     * This is not always that of the last element so far, if the last 
+     * This is not always that of the last element so far, if the last
      * element has been shifted up due to compress and its bottom is no longer
      * the lowest.*/
     double y_bottom_all;
@@ -362,14 +362,14 @@ struct LayoutColumn
      * As long as not all children are completed.*/
     AreaList completed_children_covers;
     unsigned last_action;
-    LayoutColumn(ArcList *a, unsigned c, double Y = 0, double upper=0, bool par=false, 
+    LayoutColumn(ArcList *a, unsigned c, double Y = 0, double upper=0, bool par=false,
                  Range xext = Range(), LayoutColumn *p = nullptr, unsigned action=0) :
         parent(p), number_of_children(0), list(a), arc(list->begin()), column(c), y(Y),
         y_upper_limit(upper), previous_was_parallel(par), previous_x_extent(xext),
         y_bottom_all(Y), last_action(action) {}
     /** Return true if a particular column is my parent, which means that I was
      * defined in it. So in case of {a; b; {c; d;} e;} the outer column is a parent of the inner.*/
-    bool IsMyParent(const LayoutColumn *col) const 
+    bool IsMyParent(const LayoutColumn *col) const
         { const LayoutColumn *cand = this;  while (cand && col!=cand) cand = cand->parent; return nullptr != cand; }
 };
 
@@ -384,8 +384,8 @@ class MscProgress : public Progress<EBulkSection, EBulkSection::MAX_BULK_SECTION
                                      EArcSection, EArcSection::MAX_ARC_SECTION,
                                      EArcCategory, EArcCategory::MAX_CATEGORY>
 {
-public: 
-    explicit MscProgress(ProgressCallback cb = nullptr, void *d=nullptr, double g=1) 
+public:
+    explicit MscProgress(ProgressCallback cb = nullptr, void *d=nullptr, double g=1)
         : Progress({
             {EBulkSection::PARSE, "Parsing"},
             {EArcSection::POST_PARSE, "Post Parse Process"},
@@ -400,17 +400,17 @@ public:
         }, ProgressBase::EPreferredAbortMethod::EXCEPTION, cb, d, g) {}
 };
 
-/** The main class holding a signalling chart 
-  (This is a non movable, non copyable object, once it has arcs in it, due to the 
+/** The main class holding a signalling chart
+  (This is a non movable, non copyable object, once it has arcs in it, due to the
   cross-references between them.)
 
   # Coordinate Spaces
 
-  - The pixel coordinate space (1 pixel = 1/72 inch for vector output) is the most natural. 
-    Font heights are represented in pixels. This is also called "chart coordinate space" 
+  - The pixel coordinate space (1 pixel = 1/72 inch for vector output) is the most natural.
+    Font heights are represented in pixels. This is also called "chart coordinate space"
     when drawing, see the comment for Canvas.
   - The 'pos' coordinate space, which is only used on the horizontal axis to place entities.
-    The 'pos' attribute of entities uses this. For historical purposes 1 unit of the 'pos' 
+    The 'pos' attribute of entities uses this. For historical purposes 1 unit of the 'pos'
     coordinate space is 130 pixels. The conversion is done by Msc::XCoord().
 
   # Compress mechanism
@@ -428,20 +428,20 @@ class MscChart : public ChartBase<MscContext, MscProgress, MscRefNameData, MscPa
 public:
     /** @name Language attributes
     * @{ */
-    std::string GetLanguageDescription() const override { return "Signalling Chart"; } ///<Returns the human-readable (UTF-8) short description 
+    std::string GetLanguageDescription() const override { return "Signalling Chart"; } ///<Returns the human-readable (UTF-8) short description
     std::vector<std::string> GetLanguageExtensions() const override { return {"signalling", "msc"}; } ///<Returns a set of extensions in order of preference
     std::string GetLanguageEntityName() const override { return "entity"; }
     std::string GetLanguageDefaultText() const override; ///<Returns the default text of the chart in UTF-8
     bool GetLanguageHasAutoheading() const override { return true; } ///<True if the chart has automatic headings
     bool GetLanguageHasElementControls() const override { return true; } ///<True if the elements of the chart support GUI controls
-    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc, void *param) const override;
+    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc) const override;
     std::map<std::string, std::string> RegisterLibraries() const override;
     /** @} */
 
 
     /** Data we store about a marker */
     struct MarkerData {
-        FileLineCol line; ///<Marker's location in the input file, invalid if a built-in marker 
+        FileLineCol line; ///<Marker's location in the input file, invalid if a built-in marker
         double      y;    ///<Marker's vertical position in chart space
     };
 
@@ -449,22 +449,22 @@ public:
      * @{ */
     EntityList                    AllEntities;     ///<A list of all entities defined (in no particular order)
     NEntityList                   ActiveEntities;  ///<A list of the entities that are not hidden as part of a collapsed group entity
-    EntityRef                     StartEntity;     ///<A virtual entity representing the very left side of the chart. 
+    EntityRef                     StartEntity;     ///<A virtual entity representing the very left side of the chart.
     EntityRef                     LNote;           ///<A virtual entity representing the line separating the left side comments from the chart body.
-    EntityRef                     LSide;           ///<A virtual entity representing the left side of the chart body (some margin away from LNote) 
+    EntityRef                     LSide;           ///<A virtual entity representing the left side of the chart body (some margin away from LNote)
     EntityRef                     RSide;           ///<A virtual entity representing the right side of the chart body (some margin away from RNote)
-    EntityRef                     RNote;           ///<A virtual entity representing the line separating the right side comments from the chart body. 
-    EntityRef                     EndEntity;       ///<A virtual entity representing the very right side of the chart. 
+    EntityRef                     RNote;           ///<A virtual entity representing the line separating the right side comments from the chart body.
+    EntityRef                     EndEntity;       ///<A virtual entity representing the very right side of the chart.
     EntityAppList                 AutoGenEntities; ///<A list of entity appearance objects, one for each implicitly defined entity. After parsing, they are appended to the first EntityCommand.
     ArcList                       Arcs;            ///<The list of all arcs in the chart (in order of definition). Notes are moved away from here in PostParseProcess().
-    std::map<string, MarkerData>  Markers;         ///<A set of named markers. Each one was defined via a 'mark' command.
+    std::map<string, MarkerData, std::less<>> Markers; ///<A set of named markers. Each one was defined via a 'mark' command.
     std::map<double, FillAttr>    Background;      ///<A map of background fill indexed by the position downward of which this fill is applicable. Each entry was created by a background.* chart option.
     Contour                       HideELinesHere;  ///<A complex contour used as a mask when drawing entity lines.
     ArcList                       EndNotes;        ///<We move all endnotes here during PostParseProcessArcList(). We reappend them in PostParseProcess().
-    CommandNoteList               Notes;           ///<All notes are moved here after PostParseProcess 
+    CommandNoteList               Notes;           ///<All notes are moved here after PostParseProcess
     NPtrList<const MscElement>    NoteBlockers;    ///<Ptr to all elements that may block a floating note and which therefore should not overlap with them (we do not own them here)
     /** @} */
-    
+
 protected:
     /** @name Members holding calculated information on chart geometry
      * @{ */
@@ -484,42 +484,42 @@ public:
     double GetWidthAttr() const override { return width_attr; }
     /** Returns the default distance for special endings in pixels.
      * This is 30 multiplied by hscale or 30 for hscale==auto.
-     * We call it many times during parsing when hscale may still change, 
+     * We call it many times during parsing when hscale may still change,
      * which may lead to an ending having a bad size.
-     * But since hscale is usually set at the top of the file, this is 
+     * But since hscale is usually set at the top of the file, this is
      * seldom visible...*/
     double GetDefaultSpecialEndingSize() const { return XCoord(30./130.); }
 
     /** @name Members defining various spacing values.
      * These should have an integer value for nice drawing on bitmaps.
-     * @{ */    
-    
-    double chartTailGap;     ///<Gap at the bottom of the page for lengthening entity lines 
-    double selfArrowYSize;   ///<Self Pointing arc Y size. 
-    double headingVGapAbove; ///<Vertical gap above headings 
-    double headingVGapBelow; ///<Vertical gap below headings 
-    double boxVGapOutside;   ///<Vertical gap above and below boxes 
-    double boxVGapInside;    ///<Vertical gap inside boxes 
-    double arcVGapAbove;     ///<Vertical gap above and below arcs 
-    double arcVGapBelow;     ///<Vertical gap above and below arcs 
-    double discoVgap;        ///<How much extra space above and below a discontinuity line (...) 
-    double titleVgap;        ///<How much extra space above and below a title 
-    double subtitleVgap;     ///<How much extra space above and below a subtitle     
+     * @{ */
+
+    double chartTailGap;     ///<Gap at the bottom of the page for lengthening entity lines
+    double selfArrowYSize;   ///<Self Pointing arc Y size.
+    double headingVGapAbove; ///<Vertical gap above headings
+    double headingVGapBelow; ///<Vertical gap below headings
+    double boxVGapOutside;   ///<Vertical gap above and below boxes
+    double boxVGapInside;    ///<Vertical gap inside boxes
+    double arcVGapAbove;     ///<Vertical gap above and below arcs
+    double arcVGapBelow;     ///<Vertical gap above and below arcs
+    double discoVgap;        ///<How much extra space above and below a discontinuity line (...)
+    double titleVgap;        ///<How much extra space above and below a title
+    double subtitleVgap;     ///<How much extra space above and below a subtitle
     double nudgeSize;        ///<Size of the `nudge` command
-    double activeEntitySize; ///<The width of entity activation bars 
-    double compressGap;      ///<Size of gap we keep between elements at compress. We expand by half of it 
-    double hscaleAutoXGap;   ///<Size of horizontal gap between elements at hscale=auto     
-    double sideNoteGap;      ///<Gap between the side comment line and the comments     
+    double activeEntitySize; ///<The width of entity activation bars
+    double compressGap;      ///<Size of gap we keep between elements at compress. We expand by half of it
+    double hscaleAutoXGap;   ///<Size of horizontal gap between elements at hscale=auto
+    double sideNoteGap;      ///<Gap between the side comment line and the comments
     double defWNoteWidth;    ///<The default width for word wrapping notes in pos "space"
     double entityShapeHeight;///<The default height of an entity shape with size=normal.
     /** @} */
 
-    /** @name Parse Options 
+    /** @name Parse Options
      * @{ */
-    bool pedantic;                   ///<If we require pre-defined entities. 
+    bool pedantic;                   ///<If we require pre-defined entities.
     bool simple_arc_parallel_layout; ///<If false, we use MscChart::HeightArcLists() to lay out ParallelBlocks*/
-    EMscgenCompat mscgen_compat;     ///<How shall we approach mscgen compatibility. 
-       
+    EMscgenCompat mscgen_compat;     ///<How shall we approach mscgen compatibility.
+
     EntityCollapseCatalog force_entity_collapse;      ///<These entities must be collapsed/expanded
     BoxSignatureCatalog   force_box_collapse;         ///<These boxes must be collapsed/expanded, also contains an update after parsing
     BoxSignatureCatalog   force_box_collapse_instead; ///<During parsing, this holds an updated version of the box collapse/expanded instructions (removing nonexistent boxes, etc)
@@ -527,7 +527,7 @@ public:
 
     MscChart(FileReadProcedure *p, void *param);
     ~MscChart() override;
-    static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param) 
+    static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param)
         { return std::make_unique<MscChart>(p, param); }
     bool AddCommandLineArg(const std::string& arg) override;
     void AddCommandLineOption(const Attribute &a) override;
@@ -538,7 +538,7 @@ public:
     bool DeserializeGUIState(std::string_view) override;
     std::string SerializeGUIState() const override;
     bool ControlClicked(Element *, EGUIControlType) override;
-    int SetDesign(bool full, const string &design, bool force, ArcBase **ret, 
+    int SetDesign(bool full, const string &design, bool force, ArcBase **ret,
                   const FileLineColRange &l = FileLineColRange(FileLineCol(0,0,0), FileLineCol(0,0,0)));
     void SwitchToMscgenCompatMode();
 
@@ -546,9 +546,9 @@ public:
     ArcBase *AddAttribute(const Attribute&);
     bool AddDesignAttribute(const Attribute&);
     static void AttributeNames(Csh &csh, bool designOnly);
-    static bool AttributeValues(const std::string attr, Csh &csh);
+    static bool AttributeValues(std::string_view attr, Csh &csh);
 
-    MarkerData* GetMarker(const std::string &name);
+    MarkerData* GetMarker(std::string_view name);
 
     /** Return the entity with the smaller (min=true) or larger (min=false) `pos` value*/
     EntityRef EntityMinMaxByPos(EntityRef i, EntityRef j, bool min) const;
@@ -564,7 +564,7 @@ public:
     CEntityRef EntityMaxByPos(CEntityRef i, CEntityRef j) const { return EntityMinMaxByPos(i, j, false); }
     /** Finds entity named `e` (mentioned at position `l` in the source file).
      * If not found, it creates one (and uses 'l' only then). */
-    EntityRef FindAllocEntity(const char *e, const FileLineColRange &l);
+    EntityRef FindAllocEntity(std::string_view e, const FileLineColRange &l);
     EntityRef FindLeftRightDescendant(EntityRef, bool left, bool stop_at_collapsed);
     EntityRef FindActiveParentEntity(EntityRef);
     EntityRef FindWhoIsShowingInsteadOf(EntityRef, bool left);
@@ -585,9 +585,9 @@ public:
     bool IsVirtualEntity(NEntityList::const_iterator e) const { return IsVirtualEntity(*e); }
     /** Moves the contents of `a` to the end of the arc list of the chart*/
     void AddArcs(ArcList *a) {if (!a) return; Arcs.splice(Arcs.end(), *a); delete a;}
-    Arrow *CreateArrow(ArrowSegmentData data, const char*s, const FileLineColRange &sl,
-                       const char*d, bool fw, const FileLineColRange &dl, bool had_param);
-    /** Create a block arrow from an arrow. Returns null, emits an error if not possible 
+    Arrow *CreateArrow(ArrowSegmentData data, std::string_view s, const FileLineColRange &sl,
+                       std::string_view d, bool fw, const FileLineColRange &dl, bool had_param);
+    /** Create a block arrow from an arrow. Returns null, emits an error if not possible
      * On success, it brings the given arc to a "valid, but unspecified state" .*/
     BlockArrow *CreateBlockArrow(ArcBase *);
     /** Pop the top context. May return arcs to be appended as a result (SetNumbering)*/
@@ -603,10 +603,10 @@ public:
 
     EDirType GetTouchedEntitiesArcList(const ArcList &, NEntityList &el, EDirType dir=EDirType::INDETERMINATE) const;
 
-    /** Returns the final value of the hscale chart option. 
+    /** Returns the final value of the hscale chart option.
      * We take it from the outermost scope, even if there are smaller scopes
      * still open. This is because if the user opens a "defdesign {" at the very
-     * end of the file, without closing it, a totally empty (no hscale) context 
+     * end of the file, without closing it, a totally empty (no hscale) context
      * will be the current at the end of the parsing. */
     double GetHScale() const {_ASSERT(Contexts.size() && Contexts.front().hscale); return *Contexts.front().hscale;}
     /** Converts an x coordinate from pos coordinat space to pixel space. Always returns an integer */
@@ -627,7 +627,7 @@ public:
     double PageBreakArcList(Canvas &canvas, ArcList &arcs, double netPrevPageSize,
                             double pageBreak, bool &addCommandNewpage, bool addHeading,
                             bool canChangePBPos, bool dontshiftall);
-    void CollectPageBreaks(); 
+    void CollectPageBreaks();
     void CollectPageBreakArcList(ArcList &arcs);
     void AutoPaginate(Canvas &canvas, double pageSize, bool addHeading);
     void CalculateWidthHeight(Canvas &canvas,
@@ -649,8 +649,8 @@ public:
     void RegisterAllLabels() override { RegisterLabelArcList(Arcs); } ///<Register all labels in Arcs
     void CollectIsMapElementsArcList(ArcList &arcs, Canvas &canvas) { for (auto &pArc : arcs) pArc->CollectIsMapElements(canvas); }  ///<Collect all ISMAPs from an arc list.
     void CollectIsMapElements(Canvas &canvas) override { CollectIsMapElementsArcList(Arcs, canvas); } ///<Collect all ISMAPs from an Arcs
-    void CompleteParse(bool autoPaginate=false, 
-                       bool addHeading=true, XY pageSize=XY(0,0), 
+    void CompleteParse(bool autoPaginate=false,
+                       bool addHeading=true, XY pageSize=XY(0,0),
                        bool fitWidth=true, bool collectLinkInfo=false) override;
 
     void DrawEntityLines(Canvas &canvas, double y, double height, NEntityList::const_iterator from, NEntityList::const_iterator to);
@@ -681,9 +681,6 @@ void MscChart::FinalizeLabelsArcList(list &arcs, Canvas &canvas)
 
 using msc::MscChart;
 
-/** Global function used to parse a chart. Body defined in language.cc.*/
-void MscParse(MscChart &msc, const char *buff, unsigned len);
-
 #endif
 
 /* END OF FILE */
