@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -25,20 +25,17 @@
 #ifndef MSC_PARSE_TOOLS_H
 #define MSC_PARSE_TOOLS_H
 
-#include "parse_tools.h"
+#include "parser_tools.h"
 #include "msc.h"
 
-#ifndef CHAR_IF_CSH
-#define CHAR_IF_CSH(A) A
-#endif
-
 
 namespace msc {
 
-std::string ConvertEmphasisToBox(const std::string &style, const FileLineCol &loc, MscChart &msc);
-
+std::string_view ConvertEmphasisToBox(std::string_view style, const FileLineCol &loc, MscChart &msc);
+std::pair<std::string, std::string>
+    BeautifySyntaxError(std::string msg, std::string_view last_token);
+std::string process_multiline_qstring(std::string_view s, FileLineCol loc);
 }; //namespace
 
-void yyerror(YYLTYPE*loc, MscChart &msc, void *yyscanner, const char *str);
 
 #endif //MSC_PARSE_TOOLS_H
\ No newline at end of file
