@@ -1,6 +1,6 @@
   /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -222,7 +222,7 @@ void MscArrowHeads::Draw(Canvas &canvas, XY xy, double act_size,
  *          middle on the arrow. 'first' returns the size on the left side,
  *          'second' returns the space on the right side.*/
 DoublePair MscArrowHeads::getBigWidthsForSpace(bool forward, bool bidir, EArrowEnd which,
-                                               double body_height, double act_size, const LineAttr &ltype) const
+                                               double body_height, double act_size [[maybe_unused]], const LineAttr &ltype) const
 {
     DoublePair ret(0, 0);
     auto ah = GetArrowHead(bidir, which);
