@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -44,8 +44,9 @@ public:
     WidthAttr width; ///<The width specified by the used. second<0 means the string contains the width
     NoteAttr() { Empty(); MakeComplete(); } ///<Create a fully specified note style with default values (Callout type, no specific position preference.)
     void Empty() { pointer.reset(); def_float_dist.reset(); def_float_x.reset(); def_float_y.reset(); width.Empty(); } ///<Clear all content from the note style.
+    bool IsEmpty() const noexcept { return !pointer && !def_float_dist && !def_float_x && !def_float_y && width.IsEmpty(); } ///<True if none of the note attributes are set. 
     void MakeComplete();
-    bool IsComplete() const { return pointer && def_float_dist && def_float_x && def_float_y; } ///<True if all of the note attributes are set. 
+    bool IsComplete() const { return pointer && def_float_dist && def_float_x && def_float_y && !width.IsEmpty(); } ///<True if all of the note attributes are set. 
     NoteAttr &operator +=(const NoteAttr&a); ///<Applies `a` to us: sets all our attributes, which are set in `a` to the value in `a`; leaves the rest unchanged.
     bool operator == (const NoteAttr &a) const;
     virtual bool AddAttribute(const Attribute &a, MscChart *chart, EStyleType t);
