@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -24,12 +24,12 @@
 #define _USE_MATH_DEFINES
 #include <math.h>
 #include <map>
-#include <cstring>   //strdup, free
 #include "msccsh.h"
 #include "cgen_color.h"
 #include "utf8utils.h" //for UTF8len
 #include "stringparse.h" //for extracting csh out of strings
 #include "cgen_attribute.h"  //for CaseInsensitive compares
+#include "msc_parser_csh.h"
 #include "msc.h"
 
 using namespace std;
@@ -129,7 +129,7 @@ void MscCsh::AddCSH_StyleOrAttrName(const CshPos&pos, std::string_view name)
         AddCSH(pos, COLOR_ATTRVALUE);
         was_partial = true;
         return;
-    } 
+    }
     //If we do not type or is it a prefix it add error
     AddCSH_Error(pos, "Unknown attribute or style name.");
 }
@@ -137,7 +137,7 @@ void MscCsh::AddCSH_StyleOrAttrName(const CshPos&pos, std::string_view name)
 
 void MscCsh::AddCSH_EntityOrMarkerName(const CshPos&pos, std::string_view name)
 {
-    if (EntityNames.find(string(name)) != EntityNames.end()) 
+    if (EntityNames.find(string(name)) != EntityNames.end())
         AddCSH(pos, COLOR_ENTITYNAME);
     else
         AddCSH(pos, COLOR_MARKERNAME);
@@ -218,7 +218,7 @@ bool msc::CshHintGraphicCallbackForMarkers(Canvas *canvas, CshHintGraphicParam /
     shadow.offset = 3;
     shadow.blur = 3;
 
-    Contour c(XY(int(HINT_GRAPHIC_SIZE_X*0.3), int(HINT_GRAPHIC_SIZE_Y*0.2)), 
+    Contour c(XY(int(HINT_GRAPHIC_SIZE_X*0.3), int(HINT_GRAPHIC_SIZE_Y*0.2)),
               XY(int(HINT_GRAPHIC_SIZE_X*0.3), int(HINT_GRAPHIC_SIZE_Y*0.8)),
               XY(int(HINT_GRAPHIC_SIZE_X*0.8), int(HINT_GRAPHIC_SIZE_Y*0.5)));
     canvas->Shadow(c, shadow);
@@ -227,9 +227,9 @@ bool msc::CshHintGraphicCallbackForMarkers(Canvas *canvas, CshHintGraphicParam /
     return true;
 }
 
-void MscCsh::BeforeYaccParse(const std::string& input, int cursor_p)
+void MscCsh::BeforeYaccParse(std::string&& input, int cursor_p)
 {
-    Csh::BeforeYaccParse(input, cursor_p);
+    Csh::BeforeYaccParse(std::move(input), cursor_p);
     ResetMarkers();
 }
 
@@ -253,22 +253,24 @@ void MscCsh::AfterYaccParse()
  * @param [in] cursor_p The current position of the cursor.
  * @param [in] pedantic The initial value of the pedantic chart option.
    (We ignore it in CSH parsing.)*/
-void MscCsh::ParseText(const std::string& input, int cursor_p, bool pedantic)
+void MscCsh::ParseText(std::string&& input, int cursor_p, bool pedantic)
 {
     //initialize data struct
-    BeforeYaccParse(input, cursor_p);
+    BeforeYaccParse(std::move(input), cursor_p);
     (void)pedantic;
     //call parsing
-    MscCshParse(*this, input_text.c_str(), unsigned(input_text.length()+1)); //include terminating zero
+    sv_reader<true> reader(input_text, 0);
+    msc::lexing_state state = mscgen_compat == EMscgenCompat::FORCE_MSCGEN ? msc::lexing_state::mscgen_compat: msc::lexing_state::basic;
+    msc_csh_parse(*this, state, reader); //return value ignored
     //Tidy up afterwards
     AfterYaccParse();
 }
 
-void MscCsh::ParseDesignText(const std::string& input, int cursor_p) {
+void MscCsh::ParseDesignText(std::string&& input, int cursor_p) {
     //Design libs shall be parsed with EMscgenCompat::AUTODETECT.
     //Save & restore the value
     const EMscgenCompat saved_mscgen_compat = std::exchange(mscgen_compat, EMscgenCompat::AUTODETECT);
-    ParseText(input, cursor_p, true);
+    ParseText(std::move(input), cursor_p, true);
     mscgen_compat = saved_mscgen_compat;
 }
 
@@ -278,9 +280,9 @@ void MscCsh::ParseDesignText(const std::string& input, int cursor_p) {
 //use defaultDesign only to get default styles and colors to produce hints.
 //Styles are polymorphic, so their AttributeNames() and AttributeValues()
 //will return even the MscStyle specific attributes, even if stored in a ContextBase.
-MscCsh::MscCsh(Csh::FileListProc proc, void *param) :
-    Csh(MscContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc, param),
-    addMarkersAtEnd(false), 
+MscCsh::MscCsh(Csh::FileListProc proc) :
+    Csh(MscContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc),
+    addMarkersAtEnd(false),
     hint_vertical_shapes(false),
     mscgen_compat(EMscgenCompat::AUTODETECT)
 {
@@ -332,11 +334,11 @@ void MscCsh::AddSymbolTypesToHints()
 /** Add the left/right/center keyword (for symbol positioning) to the hints.*/
 void MscCsh::AddLeftRightCenterToHints()
 {
-    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "left", 
+    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "left",
         "Use this if you want to specify where the left edge shall be positioned.",
         EHintType::KEYWORD, true, CshHintGraphicCallbackForTextIdent,
         CshHintGraphicParam(MSC_IDENT_LEFT)));
-    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "right", 
+    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "right",
         "Use this if you want to specify where the right edge shall be positioned.",
         EHintType::KEYWORD, true, CshHintGraphicCallbackForTextIdent,
         CshHintGraphicParam(MSC_IDENT_RIGHT)));
@@ -385,18 +387,18 @@ void MscCsh::AddVerticalTypesToHints()
 void MscCsh::AddEntitiesToHints()
 {
     for (auto &e : EntityNames)
-        AddToHints(CshHint(HintPrefix(COLOR_ENTITYNAME) + e, nullptr, EHintType::ENTITY, true, 
+        AddToHints(CshHint(HintPrefix(COLOR_ENTITYNAME) + e, nullptr, EHintType::ENTITY, true,
             CshHintGraphicCallbackForMscEntities));
 }
 
 /** Add chart option names to the list of hints. */
-void MscCsh::AddOptionsToHints() 
+void MscCsh::AddOptionsToHints()
 {
     MscChart::AttributeNames(*this, false);
 }
 
 /** Add names of chart options that are valid inside a design definition to the list of hints. */
-void MscCsh::AddDesignOptionsToHints() 
+void MscCsh::AddDesignOptionsToHints()
 {
     MscChart::AttributeNames(*this, true);
 }
@@ -493,7 +495,7 @@ bool msc::CshHintGraphicCallbackForMscEntities(Canvas *canvas, CshHintGraphicPar
 
 void MscCsh::AddVertXPosSyntaxNonSelectableToHints(bool include_at)
 {
-    const string prefix = include_at ? 
+    const string prefix = include_at ?
                            "\\|"+HintPrefix(COLOR_KEYWORD)+"at\\s()"+HintPrefixNonSelectable() :
                            HintPrefixNonSelectable();
     AddToHints(CshHint(prefix + "<entity> [offset]",
@@ -522,7 +524,7 @@ void MscCsh::AddArrowSpecifiersToHints()
 {
     AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "lost at",
         "Use the 'lost at' keyword to indicate that the message has been lost.",
-        EHintType::KEYWORD, true)); 
+        EHintType::KEYWORD, true));
     AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "start before",
         "Use the 'start before' keyword to indicate where the message originates from.",
         EHintType::KEYWORD, true));
@@ -618,7 +620,7 @@ void MscCsh::FillNamesHints()
     Symbol::AttributeNames(*this);
     AddVSpace::AttributeNames(*this);
     EntityApp::AttributeNames(*this);
-    //At this point we do not have shapes read from the default lib, so 
+    //At this point we do not have shapes read from the default lib, so
     //SimpleStyle::Attributenames will not add these.
     AddToHints(CshHint(HintPrefix(COLOR_ATTRNAME) + "shape",
         "Set the shape of the entity.",
