@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -32,15 +32,15 @@ class MscContext;
 /** This message is given when a user specifies multiple asterisks in an arrow.*/
 #define MULTIPLE_ASTERISK_ERROR_MSG "One arrow may be lost only once. Use a single asterisk ('*')."
 
-/** A class collecting syntax coloring infor and hints during a csh parse.
+/** A class collecting syntax coloring info and hints during a csh parse.
  *
  * A few words on how hints are collected.
  * Often we realize what is the type of hint in a low-level yacc rule, but know no specifics.
- * For example, we have yacc rues describing how an attribute looks like, that is "name = value". 
+ * For example, we have yacc rues describing how an attribute looks like, that is "name = value".
  * In this rule we can realize that the cursor is inside value (or we match to a rule saying
  * "name =" and realize that the cursor is just after). Here we can conclude that the hint
  * will be a sort of attribute value. (So we pick EHintType of HINT_ATTRVALUE.) But we do not know
- * what values the attribute can take, because that depens on what arc we are parsing. So after we macth
+ * what values the attribute can take, because that depends on what arc we are parsing. So after we match
  * the low level rule, we set the hint type only and when processing the higher level rule for the
  * full arc, do we fill in the actual hints. In the meantime we set the hintStatus member to
  * HINT_LOCATED. Some cases we cannot fill in all the hints in one go and must go to an even higher
@@ -57,10 +57,10 @@ protected:
     void FillNamesHints() override;
     /** @}*/
     bool DoHintLocated(EHintSourceType hsource, std::string_view a_name) override;
-    void BeforeYaccParse(const std::string& input, int cursor_p) override;
+    void BeforeYaccParse(std::string&& input, int cursor_p) override;
     void AfterYaccParse() override;
 public:
-    /** @name The collected CSH info and hints 
+    /** @name The collected CSH info and hints
      * @{ */
     std::set<std::string>  MarkerNames;          ///<The names of markers defined so far
     unsigned               asteriskNo;           ///<Number of asterisks inside an arrow spec so far.
@@ -73,7 +73,7 @@ public:
     /** @} */
 
     /** Initializes the Csh Object.*/
-    MscCsh(Csh::FileListProc proc, void *param);
+    explicit MscCsh(Csh::FileListProc proc);
     ~MscCsh() override = default;
     std::unique_ptr<Csh> Clone() const override { return std::make_unique<MscCsh>(*this); }
     void clear() override { MarkerNames.clear(); asteriskNo = 0; addMarkersAtEnd = false; }
@@ -83,7 +83,7 @@ public:
     /** Clean up after parsing a design library */
     void CleanupAfterDesignlib() override;
 
-    /** @name Functions to add a CSH entry 
+    /** @name Functions to add a CSH entry
      * @{  */
     void AddCSH_ErrorAfter_Mscgen(const CshPos&pos) { if (mscgen_compat != EMscgenCompat::FORCE_MSCGEN) AddCSH_ErrorAfter(pos, "This is mscgen syntax."); } ///<Notifies of an mscgen syntax, if not in compatibility mode
     void AddCSH_AttrName(const CshPos&, std::string_view name, EColorSyntaxType) override; ///<At pos there is either an option or attribute name (specified by the type). Search and color.
@@ -93,8 +93,8 @@ public:
     void AddCSH_SymbolName(const CshPos&pos, std::string_view name); ///<At pos there is a symbol. Color it.
     void AddCSH_LeftRightCenterMarker(const CshPos&pos, std::string_view name); ///<At pos there is either left/right/center or a marker. Color it appropriately.
     /** @}*/
-    void ParseText(const std::string& input, int cursor_p, bool pedantic) override;
-    void ParseDesignText(const std::string& input, int cursor_p) override;
+    void ParseText(std::string&& input, int cursor_p, bool pedantic) override;
+    void ParseDesignText(std::string&& input, int cursor_p) override;
     void SwitchToMscgenCompatMode();
 
     /** @name Add specific items to the list of hints.
@@ -123,5 +123,4 @@ bool CshHintGraphicCallbackForMarkers(Canvas *canvas, CshHintGraphicParam /*p*/,
 
 } //namespace
 
-void MscCshParse(msc::MscCsh &csh, const char *buff, unsigned len);
 #endif
