@@ -1,6 +1,6 @@
 /*
   This file is part of Msc-generator.
-  Copyright (C) 2008-2022 Zoltan Turanyi
+  Copyright (C) 2008-2023 Zoltan Turanyi
   Distributed under GNU Affero General Public License.
 
   Msc-generator is free software: you can redistribute it and/or modify
@@ -64,7 +64,7 @@ protected:
     const MscStyleCoW
         indicator_style;    ///<The style to be used if we need to replace this with an indicator
                             /** Returns the size of an indicator that is supposed to replace us considering `indicator_style`*/
-    XY GetIndiactorSize() const { const double a = *indicator_style.read().shadow.offset + indicator_style.read().line.LineWidth()*2; return indicator_size+XY(a, a); }
+    XY GetIndicatorSize() const { return Element::GetIndicatorSize(indicator_style.read().line, indicator_style.read().fill, indicator_style.read().shadow); }
 
 public:
     EMscDrawPass    draw_pass; ///<Gives the Z-order position of this arc
@@ -77,7 +77,7 @@ public:
 
     virtual bool AddAttribute(const Attribute &a);
     static void AttributeNames(Csh &csh);
-    static bool AttributeValues(const std::string attr, Csh &csh);
+    static bool AttributeValues(std::string_view attr, Csh &csh);
     void ShiftBy(double y) override;
     virtual void AttachComment(Note *cn);
     void CombineComments(MscElement *); //move comments to us
@@ -88,8 +88,8 @@ public:
     virtual void RegisterCover(EMscDrawPass pass) const;
 
 
-    Block GetIndicatorCover(const XY &pos) const;
-    void DrawIndicator(XY pos, Canvas *canvas) const;
+    Block GetIndicatorCover(const XY& pos) const { return Element::GetIndicatorCover(pos, indicator_style.read().line, indicator_style.read().fill, indicator_style.read().shadow); }
+    void DrawIndicator(const XY& pos, Canvas* canvas) const { Element::DrawIndicator(pos, canvas, indicator_style.read().line, indicator_style.read().fill, indicator_style.read().shadow); }
 };
 
 } //msc namespace
