@@ -401,7 +401,7 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
     std::unique_ptr<EntityAppHelper> ret = std::make_unique<EntityAppHelper>();
     //If we have children, add them to "ret->entities"
     ret->target = name; //this will be the target for any note after us.
-    if (ch)
+    if (ch) {
         for (auto j = ch->begin(); j!=ch->end(); /*nope*/)
             if (EntityCommand* const ce = dynamic_cast<EntityCommand*>(j->get())) {
                 ce->MoveMyContentAfter(*ret);  //ce is emptied out of all EntityDefs and tmp stored notes
@@ -417,6 +417,7 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
                 chart->Error.Error((*j)->file_pos.start, "Only entity definitions are allowed here. Ignoring this.");
                 j++;
             }
+    }
     delete ch;
 
     //Remove children we are not defining
