@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -151,12 +151,12 @@ public:
     using std::list<Entity>::emplace_back;
     /** Find an entity by name.
       * If not found, returns nullptr*/
-    EntityRef Find_by_Name(const string &name) 
-    {auto i = std::find_if(begin(), end(), [&name](auto &p) {return p.name==name; }); return i==end() ? nullptr : &*i;} 
+    EntityRef Find_by_Name(std::string_view name)
+    {auto i = std::find_if(begin(), end(), [name](auto &p) {return p.name==name; }); return i==end() ? nullptr : &*i;}
     /** Find an entity by name.
     * If not found, returns nullptr*/
-    CEntityRef Find_by_Name(const string &name) const
-    { auto i = std::find_if(begin(), end(), [&name](auto &p) {return p.name==name; }); return i==end() ? nullptr : &*i; }
+    CEntityRef Find_by_Name(std::string_view name) const
+    { auto i = std::find_if(begin(), end(), [name](auto &p) {return p.name==name; }); return i==end() ? nullptr : &*i; }
     /** Find an entity by pointer.
       * If not found, returns end()*/
     iterator Find_by_Ptr(Entity *p)
@@ -175,7 +175,7 @@ public:
     { return std::find_if(begin(), end(), [index](auto &pp) {return pp.index==index; }); }
 };
 
-/** A list of Entity object pointers ownning their content.
+/** A list of Entity object pointers owning their content.
 * Has special search functions */
 class NEntityList : public NPtrList<Entity>
 {
@@ -222,7 +222,7 @@ using StoredNotes = std::vector<StoredNote>;
  * This is used during parsing to build up EntityCommand objects. */
 struct EntityAppHelper
 {
-    EntityAppList entities; ///<A list of entity definitions. 
+    EntityAppList entities; ///<A list of entity definitions.
     StoredNotes   notes;    ///<A set of notes associated with one of the entities in `entities` member.
     std::string   target;   ///<A name of an entity in `entity` to be the target for a subsequent note, if that has no 'at' clause.
     /** Prepend a set of entities, and merge the list of notes.*/
@@ -233,9 +233,9 @@ class DirArrow;
 
 /** Describes information for each appearance of an entity in the input.
  * Each time we mention an entity (not as part of an arc, but to change its attributes)
- * one EntityApp is created. 
- * There is a special EntityApp for each Entity though, the first one, the one which 
- * actually *defines* the Entity. Parsing always generates EntityApp objects and 
+ * one EntityApp is created.
+ * There is a special EntityApp for each Entity though, the first one, the one which
+ * actually *defines* the Entity. Parsing always generates EntityApp objects and
  * if we see that the Entity does not exist yet, we create one Entity object.
  * So this class have to be able contain all things and entity definition needs,
  * like label, grouping and so on, not just the things that may be changed later on.
@@ -244,9 +244,9 @@ class DirArrow;
  *
  * The creation of this object goes as follows.
  * (Note that entities can be listed separated by commas, each may have attributes
- * they can also have child entities behind them in curly braces, plus they can be 
- * preceeded by one of `show`, `hide`, `activate` and `deactivate` keyword, which
- * is syntactic sugar to set the corresponding attribute in all entitydefs in the 
+ * they can also have child entities behind them in curly braces, plus they can be
+ * preceded by one of `show`, `hide`, `activate` and `deactivate` keyword, which
+ * is syntactic sugar to set the corresponding attribute in all entitydefs in the
  * comma-separated list.)
  * - first we parse it learning its name (`entity_string` rule in language.yy)
  * - then we parse its associated attributes (rule `full_arcattrlist_with_label`)
@@ -264,9 +264,9 @@ public:
     const string        name;                ///<The name of the Entity. Empty if we refer to multiple already, defined entities.
     const bool          only_shown_if_multi; ///<If name is empty, we refer to only visible entities (true) or non-visible ones (false).
     OptAttrLine<string> label;               ///<The label specified at this mention of the entity (if any). `file_pos` contains the position of the attribute (name).
-    FileLineCol         linenum_label_value; ///<Locatin of label text (attribute value) in the input file (if any). Only a minor difference to `label.file_pos`.
+    FileLineCol         linenum_label_value; ///<Location of label text (attribute value) in the input file (if any). Only a minor difference to `label.file_pos`.
     string              url;                 ///<The value of the url attribute (if any).
-    FileLineCol         linenum_url_attr;    ///<Locatin of url attribute (if any).
+    FileLineCol         linenum_url_attr;    ///<Location of url attribute (if any).
     OptAttrLine<double> pos;                 ///<The `pos=` attribute if specified by the user. `file_pos` contains the location of the attribute (name) in the input file.
     OptAttrLine<string> rel;                 ///<The `rel=` attribute if specified by the user. `file_pos` contains the location of the attribute (name) in the input file.
     OptAttrLine<bool>   large;               ///<The `large=` attribute if specified by the user. `file_pos` contains the location of the attribute (name) in the input file.
@@ -292,14 +292,14 @@ public:
     mutable Block  outer_edge;           ///<The outer edge of a heading we have to draw, set in Height().
     mutable double indicator_ypos_offset;///<The y position of the indicator of a collapsed group entity within the entity heading. -1 if no indicator to be shown.
 
-    EntityApp(const char *s, MscChart* chart);
-    EntityApp(MscChart* chart, bool shown_only, const char *prefix, const  FileLineCol &l);
+    EntityApp(std::string_view s, MscChart* chart);
+    EntityApp(MscChart* chart, bool shown_only, std::string_view prefix, const  FileLineCol &l);
 
     bool AddAttribute(const Attribute&) override;
     std::unique_ptr<EntityAppHelper> AddAttributeList(AttributeList *, ArcList *children, FileLineCol l);
     static void AttributeNames(Csh &csh);
-    static bool AttributeValues(const std::string attr, Csh &csh);
-    void Combine(EntityApp *ed); 
+    static bool AttributeValues(std::string_view attr, Csh &csh);
+    void Combine(EntityApp *ed);
 
     double Width() const;
     Range Height(Area &cover, const NPtrList<EntityApp> &edl);
