@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -142,6 +142,25 @@ void MscStyle::Empty()
     tag_text.Empty();
 }
 
+/** Make the style empty be unsetting all attributes it contains.*/
+bool MscStyle::IsEmpty() const noexcept 
+{
+    return SimpleStyleWithArrow<MscArrowHeads>::IsEmpty()
+        && (!f_vline || vline.IsEmpty())
+        && (!f_vfill || vfill.IsEmpty())
+        && (!f_alt || (aline.IsEmpty() && atext.IsEmpty()))
+        && (!f_solid || !solid)
+        && (f_side== ESideType::NO || !side)
+        && (!f_vspacing || !vspacing)
+        && (f_indicator ||!indicator)
+        && (f_makeroom || !makeroom)
+        && (!f_note || note.IsEmpty())
+        && (!f_lost || (lost_line.IsEmpty() && lost_arrow.read().IsEmpty() && lost_text.IsEmpty()))
+        && (!f_lsym || (lsym_line.IsEmpty() && (!lsym_size || !lsym_size)))
+        && (!f_tag || (tag_line.IsEmpty() && tag_fill.IsEmpty() && tag_text.IsEmpty()));
+}
+
+
 /** Merge another style to us by copying the value of those attributes which are contained by both and are set in `toadd`.*/
 Style & MscStyle::operator +=(const Style &t)
 {
