@@ -1,6 +1,6 @@
                 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -81,9 +81,9 @@ public:
     MscStyle &operator=(const MscStyle&) = default;
     MscStyle &operator=(MscStyle&&) = default;
     void Empty() override;
+    bool IsEmpty() const noexcept override;
     void MakeCompleteButText() override;
     Style &operator +=(const Style &toadd) override;
-    Style operator +(const Style &toadd) const override { return MscStyle(*this) += toadd; }
     bool AddAttribute(const Attribute &a, Chart *chart) override;
     bool DoIAcceptUnqualifiedColorAttr() const override;
     void AttributeNames(Csh &csh) const override;
@@ -124,23 +124,23 @@ public:
     FillAttr defCommentFill;     ///<The fill of the comment area
     FillAttr defBackground;      ///<The background
     /** This constructor is used to create an empty context or one set to plain.
-     * Used an initializing the first context at parse, when starting to record 
-     * a design, storing a procedure or moving a newly defined design to the design 
+     * Used an initializing the first context at parse, when starting to record
+     * a design, storing a procedure or moving a newly defined design to the design
      * store.
      * @param [in] f If true, the context contains a value for all styles and attributes (Full)
      * @param [in] p Tells us what components to observe and how to behave during parsing.
      * @param [in] t Tells us with what content to create the context. It should not be 'COPY
      * @param [in] l The first character of the context in the input file.*/
-    MscContext(bool f, EContextParse p, EContextCreate t, const FileLineCol &l) : 
+    MscContext(bool f, EContextParse p, EContextCreate t, const FileLineCol &l) :
         ContextBase<MscStyle>(f, p, EContextCreate::CLEAR, l)
     { switch (t) {
         default: _ASSERT(0); FALLTHROUGH;
         case EContextCreate::PLAIN: Plain(); break;
-        case EContextCreate::EMPTY: Empty(); break; 
+        case EContextCreate::EMPTY: Empty(); break;
         case EContextCreate::CLEAR: defCommentFill.Empty(); defCommentLine.Empty(); defBackground.Empty();
             hscale.reset(); numbering.reset(); vspacing.reset(); indicator.reset(); slant_angle.reset(); slant_depth.reset(); auto_heading.reset();
     }}
-    /** This constructor is used when a context needs to be duplicated due to 
+    /** This constructor is used when a context needs to be duplicated due to
      * the opening of a new scope. You can change the parse mode, e.g., when
      * parsing the not-selected branch of an ifthenelse.*/
     MscContext(const MscContext &o, EContextParse p, const FileLineCol &l) :
