@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -33,12 +33,16 @@
 #include "canvas.h"
 #include "xxxchart.h"
 #include "xxxcsh.h"
+#include "xxx_parser_compile.h"
 
 using namespace xxx;
 
-std::unique_ptr<Csh> XxxChart::CshFactory(Csh::FileListProc proc, void *param) const
+XxxInstruction::XxxInstruction(XxxChart&) {
+}
+
+std::unique_ptr<Csh> XxxChart::CshFactory(Csh::FileListProc proc) const
 {
-    return std::make_unique<XxxCsh>(proc, param);
+    return std::make_unique<XxxCsh>(proc);
 }
 
 std::map<std::string, std::string> XxxChart::RegisterLibraries() const
@@ -62,7 +66,7 @@ XxxChart::~XxxChart()
 }
 
 
-std::unique_ptr<XxxInstrList> XxxChart::PopContext()
+XxxInstrList XxxChart::PopContext()
 {
     Contexts.pop_back();
     //you may return an instruction if needed. It will be the last instruction
@@ -95,7 +99,7 @@ bool XxxChart::AddDesignAttribute(const Attribute&)
 
 
 
-bool XxxChart::DeserializeGUIState(std::string_view s)
+bool XxxChart::DeserializeGUIState(std::string_view)
 {
     return true;
 }
@@ -105,7 +109,7 @@ std::string XxxChart::SerializeGUIState() const
     return std::string();
 }
 
-bool XxxChart::ControlClicked(Element *arc, EGUIControlType t)
+bool XxxChart::ControlClicked(Element*, EGUIControlType)
 {
     return false;
 }
@@ -137,15 +141,15 @@ bool XxxChart::ApplyForcedDesign(const string & name)
 unsigned XxxChart::ParseText(std::string_view input, std::string_view filename)
 {
     unsigned ret = current_file = Error.AddFile(filename);
-    //if you support progres::
+    //if you support progress::
     //Progress.RegisterParse(len);
-    XxxParse(*this, input.data(), unsigned(input.length()));
+    sv_reader<false> reader(input, current_file);
+    procedure_parse_helper proc_helper;
+    xxx_compile_parse(*this, proc_helper, reader); //return value ignored
     return ret;
 }
 
-
-void XxxChart::CompleteParse(bool autoPaginate, bool addHeading,
-    XY pageSize, bool fitWidth, bool collectLinkInfo)
+void XxxChart::CompleteParse(bool, bool, XY, bool, bool)
 {
     Canvas canvas(Canvas::Empty::Query);
     pageBreakData.clear();
