@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -28,9 +28,16 @@
 /** This is the namespace containing xxx chart elements, except parsing.*/
 namespace xxx {
 
-class XxxInstruction {};
+class XxxChart;
 
-typedef UPtrList<XxxInstruction> XxxInstrList;
+class XxxInstruction {
+public:
+    XxxInstruction(XxxChart&);
+    void SetLineEnd(const FileLineColRange&) {}
+};
+
+
+using XxxInstrList = UPtrList<XxxInstruction>;
 
 
 
@@ -40,20 +47,20 @@ class XxxChart : public ChartBase<XxxContext>
 public:
     /** @name Language attributes
     * @{ */
-    std::string GetLanguageDescription() const override { return "Xxx Graph"; } ///<Returns the human-readable (UTF-8) short description 
+    std::string GetLanguageDescription() const override { return "Xxx Graph"; } ///<Returns the human-readable (UTF-8) short description
     std::vector<std::string> GetLanguageExtensions() const override { return{"xxx", "xx2"}; } ///<Returns a set of extensions in order of preference
     std::string GetLanguageEntityName() const override { return "xxx entity"; }
     std::string GetLanguageDefaultText() const override { return "#This is the default xxx text\n"; } ///<Returns the default text of the chart in UTF-8
     bool GetLanguageHasAutoheading() const override { return false; } ///<True if the chart has automatic headings
     bool GetLanguageHasElementControls() const override { return false; } ///<True if the elements of the chart support GUI controls
-    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc, void *param) const override;
+    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc) const override;
     std::map<std::string, std::string> RegisterLibraries() const override;
     /** @} */
 public:
     XxxChart(FileReadProcedure *p, void *param);
     ~XxxChart();
-                 
-    std::unique_ptr<XxxInstrList> PopContext();            ///<Closes a scope 
+
+    XxxInstrList PopContext();            ///<Closes a scope
 
     static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param) { return std::make_unique<XxxChart>(p, param); }
 
@@ -64,7 +71,7 @@ public:
     bool ControlClicked(Element *, EGUIControlType) override;
     bool ApplyForcedDesign(const string & name) override;
     bool GetPedantic() const noexcept override { return true; }
-    void SetPedantic(bool pedantic) noexcept override { }
+    void SetPedantic(bool) noexcept override { }
 
     bool AddChartOption(const Attribute &a);
     std::unique_ptr<XxxInstruction> AddAttribute(const Attribute&);
@@ -84,7 +91,4 @@ public:
 
 using xxx::XxxChart;
 
-/** Yacc-generated function to parse a graph. defined in gv_lang.cpp. */
-void XxxParse(XxxChart &, const char *buff, unsigned len);
-
 #endif
\ No newline at end of file
