@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -24,11 +24,12 @@
 #include <cstring>
 #include "xxxcsh.h"
 #include "xxxstyle.h"
+#include "xxx_parser_csh.h"
 
 using namespace xxx;
 
-XxxCsh::XxxCsh(Csh::FileListProc proc, void *param) :
-    Csh(XxxContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc, param)
+XxxCsh::XxxCsh(Csh::FileListProc proc) :
+    Csh(XxxContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc)
 {
     FillNamesHints();
 }
@@ -52,13 +53,13 @@ void XxxCsh::FillNamesHints()
  * @param [in] input The chart text
  * @param [in] cursor_p The current position of the cursor.
  * @param [in] pedantic The initial value of the pedantic chart option.*/
-void XxxCsh::ParseText(const std::string& input, int cursor_p, bool pedantic)
+void XxxCsh::ParseText(std::string&& input, int cursor_p, bool pedantic)
 {
     //initialize data struct
-    BeforeYaccParse(input, cursor_p);
+    BeforeYaccParse(std::move(input), cursor_p);
     (void)pedantic;
-    //call parsing
-    XxxCshParse(*this, input_text.c_str(), unsigned(input_text.length()+1)); //include terminating zero
+    sv_reader<true> reader(input_text, 0);
+    xxx_csh_parse(*this, reader); //return value ignored
     //Tidy up afterwards
     AfterYaccParse();
 }
