@@ -504,8 +504,22 @@ int main(int argc, char* argv[])
         SDL_ShowWindow(window);
         SDL_EventState(SDL_DROPFILE, SDL_ENABLE);
 
+        timespec last_refresh;
+        clock_gettime(CLOCK_REALTIME, &last_refresh);
+        const long long inter_frame_time_ns = 1e9 / std::get<GUIInit>(ret).max_fps;
+
         // Main loop
         while (true) {
+            //Sleep if needed to limit max FPS
+            timespec now;
+            clock_gettime(CLOCK_REALTIME, &now);
+            const long long diff_ns = (long long)(now.tv_sec - last_refresh.tv_sec) * 1'000'000'000 + (now.tv_nsec - last_refresh.tv_nsec);
+            if (diff_ns < inter_frame_time_ns) {
+                const timespec sleep_ns{0, std::min(inter_frame_time_ns - diff_ns, 999'999'999LL)};
+                nanosleep(&sleep_ns, nullptr);
+            }
+            last_refresh = now;
+
             GUIEvents events;
             // Poll and handle events (inputs, window resize, etc.)
             // You can read the io.WantCaptureMouse, io.WantCaptureKeyboard flags to tell if dear imgui wants to use your inputs.
@@ -674,18 +688,22 @@ std::set<std::string> GetFontNames() {
 
 #include "clipboard.h"
 //On the Mac, it is in clipboard.mm
-std::pair<ClipboardFormat, std::string> GetClipboard() {
+std::optional<std::pair<ClipboardFormat, std::string>> GetClipboard() {
     MacClipboardFormat format;
-    unsigned len;
+    unsigned len = 0;
     char* content = MacGetClipboard(&format, &len);
-    std::pair<ClipboardFormat, std::string> ret;
+    std::optional<std::pair<ClipboardFormat, std::string>> ret;
+    ClipboardFormat fmt;
+    switch (format) {
+    case MacClipboardFormat::Art_GVML: fmt = ClipboardFormat::Art_GVML; break;
+    case MacClipboardFormat::Escaped_Text: fmt = ClipboardFormat::Escaped_Text; break;
+    default:
+    case MacClipboardFormat::Else: fmt = ClipboardFormat::Else; break;
+    case MacClipboardFormat::No_Change: return ret;
+    }
+    ret.emplace(fmt, "");
     if (content && len) {
-        switch (format) {
-        case MacClipboardFormat::Art_GVML: ret.first = ClipboardFormat::Art_GVML; break;
-        case MacClipboardFormat::Escaped_Text: ret.first = ClipboardFormat::Escaped_Text; break;
-        default: return ret;
-        }
-        ret.second.assign(content, len);
+        ret->second.assign(content, len);
         free(content);
     }
     return ret;
@@ -705,7 +723,7 @@ bool CopyToClipboard(ClipboardFormat format, const std::string& content, std::st
 
 #else
 //On Linux we have no clipboard support for now
-std::pair<ClipboardFormat, std::string> GetClipboard() {
+std::optional<std::pair<ClipboardFormat, std::string>> GetClipboard() {
     return {};
 }
 
