@@ -24,7 +24,7 @@ int main(int argc, char** argv) {
     // std::cerr << "succ reading\n";
     const auto w1 = cairo_image_surface_get_width(s1), h1 = cairo_image_surface_get_height(s1),
                w2 = cairo_image_surface_get_width(s2), h2 = cairo_image_surface_get_height(s2);
-    auto f1 = cairo_image_surface_get_format(s1);
+    auto f1 [[maybe_unused]] = cairo_image_surface_get_format(s1);
     _ASSERT(CAIRO_FORMAT_RGB24 == f1 || CAIRO_FORMAT_ARGB32 == f1); //we ignore the 'a' channel anyway
     if (s3) {
         auto cr = cairo_create(s3);
